ifeq ("$(NCT5_BUILD)","true")

LOCAL_PATH := $(call my-dir)
include $(CLEAR_VARS)
ROOTDIR := $(abspath $(TOP))
# To Build MTK bootloader, you need to check this tool in your build env.
HOSTCC := /usr/bin/cc

UBOOT_64BIT:=true

INSTALLED_UBOOT_TARGET := $(PRODUCT_OUT)/mboot.bin

UBOOT_OUT := $(PRODUCT_OUT)/obj/UBOOT_OBJ
UBOOT_OUT_SRC := $(UBOOT_OUT)/u-boot-2019.04
TARGET_UBOOT_CONFIG := $(UBOOT_OUT_SRC)/.config
TARGET_BUILT_UBOOT := $(UBOOT_OUT)/out/mboot.bin

LOCAL_MODULE := mboot.bin
LOCAL_MODULE_CLASS := MBOOT
LOCAL_MODULE_PATH := $(PRODUCT_OUT)
include $(BUILD_SYSTEM)/base_rules.mk
$(LOCAL_BUILT_MODULE): $(TARGET_BUILT_UBOOT)
	$(copy-file-to-target)

ifeq ($(AVB_ENABLE), true)
ifneq ($(LINUX_ONLY), true)
INSTALLED_UBOOT_CUSTOMER_KEY_TARGET := $(PRODUCT_OUT)/mboot_Google_UB_A_AVB_Key.bin
TARGET_BUILT_CUSTOMER_KEY_UBOOT := $(UBOOT_OUT)/out/mboot_Google_UB_A_AVB_Key.bin
endif
endif

ifeq ($(LINUX_ONLY), true)
UBOOT_TOOLCHAIN_ROOT := $(TOOLCHAIN_ROOT)
else
UBOOT_TOOLCHAIN_ROOT := $(ROOTDIR)/prebuilts/mtk_toolchain
endif

MAKE_UBOOT := source $(VAR_EXPORT);source $(patsubst $(ROOTDIR)/%,%,$(ENV_SCRIPT));source $(LOCAL_PATH)/env.sh $(patsubst $(ROOTDIR)/%,%,$(UBOOT_TOOLCHAIN_ROOT)) $(UBOOT_64BIT);make
UBOOT2019_CROSS_COMPILE ?= $(UBOOT_TOOLCHAIN_ROOT)/gcc-linaro-7.2.1-2017.11-x86_64_aarch64-linux-gnu/bin/aarch64-linux-gnu-
UBOOT2019_CROSS_COMPILE_FILENAME = $(notdir $(UBOOT2019_CROSS_COMPILE))
UBOOT_PYTHON3_SITE_PACKAGES ?= $(PRODUCT_OUT)/obj/MBOOT_OBJ/sboot/python3/3.6/site-packages

# About UBOOT_BOOTCMD
# For AOW, UBOOT_BOOTCMD is initialised in make/release_image_uboot2019.mk. Please refer to 5478238
# For AOSP, UBOOT_BOOTCMD is initialised in release_image_uboot2019.mk. Please refer to 5480992
# For MLR, UBOOT_BOOTCMD will be initialised here
ifeq "$(AVB_PURELINUX_ENABLE)" "true"
UBOOT_BOOTCMD := $(shell grep bootcmd= $(SYS_BUILD_DIR)/$(CUSTOMER)/$(MODEL_NAME)/data/ubootenv.txt | sed 's/bootcmd=//g' |\
                   sed 's/DTB_PARTITION_SIZE/$(shell printf "0x%X" $(AVB_PURELINUX_DTB_PARTSIZE))/g' |\
                   sed 's/KL_PARTITION_SIZE/$(shell printf "0x%X" $(AVB_PURELINUX_KL_PARTSIZE))/g' |\
                   sed 's/\r//g')
endif

# we rebuild if the files are modified except Android.mk (handled by AOSP) and .git and the files contain the space (hard to handle in Linux)
UBOOT_LOCAL_SRC_FILES := $(sort $(filter-out %/Android.mk,$(shell find $(ROOTDIR)/$(UBOOT_DIR)/arch/arm/dts -type f -not -path '*\.git*' | grep -v ' ')))

UBOOT_MAIN_DTB := $(MTK_CHIP)-rfb.dtb
UBOOT_OUT_DTS_PATH := $(UBOOT_OUT_SRC)/arch/arm/dts
UBOOT_MAIN_DTB_FILE := $(addprefix $(UBOOT_OUT_DTS_PATH)/,$(UBOOT_MAIN_DTB))

ifeq ($(OFFLINE_UBOOT_DTB_OVERLAY), y)
# Add a step to delete the DTB file before building it.
$(shell rm -f $(UBOOT_MAIN_DTB_FILE))

$(UBOOT_MAIN_DTB_FILE): $(VAR_EXPORT) $(UBOOT_LOCAL_SRC_FILES)
	echo "Build uboot 2019 dtb";
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) $(UBOOT_DEFCONFIG)
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) dtbs clean
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) dtbs

$(BOARD_PREBUILT_DTBOIMAGE): $(UBOOT_MAIN_DTB_FILE)
$(INSTALLED_OFFLINE_UBOOT_DTB_TARGET): $(BOARD_PREBUILT_DTBOIMAGE)
$(TARGET_UBOOT_CONFIG): $(BOARD_PREBUILT_DTBOIMAGE) $(INSTALLED_OFFLINE_UBOOT_DTB_TARGET)
else
$(UBOOT_MAIN_DTB_FILE): $(TARGET_BUILT_UBOOT)
endif


$(TARGET_UBOOT_CONFIG):$(VAR_EXPORT)
	echo "Produce uboot 2019 config";
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) $(UBOOT_DEFCONFIG)
	echo "Modify uboot 2019 config";
	mkdir -p $(UBOOT_OUT)/out
ifeq ($(AB_UPDATE_ENABLE), true)
	@echo "### BUILD UBOOT2019 -- Enable VAB update ###"
	sed -i 's/# CONFIG_ANDROID_AB is not set/CONFIG_ANDROID_AB=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_CMD_AB_SELECT is not set/CONFIG_CMD_AB_SELECT=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_AB_SIDELOAD is not set/CONFIG_AB_SIDELOAD=y/g' $(TARGET_UBOOT_CONFIG)
else
	@echo "### BUILD UBOOT2019 -- Disable VAB update ###"
	sed -i 's/CONFIG_ANDROID_AB=y/# CONFIG_ANDROID_AB is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_CMD_AB_SELECT=y/# CONFIG_CMD_AB_SELECT is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_AB_SIDELOAD=y/# CONFIG_AB_SIDELOAD is not set/g' $(TARGET_UBOOT_CONFIG)
endif
ifneq ($(filter true ,$(AVB_ENABLE) $(AVB_PURELINUX_ENABLE)),)
	@echo "### BUILD UBOOT2019 -- Enable AVB ###"
	sed -i 's/# CONFIG_AVB_VERIFY is not set/CONFIG_AVB_VERIFY=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_CMD_AVB is not set/CONFIG_CMD_AVB=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_LIBAVB is not set/CONFIG_LIBAVB=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_LEGACY_AVB_PARTITION is not set/CONFIG_LEGACY_AVB_PARTITION=y/g' $(TARGET_UBOOT_CONFIG)
ifeq "$(AVB_PURELINUX_ENABLE)" "true"
	sed -i 's/# CONFIG_ANDROID_AVB_PURELINUX_ENABLE is not set/CONFIG_ANDROID_AVB_PURELINUX_ENABLE=y/g' $(TARGET_UBOOT_CONFIG)
endif
ifeq "$(AVB_ANTI_ROLLBACK)" "false"
	sed -i 's/CONFIG_OPTEE_TA_AVB=y/# CONFIG_OPTEE_TA_AVB is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_TEE=y/# CONFIG_TEE is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_OPTEE=y/# CONFIG_OPTEE is not set/g' $(TARGET_UBOOT_CONFIG)
else ifeq "$(TEE_SOCKET_BOARD_TEST)" "1"
	sed -i 's/CONFIG_OPTEE_TA_AVB=y/# CONFIG_OPTEE_TA_AVB is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_TEE=y/# CONFIG_TEE is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_OPTEE=y/# CONFIG_OPTEE is not set/g' $(TARGET_UBOOT_CONFIG)
else
	sed -i 's/# CONFIG_OPTEE_TA_AVB is not set/CONFIG_OPTEE_TA_AVB=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_TEE is not set/CONFIG_TEE=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_OPTEE is not set/CONFIG_OPTEE=y/g' $(TARGET_UBOOT_CONFIG)
ifeq ($(EFUSE_ROLLBACK_INDEX_CONTROL), 1)
	@echo "[uboot2109] CONFIG_ROLLBACK_INDEX_IN_EFUSE=y by EFUSE_ROLLBACK_INDEX_CONTROL=1"
	sed -i 's/# CONFIG_ROLLBACK_INDEX_IN_EFUSE is not set/CONFIG_ROLLBACK_INDEX_IN_EFUSE=y/g' $(TARGET_UBOOT_CONFIG)
endif
endif
else
	@echo "### BUILD UBOOT2019 -- Disable AVB ###"
	sed -i 's/CONFIG_AVB_VERIFY=y/# CONFIG_AVB_VERIFY is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_CMD_AVB=y/# CONFIG_CMD_AVB is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_LIBAVB=y/# CONFIG_LIBAVB is not set/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/CONFIG_LEGACY_AVB_PARTITION=y/# CONFIG_LEGACY_AVB_PARTITION is not set/g' $(TARGET_UBOOT_CONFIG)
endif
ifeq ("$(FASTBOOT_RESCUE)","true")
	@echo "### BUILD UBOOT2019 -- FASTBOOT RESCUE MODE= true ###"
	sed -i 's/# CONFIG_FASTBOOT_SYSTEM_RESCUE is not set/CONFIG_FASTBOOT_SYSTEM_RESCUE=y/g' $(TARGET_UBOOT_CONFIG)
endif
ifeq ("$(AEE_SUPPORT)","true")
	@echo "### BUILD UBOOT2019 -- Enable AEE ###"
	sed -i 's/# CONFIG_MTK_AEE_SUPPORT is not set/CONFIG_MTK_AEE_SUPPORT=y/g' $(TARGET_UBOOT_CONFIG)
	sed -i 's/# CONFIG_OF_SYSTEM_SETUP is not set/CONFIG_OF_SYSTEM_SETUP=y/g' $(TARGET_UBOOT_CONFIG)
endif
	@echo "build uboot config";
	$(ACP) $(TARGET_UBOOT_CONFIG) $(ROOTDIR)/$(UBOOT_DIR)/configs/configs_defconfig
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) configs_defconfig


$(TARGET_BUILT_UBOOT): $(VAR_EXPORT) $(TARGET_UBOOT_CONFIG)
	@echo "Building uboot 2019";
	@echo "[uboot/Android.mk]UBOOT_BOOTCMD=$(UBOOT_BOOTCMD)"
	@echo "[uboot/Android.mk]UBOOT_REVOCERYCMD=$(UBOOT_REVOCERYCMD)"
ifeq ($(ANDROID_VTS_LTPKTF),yes)
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) ENABLE_VTS_TEST=y HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) UBOOT_BOOTCMD="$(UBOOT_BOOTCMD)" UBOOT_REVOCERYCMD="$(UBOOT_REVOCERYCMD)"
else
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) UBOOT_BOOTCMD="$(UBOOT_BOOTCMD)" UBOOT_REVOCERYCMD="$(UBOOT_REVOCERYCMD)"
endif
	$(ACP) $(UBOOT_OUT_SRC)/mboot.bin $(TARGET_BUILT_UBOOT)


ifeq ($(AVB_ENABLE), true)
ifneq ($(LINUX_ONLY), true)
$(TARGET_BUILT_CUSTOMER_KEY_UBOOT): $(TARGET_BUILT_UBOOT)
	@echo "Rebuilding mboot with Google_UB_A_AVB_Key from $(UBOOT_OUT_SRC)"
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) COMMON_CFG=$(COMMON_CFG) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) AVB_CUSTOMER_KEY_SELECT=GOOGLE_AVB_PUBLIC_KEY UBOOT_BOOTCMD="$(UBOOT_BOOTCMD)" UBOOT_REVOCERYCMD="$(UBOOT_REVOCERYCMD)"
	mv $(UBOOT_OUT_SRC)/mboot.bin $(TARGET_BUILT_CUSTOMER_KEY_UBOOT)
endif
endif

ifeq ($(AVB_ENABLE), true)
ifneq ($(LINUX_ONLY), true)
$(INSTALLED_UBOOT_CUSTOMER_KEY_TARGET): $(TARGET_BUILT_CUSTOMER_KEY_UBOOT)
	@echo Copies the built mboot with Google key to $(PRODUCT_OUT)
	$(ACP) $(TARGET_BUILT_CUSTOMER_KEY_UBOOT) $(INSTALLED_UBOOT_CUSTOMER_KEY_TARGET)
endif
endif

droidcore: $(INSTALLED_UBOOT_CUSTOMER_KEY_TARGET) $(TARGET_BUILT_CUSTOMER_KEY_UBOOT) $(INSTALLED_UBOOT_TARGET) $(TARGET_BUILT_UBOOT)

ubootclean: $(VAR_EXPORT) $(ACP)
	@echo "Executing uboot 2019 clean";
	$(MAKE_UBOOT) -C $(ROOTDIR)/$(UBOOT_DIR) O=$(ROOTDIR)/$(UBOOT_OUT_SRC) HOSTCC=$(HOSTCC) CROSS_COMPILE=$(UBOOT2019_CROSS_COMPILE_FILENAME) PYTHON3_SITE_PACKAGES=$(UBOOT_PYTHON3_SITE_PACKAGES) clean
	@rm -rf $(TARGET_BUILT_UBOOT)
	@rm -rf $(TARGET_BUILT_CUSTOMER_KEY_UBOOT)
	@rm -rf $(PRODUCT_OUT)/mboot.bin
	@rm -rf $(PRODUCT_OUT)/mboot_Google_UB_A_AVB_Key.bin

# This is for developer to only build uboot.
# when make any change above, please make chanege to this section as well if needed
uboot: $(INSTALLED_UBOOT_CUSTOMER_KEY_TARGET) $(INSTALLED_UBOOT_TARGET) $(TARGET_BUILT_UBOOT) $(VAR_EXPORT) $(ACP)

.PHONY: uboot ubootclean

endif	#end of ifeq ("$(NCT5_BUILD)","true")
