/*
 * Copyright 2020,2021,2022,2023 Sony Corporation
 */

#ifndef _SONY_AUTODET_H_
#define _SONY_AUTODET_H_

#define BOOTDATA_PART_NAME				"bootdata"
#define SONY_DEFAULT_DTB_FILE			"Sony_default"
#define SONY_DEFAULT_DTB_KERNEL_FILE		"Sony_default_kernel"
#define SONY_DEFAULT_DTB_UBOOT_FILE		"Sony_default_uboot"
#define SONY_DEFAULT_DTB_EMPTY_FILE		"Sony_default_empty"
#define SONY_MULTI_KEY_PREFIX			"Multikey_"
#define SONY_FOR_PMU_SUFFIX			"_For_PMU"
#define SONY_CM4_DTBO_PATH			"dtbo-path"
#define SONY_CM4_COMPATIBLE			"mediatek,merak-pmu-cm4-rproc"
#define SONY_CM4_DEFAULT_DTB_FILE		"pmu_Sony_pmu_default"
#define SONY_DEVICE_NAME			"device name"

#define MAX_PCD_CATEGORY				(128)
#define MAX_PCD_I2C_CATEGORY			(64)
#define MAX_PCD_ADC_CATEGORY			(64)
#define MAX_PCD_GPIO_CATEGORY			(64)
#define MAX_ADC_CATEGORY				(64)
#define MAX_UBOOT_ADC_CATEGORY				(64)
#define MAX_ADC_I2C_CATEGORY			(64)
#define MAX_GPIO_CATEGORY				(64)
#define MAX_GPIO_I2C_CATEGORY			(64)
#define MAX_I2C_CATEGORY				(64)
#define MAX_PANEL_CATEGORY				(2)
#define MAX_PANEL_ITEMS				(512)
#define MAX_CHASSIS_CATEGORY				(2)
#define MAX_CHASSIS_ITEMS				(256)
#define MAX_AUTODET_ITEMS				(32)
#define MAX_DT_MAP_ITEMS				(1024)
#define MAX_CATEGORY_LENGTH				(64)
#define MAX_KEY_LENGTH				(64)
#define MAX_VALUE_LENGTH				(64)
#define MAX_DEVICE_NAME_LENGTH			(MAX_VALUE_LENGTH)
#define MAX_PANELID_LENGTH				(16)
#define MAX_PANEL_RETRY_COUNT			(10)
#define MAX_RULE_NUM				(16)


#define AUTODET_NO_RULES				(-1)
#define AUTODET_NO_DATA				(-2)

// autodet type
enum autodet_id {
	AUTODET_PCD = 0,
	AUTODET_PCD_I2C,
	AUTODET_PCD_ADC,
	AUTODET_PCD_GPIO,
	AUTODET_ADC,
	AUTODET_UBOOT_ADC,
	AUTODET_ADC_I2C,
	AUTODET_GPIO,
	AUTODET_GPIO_I2C,
	AUTODET_I2C,
	AUTODET_PANEL,
	AUTODET_CHASSIS,
	AUTODET_DT_MAP,
	AUTODET_UNKNOWN
};

// PCD
struct sony_pcd_item {
	char key[MAX_RULE_NUM][MAX_KEY_LENGTH];
	char value[MAX_RULE_NUM][MAX_VALUE_LENGTH];
};

struct sony_pcd_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	bool is_multi_key;
	uint32_t count;
	struct sony_pcd_item *items;
};

// CHASSIS
struct sony_chassis_item {
	char key[MAX_RULE_NUM][MAX_KEY_LENGTH];
	char value[MAX_RULE_NUM][MAX_VALUE_LENGTH];
};

struct sony_chassis_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_chassis_item *items;
};

// PCD x I2C
struct sony_pcd_i2c_item {
	char first_key[MAX_KEY_LENGTH];
	char first_value[MAX_VALUE_LENGTH];
	uint32_t bus;
	uint32_t slave;
	uint32_t addr;
	uint32_t result;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_pcd_i2c_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_pcd_i2c_item *items;
};

// PCD x ADC
struct sony_pcd_adc_item {
	char first_key[MAX_KEY_LENGTH];
	char first_value[MAX_VALUE_LENGTH];
	uint32_t channel;
	uint32_t min;
	uint32_t max;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_pcd_adc_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_pcd_adc_item *items;
};

// PCD x GPIO
struct sony_pcd_gpio_item {
	char first_key[MAX_KEY_LENGTH];
	char first_value[MAX_VALUE_LENGTH];
	uint32_t gpio;
	uint32_t result;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_pcd_gpio_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_pcd_gpio_item *items;
};

// ADC
struct sony_adc_item {
	uint32_t channel;
	uint32_t min;
	uint32_t max;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_adc_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_adc_item *items;
};

struct sony_uboot_adc_item {
	uint32_t channel;
	uint32_t min;
	uint32_t max;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_uboot_adc_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_uboot_adc_item *items;
};

// ADC x I2C
struct sony_adc_i2c_item {
	uint32_t channel;
	uint32_t min;
	uint32_t max;
	uint32_t bus;
	uint32_t slave;
	uint32_t addr;
	uint32_t result;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_adc_i2c_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_adc_i2c_item *items;
};

// GPIO
struct sony_gpio_item {
	uint32_t gpio;
	uint32_t result;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_gpio_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_gpio_item *items;
};

// GPIO x I2C
struct sony_gpio_i2c_item {
	uint32_t gpio;
	uint32_t result;
	uint32_t bus;
	uint32_t slave;
	uint32_t addr;
	uint32_t value;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_gpio_i2c_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_gpio_i2c_item *items;
};

// I2C
struct sony_i2c_item {
	uint32_t bus;
	uint32_t slave;
	uint32_t addr;
	uint32_t result;
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_i2c_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_i2c_item *items;
};

// PANEL
struct sony_panel_item {
	uint32_t bus;
	uint32_t slave;
	uint32_t addr;
	uint8_t panel_id[MAX_PANELID_LENGTH];
	char device_name[MAX_DEVICE_NAME_LENGTH];
};

struct sony_panel_rule_t {
	char category[MAX_CATEGORY_LENGTH];
	uint32_t count;
	struct sony_panel_item *items;
};


// DT MAP
struct sony_dt_map_t {
	char device_name[MAX_DEVICE_NAME_LENGTH];
	uint32_t dtb_entry;
};

int sony_dtbo_selection_and_overlay(void);
int select_sony_overlay_for_uboot(void);
int select_sony_pcd_overlay(const char *category);
int select_sony_chassis_overlay(const char *category);
int select_sony_pcd_i2c_overlay(const char *first_key, int bus, int slave, int addr, int len);
int select_sony_pcd_adc_overlay(const char *first_key, int adc_ch);
int select_sony_pcd_gpio_overlay(const char *first_key, int gpio);
int select_sony_adc_overlay(int adc_ch);
int select_sony_adc_i2c_overlay(int adc_ch, int bus, int slave, int addr, int len);
int select_sony_gpio_overlay(int gpio);
int select_sony_gpio_i2c_overlay(int gpio, int bus, int slave, int addr, int len);
int select_sony_i2c_overlay(int bus, int slave, int addr, int len);
int select_sony_panel_overlay(int bus, int slave, int addr);
int select_sony_pcd_overlay_for_pmu(void);
#endif // _SONY_AUTODET_H_
