/*
 * Copyright 2021 Sony Corporation
 */

#ifndef __VBOFRAMEINFO_H__
#define __VBOFRAMEINFO_H__

#include <linux/types.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define FRAME_HEADER_PARAM_SIZE_MAX 128 // Maximum value of parameters stored in frame header
#define FRAME_HEADER_NUM_MAX 10 // Maximum number of frames
#define IKEY_LENGTH 4 // Number of digits of IKEY
#define STR_LENGTH 8 // Number of characters in the string to get
#define TID_NUM_SIZE_MAX 16 // Number of A that can be stored

#define DATA_PART_NAME "featuredata"

enum E_VBO_FRAME_STATUS {
	E_VBO_FRAME_OK,
	E_VBO_FRAME_CSV_FILEOPEN_NG,
	E_VBO_FRAME_KEY_FORMAT_NG,
	E_VBO_FRAME_KEY_DATA_NG,
	E_VBO_FRAME_KEY_LINE_NG,
	E_VBO_FRAME_IKEY_FORMAT_NG,
	E_VBO_FRAME_IKEY_DATA_NG,
	E_VBO_FRAME_IKEY_LINE_NG,
	E_VBO_FRAME_BOOT_STATE_NG,
	E_VBO_FRAME_FRAME_HEADER_NG,
	E_VBO_FRAME_NO_PACKET_LINE,
	E_VBO_FRAME_ADC_NG,
	E_VBO_FRAME_MALLOC_NG,
	E_VBO_FRAME_FILE_SIZE_NG,
	E_VBO_FRAME_POINTER_NULL,
	E_VBO_FRAME_PRODUCT_CONFIG_ERROR,
	E_VBO_FRAME_BCM_FILEOPEN_NG
};

struct ST_PACKETS_HEADER {
	char tid_frame[TID_NUM_SIZE_MAX][STR_LENGTH];
};

struct ST_FRAME_HEADER {
	uint8_t param[FRAME_HEADER_NUM_MAX][FRAME_HEADER_PARAM_SIZE_MAX];
	uint8_t num;
	uint8_t ikey[IKEY_LENGTH];
	struct ST_PACKETS_HEADER packets_frame[FRAME_HEADER_NUM_MAX];
};

extern enum E_VBO_FRAME_STATUS make_vb1_frame_header(struct ST_FRAME_HEADER *frame_header);
extern uint16_t calc_vb1_header_crc(uint8_t *buf, uint32_t len);
extern int8_t load_frame_info_num(void);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif // __VBOFRAMEINFO_H__
