/*
 * Copyright 2022 Sony Corporation
 */

#include <common.h>
#include <command.h>
#include <debug_impl.h>
#include <mmc.h>
#include "eMMC.h"

bool is_emmc_initialized(void)
{
	bool ret = true;
	U8 gau8_eMMC_SectorBuf[EMMC_SECTOR_512BYTE];

	UBOOT_TRACE("In\n");

	memset(&gau8_eMMC_SectorBuf, 0, EMMC_SECTOR_512BYTE);

	emmc_cmd8(gau8_eMMC_SectorBuf);

	UBOOT_INFO("ecsd[162]:0x%x\r\n", gau8_eMMC_SectorBuf[162]);
	UBOOT_INFO("ecsd[177]:0x%x\r\n", gau8_eMMC_SectorBuf[177]);
	UBOOT_INFO("ecsd[179]:0x%x\r\n", gau8_eMMC_SectorBuf[179]);

	if ((gau8_eMMC_SectorBuf[162] != 1) ||
		(gau8_eMMC_SectorBuf[177] != 2) ||
		((gau8_eMMC_SectorBuf[179]>>3) != 1 &&
		(gau8_eMMC_SectorBuf[179]>>3) != 2)) {
		ret = false;
	}

	UBOOT_TRACE("OK\n");

	return ret;
}

int do_emmc_register_init(void)
{
	int ret = 0;

	UBOOT_TRACE("In\n");

	if (!is_emmc_initialized()) {
		char cmdbuf[32];
		const char *cmdList[4] = {
			"mmc rescan",
			"mmc rst-function 0 1",
			"mmc bootbus 0 2 0 0",
			"mmc partconf 0 0 1 0"
		};

		UBOOT_INFO("needs emmc_register_init\n");

		for (int i = 0; i < 4; i++) {
			snprintf(cmdbuf, sizeof(cmdbuf), cmdList[i]);
			UBOOT_INFO("Execute: %s\n", cmdbuf);
			if (run_command(cmdbuf, 0)) {
				UBOOT_ERROR("Failed to run command %s for emmc ecsd init\n", cmdbuf);
				ret = -1;
				break;
			}
		}

		if (ret != -1) {
			if (!is_emmc_initialized()) {
				UBOOT_ERROR("EMMC init register error !!\n");
				ret = -1;
			} else {
				UBOOT_INFO("EMMC init register OK !!\n");
			}
		}
	}

	UBOOT_TRACE("OK\n");

	return ret;
}
