/*
 * Copyright 2020,2021 Sony Corporation
 */

#include <stdio.h>
#include <command.h>
#include <debug_impl.h>
#include <sony_panel.h>
#include <sony_panel_oled.h>

int do_paneldet_callback(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;

	ret = paneldet_callback();
	if (ret != 0) {
		printf("failed to set sony panel to bootargs.\n");
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}

int do_paneldet_power_set(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;
	bool enable;

	if (argv[1][0] == '0')
		enable = false;
	else
		enable = true;

	ret = paneldet_setPanelPower(enable);
	if (ret != 0) {
		printf("failed to set sony panel power.\n");
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}

int do_paneldet_backlight_flush(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;

	ret = paneldet_setBacklightPowerFlush();
	if (ret != 0) {
		printf("failed to reflect backlight power setting to HW.\n");
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}

int do_disable_panel_test_pattern(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	paneldet_disablePanelTestPattern();
	return CMD_RET_SUCCESS;
}

int do_oled_panel_control(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;

	ret = panel_oledPanelControl();
	if (ret != 0) {
		printf("failed to oled panel control.\n");
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}

int do_oled_logo_brightness_control(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;

	ret = panel_oledLogoBrightnessControl();
	if (ret != 0) {
		printf("failed to logo brightness control.\n");
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	paneldet_callback, CONFIG_SYS_MAXARGS, 0, do_paneldet_callback,
	"Read panel ID and call autodet functions for the panel with panel and backlight power enable.",
	NULL
);

U_BOOT_CMD(
	panel_power_set, CONFIG_SYS_MAXARGS, 0, do_paneldet_power_set,
	"Set panel or 12V power IOs with delay.",
	"0: Power OFF, Otherwise: Power ON"
);

U_BOOT_CMD(
	backlight_power_flush, CONFIG_SYS_MAXARGS, 0, do_paneldet_backlight_flush,
	"Reflect backlight or 24V power IOs actually with delay.",
	NULL
);

U_BOOT_CMD(
	disable_panel_test_pattern, CONFIG_SYS_MAXARGS, 0, do_disable_panel_test_pattern,
	"Depends on the panelID, send i2c to disable test pattern.",
	NULL
);

U_BOOT_CMD(
	oled_panel_control, CONFIG_SYS_MAXARGS, 0, do_oled_panel_control,
	"Control OLED panel specific settings",
	NULL
);

U_BOOT_CMD(
	oled_logo_brightness_control, CONFIG_SYS_MAXARGS, 0, do_oled_logo_brightness_control,
	"Lower brightness during boot logo to prevent OLED panel burn in",
	NULL
);
