/*
 * Copyright 2022 Sony Corporation
 */

#include <stdio.h>
#include <string.h>
#include <command.h>
#include <debug_impl.h>
#include <common.h>
#include <part.h>
#include <partition/include/partition.h>

int do_sony_part_init(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int ret = -1;
	unsigned int partition_type = PARTITION_LAYOUT_DEFAULT;
	int dev = 0;
	char *ep;
	struct blk_desc *blk_dev_desc = NULL;

	if (argc != 4)
		return CMD_RET_USAGE;

	if (strcmp(argv[1], "default") == 0)
		partition_type = PARTITION_LAYOUT_DEFAULT;
	else if (strcmp(argv[1], "full") == 0)
		partition_type = PARTITION_LAYOUT_FULL;
	else if (strcmp(argv[1], "full_ext_data") == 0)
		partition_type = PARTITION_LAYOUT_FULL_WITH_EXTENDED_USERDATA;
	else
		UBOOT_ERROR("No such partition type: %s.\n", argv[1]);

	dev = (int)strtoul(argv[3], &ep, 10);
	if (!ep || ep[0] != '\0') {
		printf("'%s' is not a number\n", argv[3]);
		return CMD_RET_USAGE;
	}
	blk_dev_desc = blk_get_dev(argv[2], dev);
	if (!blk_dev_desc) {
		printf("%s: %s dev %d NOT available\n",
				__func__, argv[2], dev);
		return CMD_RET_FAILURE;
	}
	ret = sony_partition_init(partition_type, blk_dev_desc);
	if (ret != 0) {
		UBOOT_ERROR("failed to initialize sony partitions.\n");
		return CMD_RET_FAILURE;
	}
	return CMD_RET_SUCCESS;
}

U_BOOT_CMD(
	sony_part_init, CONFIG_SYS_MAXARGS, 1, do_sony_part_init,
	"init sony partitions {default|full|full_ext_data}\n",
	NULL
);
