// SPDX-License-Identifier: (GPL-2.0-only OR BSD-3-Clause)
/*
 * Copyright (c) 2023 MediaTek Inc.
 */

#include <string.h>
#include <common.h>
#include <partition/include/partition.h>

#ifndef USE_CUSTOM_PARTITION_SIZE
#define BOARD_MBOOTIMAGE_PARTITION_SIZE			"10M"
#define BOARD_UENVIMAGE_PARTITION_SIZE			"8M"
#define BOARD_CERTIFICATE_PARTITION_SIZE		"8M"
#define BOARD_PERSIST_PARTITION_SIZE			"32M"
#define BOARD_SONY_CONFIG_PARTITION_SIZE		"32M"
#define BOARD_CONFIG_PARTITION_SIZE			"32M"
#define BOARD_BOOTDATA_PARTITION_SIZE			"64M"
#define BOARD_VBMETAIMAGE_PARTITION_SIZE		"1M"
#define BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE		"1M"
#define BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE		"1M"
#define BOARD_TVCONFIGIMAGE_PARTITION_SIZE		"200M"
#define BOARD_MISCIMAGE_PARTITION_SIZE			"1M"
#define BOARD_BOOTIMAGE_PARTITION_SIZE			"64M"
#define BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE		"64M"
#define BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE		"8M"
#define BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE		"256M"
#define BOARD_METADATAIMAGE_PARTITION_SIZE		"16M"
#define BOARD_DTBOIMG_PARTITION_SIZE			"8M"
#define BOARD_SUPER_PARTITION_SIZE			"8704M"
#define BOARD_SUPPLICANT_PARTITION_SIZE			"500M"
#define BOARD_FEATUREDATA_PARTITION_SIZE		"512M"
#define BOARD_CACHEIMAGE_PARTITION_SIZE			"100M"
#define BOARD_OPTEEIMAGE_PARTITION_SIZE			"24M"
#define BOARD_ARMFWIMAGE_PARTITION_SIZE			"1M"
#define BOARD_USERDATAIMAGE_PARTITION_SIZE		"16268M"
#define BOARD_DEMO_PARTITION_SIZE			"1024M"
#define BOARD_32GB_USERDATAIMAGE_PARTITION_SIZE		"16268M"
#define BOARD_64GB_USERDATAIMAGE_PARTITION_SIZE		"46212M"
#define BOARD_32GB__EXT_USERDATAIMAGE_PARTITION_SIZE	"13708M"
#define BOARD_64GB__EXT_USERDATAIMAGE_PARTITION_SIZE	"43652M"
#endif


char part_mboot_a[]          =  "mboot_a";
char part_mboot_b[]          =  "mboot_b";
char part_uenv[]             =  "uenv";

char default_par_info[DEFAULT_PAR_STR_LEN];
char usb_default_par_info[] =
	"name=sboot,size=1M;"
	"name=mboot,size=10M;"
	"name=uenv,size=16M";

static char *sony_partitions[NUM_DISK_SIZE][NUM_PARTITION_LAYOUT] = {{
	"name=mboot_a,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=mboot_b,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=uenv_a,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=uenv_b,size=" BOARD_UENVIMAGE_PARTITION_SIZE
	,
	"name=mboot_a,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=mboot_b,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=uenv_a,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=uenv_b,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=certificate,size=" BOARD_CERTIFICATE_PARTITION_SIZE ";"
	"name=persist,size=" BOARD_PERSIST_PARTITION_SIZE ";"
	"name=sony_config,size=" BOARD_SONY_CONFIG_PARTITION_SIZE ";"
	"name=config_a,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=config_b,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=bootdata_a,size="	BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=bootdata_b,size=" BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=vbmeta_a,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_b,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_a,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_b,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_a,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_b,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_a,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_b,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=misc,size=" BOARD_MISCIMAGE_PARTITION_SIZE ";"
	"name=boot_a,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=boot_b,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=init_boot_a,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=init_boot_b,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=vendor_bootconfig_a,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_bootconfig_b,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_boot_a,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=vendor_boot_b,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=metadata,size=" BOARD_METADATAIMAGE_PARTITION_SIZE ";"
	"name=dtbo_a,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=dtbo_b,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=super,size=" BOARD_SUPER_PARTITION_SIZE ";"
	"name=supplicant_a,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=supplicant_b,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=featuredata_a,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=featuredata_b,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=cache,size=" BOARD_CACHEIMAGE_PARTITION_SIZE ";"
	"name=optee_a,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=optee_b,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=armfw_a,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=armfw_b,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=userdata,size=" BOARD_32GB_USERDATAIMAGE_PARTITION_SIZE ";"
	"name=demo,size=" BOARD_DEMO_PARTITION_SIZE
	,
	"name=mboot_a,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=mboot_b,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=uenv_a,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=uenv_b,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=certificate,size=" BOARD_CERTIFICATE_PARTITION_SIZE ";"
	"name=persist,size=" BOARD_PERSIST_PARTITION_SIZE ";"
	"name=sony_config,size=" BOARD_SONY_CONFIG_PARTITION_SIZE ";"
	"name=config_a,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=config_b,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=bootdata_a,size="	BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=bootdata_b,size=" BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=vbmeta_a,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_b,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_a,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_b,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_a,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_b,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_a,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_b,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=misc,size=" BOARD_MISCIMAGE_PARTITION_SIZE ";"
	"name=boot_a,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=boot_b,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=init_boot_a,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=init_boot_b,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=vendor_bootconfig_a,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_bootconfig_b,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_boot_a,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=vendor_boot_b,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=metadata,size=" BOARD_METADATAIMAGE_PARTITION_SIZE ";"
	"name=dtbo_a,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=dtbo_b,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=super,size=" BOARD_SUPER_PARTITION_SIZE ";"
	"name=supplicant_a,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=supplicant_b,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=featuredata_a,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=featuredata_b,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=cache,size=" BOARD_CACHEIMAGE_PARTITION_SIZE ";"
	"name=optee_a,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=optee_b,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=armfw_a,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=armfw_b,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=userdata,size=" BOARD_32GB_EXT_USERDATAIMAGE_PARTITION_SIZE
	},
	{
	"name=mboot_a,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=mboot_b,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=uenv_a,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=uenv_b,size=" BOARD_UENVIMAGE_PARTITION_SIZE
	,
	"name=mboot_a,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=mboot_b,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=uenv_a,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=uenv_b,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=certificate,size=" BOARD_CERTIFICATE_PARTITION_SIZE ";"
	"name=persist,size=" BOARD_PERSIST_PARTITION_SIZE ";"
	"name=sony_config,size=" BOARD_SONY_CONFIG_PARTITION_SIZE ";"
	"name=config_a,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=config_b,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=bootdata_a,size="	BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=bootdata_b,size=" BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=vbmeta_a,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_b,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_a,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_b,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_a,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_b,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_a,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_b,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=misc,size=" BOARD_MISCIMAGE_PARTITION_SIZE ";"
	"name=boot_a,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=boot_b,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=init_boot_a,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=init_boot_b,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=vendor_bootconfig_a,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_bootconfig_b,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_boot_a,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=vendor_boot_b,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=metadata,size=" BOARD_METADATAIMAGE_PARTITION_SIZE ";"
	"name=dtbo_a,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=dtbo_b,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=super,size=" BOARD_SUPER_PARTITION_SIZE ";"
	"name=supplicant_a,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=supplicant_b,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=featuredata_a,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=featuredata_b,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=cache,size=" BOARD_CACHEIMAGE_PARTITION_SIZE ";"
	"name=optee_a,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=optee_b,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=armfw_a,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=armfw_b,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=userdata,size=" BOARD_64GB_USERDATAIMAGE_PARTITION_SIZE ";"
	"name=demo,size=" BOARD_DEMO_PARTITION_SIZE
	,
	"name=mboot_a,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=mboot_b,size=" BOARD_MBOOTIMAGE_PARTITION_SIZE ";"
	"name=uenv_a,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=uenv_b,size=" BOARD_UENVIMAGE_PARTITION_SIZE ";"
	"name=certificate,size=" BOARD_CERTIFICATE_PARTITION_SIZE ";"
	"name=persist,size=" BOARD_PERSIST_PARTITION_SIZE ";"
	"name=sony_config,size=" BOARD_SONY_CONFIG_PARTITION_SIZE ";"
	"name=config_a,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=config_b,size=" BOARD_CONFIG_PARTITION_SIZE ";"
	"name=bootdata_a,size="	BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=bootdata_b,size=" BOARD_BOOTDATA_PARTITION_SIZE ";"
	"name=vbmeta_a,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_b,size=" BOARD_VBMETAIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_a,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_system_b,size=" BOARD_VBMETA_SYSTEMIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_a,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=vbmeta_vendor_b,size=" BOARD_VBMETA_VENDORIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_a,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=tvconfig_b,size=" BOARD_TVCONFIGIMAGE_PARTITION_SIZE ";"
	"name=misc,size=" BOARD_MISCIMAGE_PARTITION_SIZE ";"
	"name=boot_a,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=boot_b,size=" BOARD_BOOTIMAGE_PARTITION_SIZE ";"
	"name=init_boot_a,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=init_boot_b,size=" BOARD_INIT_BOOT_IMAGE_PARTITION_SIZE ";"
	"name=vendor_bootconfig_a,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_bootconfig_b,size=" BOARD_VENDOR_BOOTCONFIG_PARTITION_SIZE ";"
	"name=vendor_boot_a,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=vendor_boot_b,size=" BOARD_VENDOR_BOOTIMAGE_PARTITION_SIZE ";"
	"name=metadata,size=" BOARD_METADATAIMAGE_PARTITION_SIZE ";"
	"name=dtbo_a,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=dtbo_b,size=" BOARD_DTBOIMG_PARTITION_SIZE ";"
	"name=super,size=" BOARD_SUPER_PARTITION_SIZE ";"
	"name=supplicant_a,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=supplicant_b,size=" BOARD_SUPPLICANT_PARTITION_SIZE ";"
	"name=featuredata_a,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=featuredata_b,size=" BOARD_FEATUREDATA_PARTITION_SIZE ";"
	"name=cache,size=" BOARD_CACHEIMAGE_PARTITION_SIZE ";"
	"name=optee_a,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=optee_b,size=" BOARD_OPTEEIMAGE_PARTITION_SIZE ";"
	"name=armfw_a,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=armfw_b,size=" BOARD_ARMFWIMAGE_PARTITION_SIZE ";"
	"name=userdata,size=" BOARD_64GB_EXT_USERDATAIMAGE_PARTITION_SIZE
	}};


int sony_partition_init(unsigned int type, struct blk_desc *blk_dev_desc)
{
	int disk_size = DISK_SIZE_32GB;

	if (blk_dev_desc->lba <= 67108864) {  // 32GB (34359738368/512=67108864)
		disk_size = DISK_SIZE_32GB;
	} else { // 64GB
		disk_size = DISK_SIZE_64GB;
	}
	if (type < NUM_PARTITION_LAYOUT) {
		if (snprintf(default_par_info, DEFAULT_PAR_STR_LEN, "%s", sony_partitions[disk_size][type]) >= DEFAULT_PAR_STR_LEN) {
			printf("sony_part buffer size is not enough: %d\n", DEFAULT_PAR_STR_LEN);
			return -1;
		}
	}

	return 0;
}

