/*
 * Copyright 2022 Sony Corporation
 */

#include <common.h>
#include <mtk_cmdtable.h>
#include <time.h>
#include <utility.h>

#include <dts_parser.h>
#include <dm.h>
#include <mailbox.h>


#define MBOX_DEV_NAME "mailbox_to_cm4"
#define MBOX_CHAN_WAIT_PMU (0x10)
#define MBOX_DATA_PMU_STATUS_READY (0x11)

int do_wait_pmu_ready(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	struct mbox_chan chan;
	uint8_t data[64] = {0};
	int ret;
	struct ofnode_phandle_args args;

	args.args[0] = MBOX_CHAN_WAIT_PMU;
	args.args_count = 1;
	ret = mbox_get_by_dev_name(MBOX_DEV_NAME, &args, &chan);
	if (ret) {
		printf("%s %d mbox_get_by_dev_name fail ret=%d\n", __func__, __LINE__, ret);
		return 0;
	}

	printf("%s %d [%lu]\n", __func__, __LINE__, get_timer(0));
	ret = mbox_recv(&chan, data, 10*1000*1000);
	if (ret) {
		printf("%s %d mbox_recv fail ret=%d\n", __func__, __LINE__, ret);
		return 0;
	}

	if (data[0] == MBOX_DATA_PMU_STATUS_READY)
		printf("PMU is ready\n");
	else
		printf("%s %d receive incorrect data from PMU data=%d\n", __func__, __LINE__, data[0]);

	printf("%s %d [%lu]\n", __func__, __LINE__, get_timer(0));

	return 0;
}

U_BOOT_CMD(wait_pmu_ready, 1, 1, do_wait_pmu_ready,
		"wait PMU ready\n",
		"wait PMU ready\n"
);
