// SPDX-License-Identifier: (GPL-2.0 OR BSD-3-Clause)
/*
 * MediaTek Panel driver
 *
 * Copyright (c) 2020 MediaTek Inc.
 */
#define DEBUG //for open this file debug("xxx") log
#include <common.h>
#include <utility.h>
#include "mtk_pnl_cust.h"
#include <i2c.h>
#include <asm/gpio.h>
#include <dm/uclass.h>
#include <debug_impl.h>
#include <iniutility.h>
#include "mtk_tcon_common.h"

#define TRUE                        1
#define FALSE                       0
#define FILE_CUS_PARTITION          ""
#define FILE_DEFAULT_PARTITION_1    "bootdata" /* STI flow usage*/
#define FILE_DEFAULT_PARTITION_2    "tvconfig" /* Mixed mode flow usage*/
#define FILE_FIXED_FOLDER           "tcon/"
#define FILE_PATH_LENGTH            (128)
#define PANEL_DLG_PARTITION_PATH    "persist"
#define PANEL_DLG_INI_PATH          "panel_mode_cfg.ini"


static unsigned char *pmic_bin_buf = NULL;
static loff_t pmic_size = 0;
static unsigned char *pgamma_bin_buf = NULL;
static loff_t pgamma_size = 0;
static unsigned char *ls_bin_buf = NULL;
static loff_t ls_size = 0;
static unsigned char *vcomic_bin_buf = NULL;
static loff_t vcomic_size = 0;
static unsigned int iic_bus = 0;
static struct udevice *i2c_cur_dev = NULL;

static int _mtk_pnl_gpio_set_high(unsigned int gpio)
{
    int ret = 0;
    int value = 1;
    char name[4]={0,};
    if(snprintf(name,4,"%d",gpio) < 0){
        UBOOT_DEBUG("snprintf error! continue\n");
    }
    UBOOT_TRACE("%s,%d,gpio=%d,name=%s\n",__FUNCTION__,__LINE__,gpio,name);
    ret += gpio_lookup_name(name, NULL, NULL, &gpio);
    if (ret) {
        UBOOT_DEBUG("GPIO: '%s' not found\n", name);
    }
    ret += gpio_request(gpio, "cmd_gpio");
    if (ret && ret != -EBUSY) {
        UBOOT_DEBUG("gpio: requesting pin %u failed\n", gpio);
        return -1;
    }
    ret += gpio_direction_output(gpio, value);
    return ret;
}

static int _mtk_pnl_gpio_set_low(unsigned int gpio)
{
    int ret = 0;
    int value = 0;
    char name[4]={0,};
    if(snprintf(name,4,"%d",gpio) < 0){
        UBOOT_DEBUG("snprintf error! continue\n");
    }
    UBOOT_TRACE("%s,%d,gpio=%d,name=%s\n",__FUNCTION__,__LINE__,gpio,name);
    ret += gpio_lookup_name(name, NULL, NULL, &gpio);
    if (ret) {
        UBOOT_DEBUG("GPIO: '%s' not found\n", name);
    }
    ret += gpio_request(gpio, "cmd_gpio");
    if (ret && ret != -EBUSY) {
        UBOOT_DEBUG("gpio: requesting pin %u failed\n", gpio);
        return -1;
    }
    ret += gpio_direction_output(gpio, value);
    return ret;
}

int _mtk_pnl_gpio_set_input(unsigned int gpio)
{
    int ret = 0;
    char name[4]={0,};
    if(snprintf(name,4,"%d",gpio) < 0){
        UBOOT_DEBUG("snprintf error! continue\n");
   }
    UBOOT_TRACE("%s,%d,gpio=%d,name=%s\n",__FUNCTION__,__LINE__,gpio,name);
    ret += gpio_lookup_name(name, NULL, NULL, &gpio);
    if (ret) {
        UBOOT_DEBUG("GPIO: '%s' not found\n", name);
    }
    ret += gpio_request(gpio, "cmd_gpio");
    if (ret && ret != -EBUSY) {
        UBOOT_DEBUG("gpio: requesting pin %u failed\n", gpio);
        return -1;
    }
    ret += gpio_direction_input(gpio);
    return ret;
}

static int _mtk_pnl_cust_iic_init(int busid)
{
    int ret = 0;

    if ((ret = uclass_get_device_by_seq(UCLASS_I2C, busid, &i2c_cur_dev)) != 0) {
        UBOOT_DEBUG("uclass_get_device_by_seq(UCLASS_I2C,%d) fail=%d.\n", busid, ret);
        return ret;
    }
    else
        iic_bus = busid;
    return 0;
}

static int _mtk_pnl_cust_iic_set_bus_num(unsigned int busnum)
{
    struct udevice *bus;
    int ret;

    ret = uclass_get_device_by_seq(UCLASS_I2C, busnum, &bus);
    if (ret) {
       UBOOT_TRACE("%s: No bus %d\n", __func__, busnum);
       return ret;
    }
    i2c_cur_dev = bus;

    return 0;
}

static int _mtk_pnl_cust_iic_get_cur_bus_chip(uint chip_addr, struct udevice **devp)
{
    struct udevice *bus;
    int ret = 0;

    if (!i2c_cur_dev) {
        UBOOT_DEBUG("No I2C bus selected\n");
        return -ENODEV;
    }
    bus = i2c_cur_dev;

    ret = i2c_get_chip(bus, chip_addr, 1, devp);
    return ret;
}

static int _mtk_pnl_cust_iic_read(unsigned char  u8SlaveCfg, unsigned long uAddrCnt, unsigned char  *pRegAddr, unsigned long uSize, unsigned char  *pData)
{
    unsigned int addr;
    int ret = 0;
    struct udevice *dev;

    if(uAddrCnt > 1)
    {
        addr = (pRegAddr[0]<<8) + (pRegAddr[1]&0xff);
    }
    else
    {
       addr = (pRegAddr[0]);
    }

    if(_mtk_pnl_cust_iic_set_bus_num(iic_bus)!=0)
    {
      UBOOT_DEBUG("i2c_get_cur_bus_chip fail\n");
      return -1;
    }

    if(_mtk_pnl_cust_iic_get_cur_bus_chip(u8SlaveCfg, &dev)!=0)
    {
      UBOOT_DEBUG("i2c_get_cur_bus_chip fail\n");
      return -1;
    }

    if(i2c_set_chip_offset_len(dev, uAddrCnt)!=0)
    {
      UBOOT_DEBUG("i2c_set_chip_offset_len fail\n");
      return -1;
    }

    ret=dm_i2c_read(dev, addr, pData, uSize);
    if(ret !=0)
    {
      UBOOT_DEBUG("dm_i2c_read fail,%d\n",ret);
      return -1;
    }

    return 0;
}

static int _mtk_pnl_cust_iic_write(unsigned char u8SlaveCfg, unsigned long uAddrCnt, unsigned char *pRegAddr, unsigned long uSize, unsigned char *pData)
{
    unsigned int addr = 0;

    int ret = 0;
    struct udevice *dev;

    if(uAddrCnt > 1)
    {
         addr = (pRegAddr[0]<<8) + (pRegAddr[1]&0xff);
    }
    else
    {
        addr = (pRegAddr[0]);
    }

    if(_mtk_pnl_cust_iic_set_bus_num(iic_bus)!=0)
    {
       UBOOT_DEBUG("i2c_get_cur_bus_chip fail\n");
       return -1;
    }

    if(_mtk_pnl_cust_iic_get_cur_bus_chip(u8SlaveCfg, &dev)!=0)
    {
       UBOOT_DEBUG("i2c_get_cur_bus_chip fail\n");
       return -1;
    }

    if(i2c_set_chip_offset_len(dev, uAddrCnt)!=0)
    {
       UBOOT_DEBUG("i2c_set_chip_offset_len fail\n");
       return -1;
    }

    ret=dm_i2c_write(dev, addr, pData, uSize);
    if(ret!=0)
    {
       UBOOT_DEBUG("dm_i2c_write fail,%d\n",ret);
       return -1;
    }

    return 0;
}

bool _mtk_pnl_cust_get_file_data(unsigned char **ppData, const char *pchPath, loff_t *pSize)
{
    // search order of binary file partition:
    //   1. uboot env: ${FILE_CUS_PARTITION}
    //   2. ${FILE_DEFAULT_PARTITION_1}
    //   3. ${FILE_DEFAULT_PARTITION_2}
    static char *chPartList[] = {
        NULL,
        FILE_DEFAULT_PARTITION_1,
        FILE_DEFAULT_PARTITION_2
    };
    bool bRet = FALSE;
    int i;
    size_t len;
    char chFilePath[FILE_PATH_LENGTH];
    //const char *pchPath;

    if (!ppData)
    {
        UBOOT_ERROR("ppData parameter is null\n");
        return FALSE;
    }

    if (!pchPath)
    {
        UBOOT_ERROR("pchPath parameter is null\n");
        return FALSE;
    }

    if (!pSize)
    {
        UBOOT_ERROR("pSize parameter is null\n");
        return FALSE;
    }

    memset(chFilePath, '\0', sizeof(chFilePath));

    len = strlen(FILE_FIXED_FOLDER) + strlen(pchPath) + 1;
    if (len >= sizeof(chFilePath)) {
        UBOOT_ERROR(
            "Warning: The length of the path is %d dytes\n"
            "more than the maximum file length supported\n"
            "by the system is %d bytes.\n",
            (int)len, (int)sizeof(chFilePath));
        return FALSE;
    }

    len = sizeof(chFilePath) - 1;
    strncat(chFilePath, FILE_FIXED_FOLDER, len);
    len = len - strlen(FILE_FIXED_FOLDER);
    strncat(chFilePath, pchPath, len);

    //select partition
    chPartList[0] = env_get(FILE_CUS_PARTITION);
    for (i = 0; i < ARRAY_SIZE(chPartList); i++)
    {
        if (!chPartList[i])
        {
            UBOOT_TRACE("Partition list [%d] is null\n", i);
            continue;
        }

        if(i==1)
        {
            /* STI flow for number 1 partition */
            if(!is_tcon_sti_flow())
            {
                UBOOT_BOOTTIME("NOT STI flow [%d]\n", i);
                continue;
            }
        }

        *ppData = read_storage_file_to_memory(chPartList[i], chFilePath, pSize);

        if (*ppData && pSize > 0)
        {
            UBOOT_TRACE("File found. Path='%s/%s'\n", chPartList[i], chFilePath);
            bRet = TRUE;
            break;
        }
        else
        {
            UBOOT_ERROR("File not found. Path='%s/%s'\n", chPartList[i], chFilePath);
        }
    }

    if (!bRet)
        UBOOT_TRACE("cannot find '%s'\n", chFilePath);

    return bRet;
}

static void _mtk_pnl_cust_ic_get_bin_file(st_multi_cust_ic_info *multi_cust_ic)
{

    if (multi_cust_ic == NULL) {
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return;
    }

    char *ic_bin_path = multi_cust_ic->pmic_info.ic_bin_file_path;

    if ((ic_bin_path != NULL) && (multi_cust_ic->pmic_info.force_init == 1)){
        _mtk_pnl_cust_get_file_data(&pmic_bin_buf, ic_bin_path, &pmic_size);
        UBOOT_TRACE("\033[0;32;31m [%s][%d] pmic_size=%lld\n\033[m", __FUNCTION__, __LINE__, pmic_size);
    }
    if(pmic_bin_buf == NULL)
    {
        UBOOT_TRACE("Read pmic bin file failure!\n");
    }

    ic_bin_path = multi_cust_ic->pgamma_info.ic_bin_file_path;
    if ((ic_bin_path != NULL) && (multi_cust_ic->pgamma_info.force_init == 1)){
        _mtk_pnl_cust_get_file_data(&pgamma_bin_buf, ic_bin_path, &pgamma_size);
        UBOOT_TRACE("\033[0;32;31m [%s][%d] pgamma_size=%lld\n\033[m", __FUNCTION__, __LINE__, pgamma_size);
    }
    if(pgamma_bin_buf == NULL)
    {
        UBOOT_TRACE("Read pgamma bin file failure!\n");
    }

    ic_bin_path = multi_cust_ic->levelshift_info.ic_bin_file_path;
    if ((ic_bin_path != NULL) && (multi_cust_ic->levelshift_info.force_init == 1)){
        _mtk_pnl_cust_get_file_data(&ls_bin_buf, ic_bin_path, &ls_size);
        UBOOT_TRACE("\033[0;32;31m [%s][%d] ls_size=%lld\n\033[m", __FUNCTION__, __LINE__, ls_size);
    }
    if(ls_bin_buf == NULL)
    {
        UBOOT_TRACE("Read levelshifit bin file failure!\n");
    }

    ic_bin_path = multi_cust_ic->vcomic_info.ic_bin_file_path;
    if ((ic_bin_path != NULL) && (multi_cust_ic->vcomic_info.force_init == 1)){
        _mtk_pnl_cust_get_file_data(&vcomic_bin_buf, ic_bin_path, &vcomic_size);
        UBOOT_TRACE("\033[0;32;31m [%s][%d] vcomic_size=%lld\n\033[m", __FUNCTION__, __LINE__, vcomic_size);
    }
    if(vcomic_bin_buf == NULL)
    {
        UBOOT_TRACE("Read vcom ic bin file failure!\n");
    }
    return;
}

static void _mtk_pnl_cust_ic_write(unsigned char *w_data, int w_size, st_cust_ic_info *cust_info)
{
    uint16_t bus_id, slave_id;
    unsigned long addr_count;
    unsigned char reg_offset;
    unsigned long data_size;
    unsigned char *write_data;
    int ret;
    /******************************************************************************************************/
    // PNL_VCC on -> VCC delay(ms) -> Pre GPIO Operation -> Pre delay(ms) -> IC write (from bin) ->
    // Post GPIO Operation -> Post delay(ms)
    /******************************************************************************************************/

    if(cust_info == NULL)
        return;

    if(cust_info->vcc_dly > 0) {
        mdelay(cust_info->vcc_dly);
    }

    if(cust_info->gpio_pre_num > 0) {
        switch(cust_info->gpio_pre_ops)
        {
            case 0:
            {
                _mtk_pnl_gpio_set_low(cust_info->gpio_pre_num);
                UBOOT_TRACE("GPIO[%d] Ouput Low\n",cust_info->gpio_pre_num);
                break;
            }
            case 1:
            {
                _mtk_pnl_gpio_set_high(cust_info->gpio_pre_num);
                UBOOT_TRACE("GPIO[%d] Ouput High\n",cust_info->gpio_pre_num);
                break;
            }
            case 0xFF:
            default:
            {
                _mtk_pnl_gpio_set_input(cust_info->gpio_pre_num);
                UBOOT_TRACE("GPIO[%d] Input Hi-Z\n",cust_info->gpio_pre_num);
                break;
            }
        }
    }

    if(cust_info->gpio_pre_dly > 0) {
        mdelay(cust_info->gpio_pre_dly);
    }
    bus_id = cust_info->i2c_bus;
    slave_id = cust_info->i2c_dev_addr;
    addr_count = 1;
    reg_offset = cust_info->i2c_reg_offset;
    data_size = w_size;
    write_data = w_data;

    UBOOT_TRACE("I2C write MultiByte bus_id=%d, slave_id=0x%02x, reg_offset=%d, data_size=%ld\n ", bus_id,slave_id,reg_offset,data_size);
    _mtk_pnl_cust_iic_init(bus_id);
    ret = _mtk_pnl_cust_iic_write(slave_id, addr_count, &reg_offset, data_size, write_data);
    if(ret < 0)
        UBOOT_DEBUG("write iic error!! bus_id=%d slave_id=0x%02x \n",bus_id, slave_id);

    if(cust_info->gpio_post_dly > 0) {
        mdelay(cust_info->gpio_post_dly);
    }
    if(cust_info->gpio_post_num > 0) {
        switch(cust_info->gpio_post_num)
        {
            case 0:
            {
                _mtk_pnl_gpio_set_low(cust_info->gpio_post_num);
                UBOOT_TRACE("GPIO[%d] Ouput Low\n",cust_info->gpio_post_num);
                break;
            }
            case 1:
            {
                _mtk_pnl_gpio_set_high(cust_info->gpio_post_num);
                UBOOT_TRACE("GPIO[%d] Ouput High\n",cust_info->gpio_post_num);
                break;
            }
            case 0xFF:
            default:
            {
                _mtk_pnl_gpio_set_input(cust_info->gpio_post_num);
                UBOOT_TRACE("GPIO[%d] Input Hi-Z\n",cust_info->gpio_post_num);
                break;
            }
        }
    }
    return;
}

static void _mtk_pnl_cust_ic_write_by_mask(unsigned char *w_data, int w_size, st_cust_ic_info *cust_info)
{
    return;
}


static void _mtk_pnl_cust_ic_read(unsigned char* r_data, int r_size, st_cust_ic_info *cust_info)
{
    uint16_t bus_id, slave_id;
    unsigned long addr_count;
    unsigned char reg_offset;
    unsigned long data_size;
    int ret;
    if((r_data == NULL) && (r_size == 0)){
        UBOOT_TRACE("invalid data !\n ");
    }

    bus_id = cust_info->i2c_bus;
    slave_id = cust_info->i2c_dev_addr;
    addr_count = 1;
    reg_offset = cust_info->i2c_reg_offset;
    data_size = r_size;
    UBOOT_TRACE("I2C read MultiByte bus_id=%d, slave_id=0x%02x, reg_offset=%d, data_size=%ld\n ",bus_id,slave_id,reg_offset,data_size);
    _mtk_pnl_cust_iic_init(bus_id);
    ret = _mtk_pnl_cust_iic_read(slave_id, addr_count, &reg_offset, data_size, r_data);
    if(ret < 0)
        UBOOT_DEBUG("read iic error!! bus_id=%d slave_id=0x%02x \n",bus_id, slave_id);
    return;
}

static void _mtk_pnl_cust_ic_read_by_mask(unsigned char* r_data, int r_size, st_cust_ic_info *cust_info)
{
    return;
}

static void __mtk_pnl_print_bin_file(unsigned char *bin_buf, loff_t size)
{
    loff_t x, y;

    if((bin_buf != NULL) && (size > 0)){
        UBOOT_TRACE("\n \n");
        for (y=0; y < (size / 16 + 1); y++)
        {
            for(x = 0; x < 16; x++)
            {
                if((y*16 + x) < size)
                    UBOOT_TRACE("0x%02X ",bin_buf[y*16 + x]);
            }
            UBOOT_TRACE("\n");
        }
    }
    UBOOT_TRACE("\n \n");
}
static int _mtk_pnl_cust_pmic_init(st_multi_cust_ic_info *multi_cust_ic)
{
    //int panel_vcc_state;
    st_cust_ic_info *pmic_info = NULL;
    unsigned char *r_data = NULL;
    int index = 0;
    unsigned long bin_checksum = 0;
    unsigned long read_checksum = 0;

    // 1.check panl_vcc
    if (multi_cust_ic == NULL) {
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return -ENXIO;
    }
/*
	panel_vcc_state = dm_gpio_get_value(&priv->gpio_vcc);
    if(panel_vcc_state != 1){
        dm_gpio_set_value(&priv->gpio_vcc, 0);
        UBOOT_TRACE("get gpio VCC value = %d\n",dm_gpio_get_value(&priv->gpio_vcc));
    }
*/
    // 2. init cust ic
    if ((pmic_size > 0) && (pmic_bin_buf != NULL)){
        pmic_info = malloc(sizeof(st_cust_ic_info));
        if(pmic_info == NULL){
            UBOOT_TRACE("malloc fail!\n");
            return -1;
        }
        pmic_info->bin_format_type = multi_cust_ic->pmic_info.bin_format_type;
        pmic_info->force_init = multi_cust_ic->pmic_info.force_init;
        pmic_info->with_nvm = multi_cust_ic->pmic_info.with_nvm;
        pmic_info->i2c_bus = multi_cust_ic->pmic_info.i2c_bus;
        pmic_info->i2c_dev_addr = multi_cust_ic->pmic_info.i2c_dev_addr;
        pmic_info->i2c_reg_offset = multi_cust_ic->pmic_info.i2c_reg_offset;
        pmic_info->vcc_dly = multi_cust_ic->pmic_info.vcc_dly;
        pmic_info->gpio_pre_num = multi_cust_ic->pmic_info.gpio_pre_num;
        pmic_info->gpio_pre_ops = multi_cust_ic->pmic_info.gpio_pre_ops;
        pmic_info->gpio_pre_dly = multi_cust_ic->pmic_info.gpio_pre_dly;
        pmic_info->gpio_post_num = multi_cust_ic->pmic_info.gpio_post_num;
        pmic_info->gpio_post_ops = multi_cust_ic->pmic_info.gpio_post_ops;
        pmic_info->gpio_post_dly = multi_cust_ic->pmic_info.gpio_post_dly;
        pmic_info->with_nvm = multi_cust_ic->pmic_info.with_nvm;

        __mtk_pnl_print_bin_file(pmic_bin_buf, pmic_size);

        if (pmic_info->bin_format_type == 0){
            _mtk_pnl_cust_ic_write(pmic_bin_buf, pmic_size, pmic_info);
        }else if (pmic_info->bin_format_type == 1){
            _mtk_pnl_cust_ic_write_by_mask(pmic_bin_buf, pmic_size, pmic_info);
        }else
            UBOOT_TRACE("not support this format:%d \n", pmic_info->bin_format_type);
        // 3. check sum
        r_data = malloc(pmic_size);
        if(r_data == NULL){
            UBOOT_TRACE("malloc fail!\n");
            goto pmic_ic_init_finish;
            return -1;
        }
        else{
            memset(r_data, 0 , pmic_size);
        }
        if (pmic_info->bin_format_type == 0){
            _mtk_pnl_cust_ic_read(r_data, pmic_size, pmic_info);
        }else if (pmic_info->bin_format_type == 1){
            _mtk_pnl_cust_ic_read_by_mask(r_data, pmic_size, pmic_info);
        }else
            UBOOT_TRACE("not support this format:%d \n", pmic_info->bin_format_type);
        for(index = 0; index < pmic_size; index++){
            bin_checksum += pmic_bin_buf[index];
            read_checksum += r_data[index];
        }
        if (bin_checksum != read_checksum){
        UBOOT_TRACE("checksum fail! write agin!\n");
        _mtk_pnl_cust_ic_write(pmic_bin_buf, pmic_size, pmic_info);
        }
        // 4. nvm cmd
        if(pmic_info->with_nvm == 1){
            // TO-DO set nvm cmd
        }else{
            goto pmic_ic_init_finish;
            return 0;
        }
    }
    goto pmic_ic_init_finish;

pmic_ic_init_finish:
    if(pmic_info != NULL){
        free(pmic_info);
        pmic_info = NULL;
    }
    if(r_data != NULL){
        free(r_data);
        r_data = NULL;
    }
    return 0;
}

static int _mtk_pnl_cust_pgamma_init(st_multi_cust_ic_info *multi_cust_ic)
{
    //int panel_vcc_state;
    st_cust_ic_info *pgamma_info = NULL;
    unsigned char *r_data = NULL;
    int index = 0;
    unsigned long bin_checksum = 0;
    unsigned long read_checksum = 0;

    // 1.check panl_vcc
    if (multi_cust_ic == NULL) {
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return -ENXIO;
    }
/*
    panel_vcc_state = dm_gpio_get_value(&priv->gpio_vcc);
    if(panel_vcc_state != 1){
        dm_gpio_set_value(&priv->gpio_vcc, 0);
        UBOOT_TRACE("get gpio VCC value = %d\n",dm_gpio_get_value(&priv->gpio_vcc));
    }
*/
    // 2. init cust ic
    if ((pgamma_size > 0) && (pgamma_bin_buf != NULL)){

        pgamma_info = malloc(sizeof(st_cust_ic_info));
        if(pgamma_info ==NULL){
            UBOOT_TRACE("malloc fail!\n");
            return -1;
        }
        pgamma_info->bin_format_type = multi_cust_ic->pgamma_info.bin_format_type;
        pgamma_info->force_init = multi_cust_ic->pgamma_info.force_init;
        pgamma_info->with_nvm = multi_cust_ic->pgamma_info.with_nvm;
        pgamma_info->i2c_bus = multi_cust_ic->pgamma_info.i2c_bus;
        pgamma_info->i2c_dev_addr = multi_cust_ic->pgamma_info.i2c_dev_addr;
        pgamma_info->i2c_reg_offset = multi_cust_ic->pgamma_info.i2c_reg_offset;
        pgamma_info->vcc_dly = multi_cust_ic->pgamma_info.vcc_dly;
        pgamma_info->gpio_pre_num = multi_cust_ic->pgamma_info.gpio_pre_num;
        pgamma_info->gpio_pre_ops = multi_cust_ic->pgamma_info.gpio_pre_ops;
        pgamma_info->gpio_pre_dly = multi_cust_ic->pgamma_info.gpio_pre_dly;
        pgamma_info->gpio_post_num = multi_cust_ic->pgamma_info.gpio_post_num;
        pgamma_info->gpio_post_ops = multi_cust_ic->pgamma_info.gpio_post_ops;
        pgamma_info->gpio_post_dly = multi_cust_ic->pgamma_info.gpio_post_dly;
        _mtk_pnl_cust_ic_write(pgamma_bin_buf, pgamma_size, pgamma_info);

        // 3. check sum
        r_data = malloc(pgamma_size);
        if(r_data == NULL){
            UBOOT_TRACE("malloc fail!\n");
            goto pgamma_ic_init_finish;
            return -1;
        }
        else{
            memset(r_data, 0 , pgamma_size);
        }
        _mtk_pnl_cust_ic_read(r_data, pgamma_size, pgamma_info);
        for(index = 0; index < pgamma_size; index++){
            bin_checksum += pgamma_bin_buf[index];
            read_checksum += r_data[index];
        }
        if (bin_checksum != read_checksum){
            UBOOT_TRACE("checksum fail! write agin!\n");
            _mtk_pnl_cust_ic_write(pgamma_bin_buf, pgamma_size, pgamma_info);
        }
    }
    goto pgamma_ic_init_finish;

pgamma_ic_init_finish:
    if(pgamma_info != NULL){
        free(pgamma_info);
        pgamma_info = NULL;
    }
    if(r_data != NULL){
        free(r_data);
        r_data = NULL;
    }
    return 0;
}

static int _mtk_pnl_cust_levelshifit_init(st_multi_cust_ic_info *multi_cust_ic)
{
    st_cust_ic_info *ls_info = NULL;
    unsigned char *r_data = NULL;
    int index = 0;
    unsigned long bin_checksum = 0;
    unsigned long read_checksum = 0;

    // 1.check panl_vcc
    if (multi_cust_ic == NULL) {
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return -ENXIO;
    }
/*
    panel_vcc_state = dm_gpio_get_value(&priv->gpio_vcc);
    if(panel_vcc_state != 1){
        dm_gpio_set_value(&priv->gpio_vcc, 0);
        UBOOT_TRACE("get gpio VCC value = %d\n",dm_gpio_get_value(&priv->gpio_vcc));
    }
*/
    // 2. init cust ic
    if ((ls_size > 0) && (ls_bin_buf != NULL)){
        ls_info = malloc(sizeof(st_cust_ic_info));
        if(ls_info ==NULL){
            UBOOT_TRACE("malloc fail!\n");
            return -1;
        }
        ls_info->bin_format_type = multi_cust_ic->levelshift_info.bin_format_type;
        ls_info->force_init = multi_cust_ic->levelshift_info.force_init;
        ls_info->with_nvm = multi_cust_ic->levelshift_info.with_nvm;
        ls_info->i2c_bus = multi_cust_ic->levelshift_info.i2c_bus;
        ls_info->i2c_dev_addr = multi_cust_ic->levelshift_info.i2c_dev_addr;
        ls_info->i2c_reg_offset = multi_cust_ic->levelshift_info.i2c_reg_offset;
        ls_info->vcc_dly = multi_cust_ic->levelshift_info.vcc_dly;
        ls_info->gpio_pre_num = multi_cust_ic->levelshift_info.gpio_pre_num;
        ls_info->gpio_pre_ops = multi_cust_ic->levelshift_info.gpio_pre_ops;
        ls_info->gpio_pre_dly = multi_cust_ic->levelshift_info.gpio_pre_dly;
        ls_info->gpio_post_num = multi_cust_ic->levelshift_info.gpio_post_num;
        ls_info->gpio_post_ops = multi_cust_ic->levelshift_info.gpio_post_ops;
        ls_info->gpio_post_dly = multi_cust_ic->levelshift_info.gpio_post_dly;

        __mtk_pnl_print_bin_file(ls_bin_buf, ls_size);

        _mtk_pnl_cust_ic_write(ls_bin_buf, ls_size, ls_info);

        // 3. check sum
        r_data = malloc(ls_size);
        if(r_data == NULL){
            UBOOT_TRACE("malloc fail!\n");
            goto ls_ic_init_finish;
            return -1;
        }
        else{
            memset(r_data, 0, ls_size);
        }
        _mtk_pnl_cust_ic_read(r_data, ls_size, ls_info);
        for(index = 0; index < ls_size; index++){
            bin_checksum += ls_bin_buf[index];
            read_checksum += r_data[index];
        }
        if (bin_checksum != read_checksum){
            UBOOT_TRACE("checksum fail! write agin!\n");
            _mtk_pnl_cust_ic_write(ls_bin_buf, ls_size, ls_info);
        }
    }
    goto ls_ic_init_finish;

ls_ic_init_finish:
    if(ls_info != NULL){
        free(ls_info);
        ls_info = NULL;
    }
    if(r_data != NULL){
        free(r_data);
        r_data = NULL;
    }
    return 0;
}

static int _mtk_pnl_cust_vcomic_init(st_multi_cust_ic_info *multi_cust_ic)
{
    return -1;
}

int _mtk_pnl_cust_pmic_override_vcom(st_multi_cust_ic_info *multi_cust_ic)
{
    st_cust_pmic_vcom_info *pInfo = NULL;
    uint32_t data_index = 0, i;

    if (!multi_cust_ic) {
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return -ENXIO;
    }

    if (!pmic_bin_buf) {
        UBOOT_TRACE("[%s][%d] invalid pmic bin data!\n", __FUNCTION__,__LINE__);
        return -ENXIO;
    }

    pInfo = &(multi_cust_ic->pmic_vcom_info);

    //The VCOM data adjusted in emmc is implemented to override the VCOM default data in the PMIC.
    //pmic_bin_buf: the parameter is original pmic data
    //pmic_size: the parameter is original pmic data length
    if (pInfo && pInfo->override_enable) {
        UBOOT_TRACE("[%s][%d] addr_start=%d addr_lengh=%d\n",
                __FUNCTION__, __LINE__, pInfo->addr_start, pInfo->addr_length);

        for (i = 0; i < pInfo->addr_length; i++) {
            data_index = pInfo->addr_start + i;

            if (data_index < pmic_size) {
                UBOOT_TRACE("[%s][%d] pmic_bin_buf[%d]=0x%X\n",
                    __FUNCTION__, __LINE__, data_index, pmic_bin_buf[data_index]);

                //do override...

            } else
                UBOOT_TRACE("[%s][%d] Override addr over the pmic length\n",
                    __FUNCTION__, __LINE__);
        }
    } else
        UBOOT_TRACE("[%s][%d] Override is disable\n", __FUNCTION__, __LINE__);

    return 0;
}

int _mtk_pnl_cust_ic_override_data(st_multi_cust_ic_info *multi_cust_ic)
{
    if (!multi_cust_ic) {
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return -ENXIO;
    }

    _mtk_pnl_cust_pmic_override_vcom(multi_cust_ic);

    return 0;
}

int _mtk_pnl_is_support_vrr_mode(st_cust_tcon_info *tcon_info)
{
    if(tcon_info->bVRR_HighFrameRateMode_Support ||
       tcon_info->bVRR_HighPixelClockMode_Support ||
       tcon_info->bGAME_Direct60HzMode_Support)
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }
}

int mtk_pnl_is_dlg_mode(void)
{
    int ret = 0;
    struct panel_dlg_info panel_dlg_info;

    memset(&panel_dlg_info, 0, sizeof(panel_dlg_info));

    ret = get_panel_dlg_info(PANEL_DLG_PARTITION_PATH, PANEL_DLG_INI_PATH, &panel_dlg_info);
    if((ret >= 0) && (panel_dlg_info.panel_dlg == 1))
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }

}

int _mtk_pnl_is_game_direct_mode(void)
{
    int ret = 0;
    struct panel_dlg_info panel_dlg_info;

    memset(&panel_dlg_info, 0, sizeof(panel_dlg_info));

    ret = get_panel_dlg_info(PANEL_DLG_PARTITION_PATH, PANEL_DLG_INI_PATH, &panel_dlg_info);
    if((ret >= 0) && (panel_dlg_info.panel_dlg == 2))
    {
        return TRUE;
    }
    else
    {
        return FALSE;
    }

}

/*
------------------------------------------------------------------
VCC --> onTiming1 delay --> Data --> onTiming2 delay --> Backlight
------------------------------------------------------------------
*/

int mtk_pnl_cust_settings_befor_vcc(st_multi_cust_ic_info *multi_cust_ic)
{
	UBOOT_TRACE("cust settings befor vcc!\n");
	return FALSE;
}

int mtk_pnl_cust_settings_vcc_ontiming1(st_multi_cust_ic_info *multi_cust_ic)
{
	UBOOT_TRACE("cust settings vcc --> ontiming1!\n");

    // 1.get bin file
    if(multi_cust_ic == NULL){
        UBOOT_TRACE("[%s][%d] invalid data!\n", __FUNCTION__,__LINE__);
        return FALSE;
    }

    _mtk_pnl_cust_ic_get_bin_file(multi_cust_ic);

    //The new data that can override the default data of the bin file.
    _mtk_pnl_cust_ic_override_data(multi_cust_ic);

    if(multi_cust_ic->pmic_info.force_init == 1){
        _mtk_pnl_cust_pmic_init(multi_cust_ic);
    }
    if(multi_cust_ic->pgamma_info.force_init == 1){
        _mtk_pnl_cust_pgamma_init(multi_cust_ic);
    }
    if(multi_cust_ic->levelshift_info.force_init == 1){
        _mtk_pnl_cust_levelshifit_init(multi_cust_ic);
    }
    if(multi_cust_ic->vcomic_info.force_init == 1){
        _mtk_pnl_cust_vcomic_init(multi_cust_ic);
    }
	return TRUE;
}

int mtk_pnl_cust_settings_ontiming1_data(st_multi_cust_ic_info *multi_cust_ic)
{
	UBOOT_TRACE("cust settings ontiming1 --> data!\n");
	return FALSE;
}

int mtk_pnl_cust_settings_data_ontiming2(st_multi_cust_ic_info *multi_cust_ic)
{
	UBOOT_TRACE("cust settings data --> ontiming2!\n");
	return FALSE;
}

int mtk_pnl_cust_settings_ontiming2_backlight(st_multi_cust_ic_info *multi_cust_ic)
{
	UBOOT_TRACE("cust settings ontiming2 --> backlight!\n");
	return FALSE;
}

int mtk_pnl_cust_settings_after_backlight(st_multi_cust_ic_info *multi_cust_ic)
{
	UBOOT_TRACE("cust settings after backlight!\n");
	return FALSE;
}

int mtk_pnl_cust_set_panel_mode(st_cust_tcon_info *tcon_info)
{
    uint16_t bus_id = 0;
    uint16_t slave_id = 0;
    unsigned long addr_count = 0;
    unsigned char reg_offset = 0;
    unsigned long data_size = 0;
    unsigned char *write_data;
    int ret = 0;

    UBOOT_TRACE("mtk_pnl_cust_set_panel_mode!\n");
    if(_mtk_pnl_is_support_vrr_mode(tcon_info) == FALSE)
    {
        UBOOT_TRACE("This panel not support VRR mode. skip it\n");
        return FALSE;
    }

    // Mode Change GPIO setting
    if((mtk_pnl_is_dlg_mode() == TRUE) && (tcon_info->bSupportGpioModeChange) && (tcon_info->u16GPIO_NUM_MODE_CHG))
    {
        if(tcon_info->u16GPIO_NUM_MODE_CHG > 0)
        {
            switch(tcon_info->ucGPIO_CTRL_MODE_CHG_ON)
            {
                case 0:
                {
                    _mtk_pnl_gpio_set_low(tcon_info->u16GPIO_NUM_MODE_CHG);
                    UBOOT_TRACE("GPIO[%d] Ouput Low\n",tcon_info->u16GPIO_NUM_MODE_CHG);
                    break;
                }
                case 1:
                {
                    _mtk_pnl_gpio_set_high(tcon_info->u16GPIO_NUM_MODE_CHG);
                    UBOOT_TRACE("GPIO[%d] Ouput High\n",tcon_info->u16GPIO_NUM_MODE_CHG);
                    break;
                }
                case 0xFF:
                default:
                {
                    _mtk_pnl_gpio_set_input(tcon_info->u16GPIO_NUM_MODE_CHG);
                    UBOOT_TRACE("GPIO[%d] Input Hi-Z\n",tcon_info->u16GPIO_NUM_MODE_CHG);
                    break;
                }
            }
        }

        if(tcon_info->u16DELAY_MODE_CHG > 0)
        {
            mdelay(tcon_info->u16DELAY_MODE_CHG);
        }
    }
    else if((mtk_pnl_is_dlg_mode() == FALSE) && (tcon_info->bSupportGpioModeChange) && (tcon_info->u16GPIO_NUM_MODE_CHG))
    {
        if(tcon_info->u16GPIO_NUM_MODE_CHG > 0)
        {
            switch(tcon_info->ucGPIO_CTRL_MODE_CHG_OFF)
            {
                case 0:
                {
                    _mtk_pnl_gpio_set_low(tcon_info->u16GPIO_NUM_MODE_CHG);
                    UBOOT_TRACE("GPIO[%d] Ouput Low\n",tcon_info->u16GPIO_NUM_MODE_CHG);
                    break;
                }
                case 1:
                {
                    _mtk_pnl_gpio_set_high(tcon_info->u16GPIO_NUM_MODE_CHG);
                    UBOOT_TRACE("GPIO[%d] Ouput High\n",tcon_info->u16GPIO_NUM_MODE_CHG);
                    break;
                }
                case 0xFF:
                default:
                {
                    _mtk_pnl_gpio_set_input(tcon_info->u16GPIO_NUM_MODE_CHG);
                    UBOOT_TRACE("GPIO[%d] Input Hi-Z\n",tcon_info->u16GPIO_NUM_MODE_CHG);
                    break;
                }
            }
        }

        if(tcon_info->u16DELAY_MODE_CHG > 0)
        {
            mdelay(tcon_info->u16DELAY_MODE_CHG);
        }
    }

    // I2C WP gpio setting before i2c write (wp off, i2c on)
    if(((mtk_pnl_is_dlg_mode())||(_mtk_pnl_is_game_direct_mode()))&&
        (tcon_info->bSupportTconI2cWP)&&(tcon_info->u16GPIO_NUM_TCON_I2C_WP))
    {
        if(tcon_info->u16GPIO_NUM_TCON_I2C_WP > 0)
        {
            switch(tcon_info->ucGPIO_TCON_I2C_WP_OFF)
            {
                case 0:
                {
                    _mtk_pnl_gpio_set_low(tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    UBOOT_TRACE("GPIO[%d] Ouput Low\n",tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    break;
                }
                case 1:
                {
                    _mtk_pnl_gpio_set_high(tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    UBOOT_TRACE("GPIO[%d] Ouput High\n",tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    break;
                }
                case 0xFF:
                default:
                {
                    _mtk_pnl_gpio_set_input(tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    UBOOT_TRACE("GPIO[%d] Input Hi-Z\n",tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    break;
                }
            }
        }

        //TODO: delay
    }

    // I2C setting
    if((mtk_pnl_is_dlg_mode() == TRUE)&&(tcon_info->bVRR_HighFrameRateMode_Support == TRUE)) // Turn on DLG mode
    {
        bus_id = tcon_info->ucVRR_HFR_ON_I2C_bus;
        slave_id = tcon_info->ucVRR_HFR_ON_I2C_dev_addr;
        addr_count = 1;
        reg_offset = tcon_info->ucVRR_HFR_ON_I2C_offst;
        data_size = tcon_info->ucVRR_HFR_ON_I2C_DATA_SIZE;
        write_data = tcon_info->ucVRR_HFR_ON_I2C_DATA;

        UBOOT_TRACE("I2C write MultiByte bus_id=%d, slave_id=0x%02x, reg_offset=%d, data_size=%ld\n ", bus_id,slave_id,reg_offset,data_size);
        _mtk_pnl_cust_iic_init(bus_id);
        ret = _mtk_pnl_cust_iic_write(slave_id, addr_count, &reg_offset, data_size, write_data);

        if(ret < 0)
            UBOOT_DEBUG("write iic error!! bus_id=%d slave_id=0x%02x \n",bus_id, slave_id);
    }
    else if((_mtk_pnl_is_game_direct_mode() == TRUE)&&(tcon_info->bGAME_Direct60HzMode_Support == TRUE)) // Turn on GAME mode
    {
        //TODO
    }

    // I2C WP gpio setting after i2c write (wp on, i2c off)
    if(((mtk_pnl_is_dlg_mode())||(_mtk_pnl_is_game_direct_mode()))&&
        (tcon_info->bSupportTconI2cWP)&&(tcon_info->u16GPIO_NUM_TCON_I2C_WP))
    {
        if(tcon_info->u16GPIO_NUM_TCON_I2C_WP > 0)
        {
            switch(tcon_info->ucGPIO_TCON_I2C_WP_ON)
            {
                case 0:
                {
                    _mtk_pnl_gpio_set_low(tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    UBOOT_TRACE("GPIO[%d] Ouput Low\n",tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    break;
                }
                case 1:
                {
                    _mtk_pnl_gpio_set_high(tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    UBOOT_TRACE("GPIO[%d] Ouput High\n",tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    break;
                }
                case 0xFF:
                default:
                {
                    _mtk_pnl_gpio_set_input(tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    UBOOT_TRACE("GPIO[%d] Input Hi-Z\n",tcon_info->u16GPIO_NUM_TCON_I2C_WP);
                    break;
                }
            }
        }
    }

    return TRUE;
}


