/*
 * Copyright 2023 Sony Corporation
 */

#include <command.h>
#include <common.h>
#include <utility.h>
#include <iniparser.h>
#include <iniutility.h>
#include <debug_impl.h>
#include <standby_mode.h>
#ifdef CONFIG_DATA_SEPARATION
#include <mtk_dataindex.h>
#endif

static int check_quiescent_and_update_enter_standby(const char *part, const char *relpath, struct standby_qhb_info *standby_info)
{
	int ret = 0;
#if (CONFIG_MTK_ANDROID_HEADER_VERSION > 3)
	char *quiescent = env_get("bootconfig");
#else
	char *quiescent = env_get("bootargs");
#endif

	if (quiescent == NULL) {
		printf("%s %d Cannot get quiescent flag\n", __func__, __LINE__);
		return -1;
	} else if (strstr(quiescent, "androidboot.quiescent=1")) {
		standby_info->enter_standby = 1;
		ret = store_standby_qhb_info(part, relpath, standby_info);
		if (ret < 0) {
			printf("%s %d cannot update standby_mode.ini\n", __func__, __LINE__);
			return ret;
		}
		printf("%s %d update standby_mode.ini\n", __func__, __LINE__);
	}
	return 0;
}

static int update_standby_mode(void)
{
	struct standby_qhb_info standby_info;
	int ret = 0;
#ifdef CONFIG_DATA_SEPARATION
	char filepath[FILE_PATH_SIZE], part[PART_NAME_SIZE];
	const char *relpath;

	memset(part, 0, sizeof(part));
	memset(filepath, 0, sizeof(filepath));
	if (dataindex_get_key(filepath, FILE_PATH_SIZE, STANDBY_SECTION, STANDBY_KEY, NULL) == 0) {
		if (dataindex_resolve_path(part, PART_NAME_SIZE, &relpath, filepath) == 0) {
			ret = load_standby_qhb_info(part, relpath, &standby_info);
			if (ret < 0) {
				UBOOT_ERROR("Read ini information failure with data separation.[%s][%s]\n", part, relpath);
				return -1;
			}
			check_quiescent_and_update_enter_standby(part, relpath, &standby_info);
		} else {
			UBOOT_ERROR("resolve path fail: %s\n", filepath);
			return -1;
		}
	} else {
		UBOOT_ERROR("cannot get %s:%s from dataindex file\n", STANDBY_SECTION, STANDBY_KEY);
		return -1;
	}
#else
	ret = load_standby_qhb_info(STANDBY_PARTITION, STANDBY_FILE_PATH, &standby_info);
	if (ret < 0) {
		UBOOT_ERROR("Read ini information failure with default setting.[%s][%s]\n", STANDBY_PARTITION, STANDBY_FILE_PATH);
		return -1;
	}
	check_quiescent_and_update_enter_standby(part, relpath, &standby_info);
#endif

	return 0;
}

int do_update_standby_mode(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	printf("%s %d\n", __func__, __LINE__);
	update_standby_mode();
	return 0;
}

U_BOOT_CMD(
		update_standby_mode, 1, 0, do_update_standby_mode,
		"Update standby_mode.ini",
		NULL
		);
