// SPDX-License-Identifier: (GPL-2.0-only OR BSD-3-Clause)
/*
 * Copyright (c) 2023 MediaTek Inc.
 */

#include <command.h>
#include <common.h>
#include <stdio.h>
#include <linux/string.h>
#include <MsTypes.h>
#include <debug_impl.h>
#include <dmalloc.h>
#include <ms_utils.h>
#include <ms_array.h>
#include <demura.h>
#include <crc_libs.h>
#include <demura_config.h>
#include <CommonDataType.h>
#include <convert_entry.h>

//#include <apiPNL.h>
#include <MDrvDemura.h>
#include <halDemura.h>
#include "vendor.h"
#include "demura_config.h"

#undef ALIGN
#define ALIGN(x,a)        __ALIGN_MASK((x),(typeof(x))(a)-1)
#define __ALIGN_MASK(x,mask)    (((x)+(mask))&~(mask))

#define minmax(v,a,b) (((v)<(a)) ? (a) : ((v)>(b)) ? (b) : (v))
//---------------------------------------------------------------
void CUC_cvt_blk_size_add_CIC(int ****data)
{
    if ((multi_vendor == EN_DEMURA_MULTI_LGD_SQ18_19_COLOR) || (multi_vendor == EN_DEMURA_MULTI_LGD_SQ18_19_MONO))
    {
        int temp_size1 = 481;
        int temp_size2 = 271;
        int div = 4;
    }
    else if ((multi_vendor == EN_DEMURA_MULTI_LGD_V19_COLOR) || (multi_vendor == EN_DEMURA_MULTI_LGD_V19_MONO))
    {
        int temp_size1 = 241;
        int temp_size2 = 136;
        int div = 2;
    }

    // CUC(R,G,B) block size conversion by linear interpolation, and add CIC at last
    int i, j, c, t;
    for (t = 0; t < 4; t++)
    {
        for (c = 0; c < 3; c++)
        {
            // Horizontal Color 32x32 -> 8x32 linear interpolation
            for (j = 0; j < 69; j++)
            {
                int tmp[temp_size1] = { 0 };
                for (i = 0; i < temp_size1; i++)
                {
                    int dist_x_0 = (i % div);
                    int dist_x_1 = div - dist_x_0;
                    tmp[i] = (data[j][i / div][c][t] * dist_x_1 + data[j][minmax(i / div + 1, 0, 120)][c][t] * dist_x_0 + (div>>1)) / div;
                }
                for (i = 0; i < 481; i++)
                {
                    data[j][i][c][t] = tmp[i];
                }
            }

            // Vertical Color 8x32 -> 8x8 linear interpolation
            // And add comp_W to each comp_C (for C = R,G,B)
            for (i = 0; i < temp_size1; i++)
            {
                int tmp[temp_size2] = { 0 };
                for (j = 0; j < temp_size2; j++)
                {
                    int dist_y_0 = (j % div);
                    int dist_y_1 = div - dist_y_0;
                    tmp[j] = (data[j / div][i][c][t] * dist_y_1 + data[minmax(j / div + 1, 0, 67)][i][c][t] * dist_y_0 + (div)>>1) / div;
                }
                for (j = 0; j < temp_size2; j++)
                {
                    data[j][i][c][t] = minmax(tmp[j] + data[j][i][3][t], -128, 127);
                }
            }
        }
    }
}

MS_BOOL Load_LGD_Bin(char **pbuf, int flash_start_addr, int bin_size)
{
    MS_BOOL  bRet;
    char *buffer;
    MS_U32 fsize;

    UBOOT_TRACE("IN\n");
    int start   = flash_start_addr;

    fsize = ALIGN(bin_size, 32);
    buffer = (char *)dmalloc(fsize + 32);
    CHECK_DMALLOC_SPACE(buffer, (uint)fsize + 32);
    memset(buffer, 0, sizeof(buffer));  //coverity issue

    // read binary file
    bRet = read_spi_flash((MS_U8 *)buffer, start, fsize);  // optimize,
    if (bRet == FALSE)
    {
        UBOOT_ERROR("read LGD Demura binary error\n");
        dfree(buffer);
        return FALSE;
    }

    *pbuf = buffer;
    UBOOT_TRACE("OK\n");
    return TRUE;
}



MS_BOOL parse_LG_bin_1tab(int n_Hnode, int n_Vnode, interface_info *info, MS_BOOL bFCIC)
{
    // Create data array
    struct array_dim4 _data;
    if(DEFINE_ARRAY_DIM4(_data, sizeof(int), 271, 492, 4, 4)!=TRUE)
    {
        return FALSE;
    }
    int **** data = (int ****)PTR_ARRAY_DIM4(_data);

    int fcic_gain[4];
    int fcic_ofst[4];

    UBOOT_TRACE("IN\n");;

    int start_addr = 0; // must be times of 32 (0x20)
    int data_str_addr = 0;
    int chroma_4tab_addr = 0;
    int empty_dummy_addr = 0;
    int gain_offset_addr = 0;
    int checksum_addr = 0;
    char *fbuf = NULL;

    /* Mark up for CCN */
    /* #if defined (CONFIG_DEMURA_VENDOR_LGD_V19)
        start_addr = 0x80000; // must be times of 32 (0x20)
        data_str_addr = 0x4;
        chroma_4tab_addr = 0x085E4;
        //empty_dummy_addr = 0xA1A30;
        gain_offset_addr = 0x21A30;
        checksum_addr = 0x21A3C;
    #elif defined (CONFIG_DEMURA_VENDOR_LGD_SQ18_19)
        start_addr = 0x0; // must be times of 32 (0x20)
        data_str_addr = 0x4;
        chroma_4tab_addr = 0x208d8;
        empty_dummy_addr = 0x39d24;
        gain_offset_addr = 0x5c304;
        checksum_addr = 0x5c310;
    #elif defined (CONFIG_DEMURA_VENDOR_LGD_V18)
        start_addr = 0x60000;        // must be times of 32 (0x20)
        data_str_addr = 0x00004;     // 0x60004 , For optimize
        gain_offset_addr = 0x085e4;  // 0x685e4
        checksum_addr = 0x085f0;     // 0x685f0
    #endif */

    MS_BOOL bRet = Load_LGD_Bin(&fbuf, start_addr, (checksum_addr + 16));
    if (bRet == FALSE)
    {
        UBOOT_ERROR("Load_LGD_Bin Error\n");
        FREE_ARRAY_DIM4(_data);
        return FALSE;
    }

    //! Start Parsing
    int byte_address = 0;
    MS_BOOL end_flag = FALSE;
    int check_sum = 0;
    int total_sum = 0;
    int h_cnt = 0, v_cnt = 0, c_cnt = 0, t_cnt = 0;

    while (end_flag != TRUE)
    {
        int i;
        char *line;  // char line[32];
        line = fbuf + byte_address;

        for (i = 0; i < 32; i++) // Parse 32 bytes in one time
        {
            if (byte_address + i < data_str_addr) // Skip bulk detect bytes
                 continue;

            //! End after get Checksum byte
            /* Mark up for CCN */
            /* if (byte_address + i >= checksum_addr)
            {
                check_sum = line[i] & 0xff;
                end_flag = TRUE;
                break;
            } */

            #if defined (CONFIG_DEMURA_VENDOR_LGD_V19) || defined (CONFIG_DEMURA_VENDOR_LGD_SQ18_19)
                if (byte_address + i >= gain_offset_addr) // 12 bytes Gain & Offset
                {
                    /* Mark up for CCN */
                    /* if (byte_address + i == gain_offset_addr)
                        fcic_gain[2] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 1)
                        fcic_gain[1] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 2)
                        fcic_gain[0] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 4)
                        fcic_ofst[0] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 7)
                        fcic_gain[3] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 9)
                        fcic_ofst[3] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 10)
                        fcic_ofst[2] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 11)
                        fcic_ofst[1] = line[i] & 0xff; */
                }
                else if (byte_address + i < chroma_4tab_addr) // luma (3:W, 1:Gray64)
                {
                    data[v_cnt][h_cnt / 4 * 4 + (3 - i % 4)][3][1] = line[i];
                    //! Add index of data matrix
                    /* Mark up for CCN */
                    /* h_cnt++;
                    #if defined (CONFIG_DEMURA_VENDOR_LGD_V19)
                    if (h_cnt == 252)
                    #elseif defined (CONFIG_DEMURA_VENDOR_LGD_SQ18_19)
                    if (h_cnt == 492)
                    #endif
                    {
                        h_cnt = 0;
                        v_cnt++;
                    } */
                }
                #if defined (CONFIG_DEMURA_VENDOR_LGD_V19)
                else if (byte_address + i < gain_offset_addr) // chroma (0~2:RGB)
                #elif defined (CONFIG_DEMURA_VENDOR_LGD_SQ18_19)
                else if (byte_address + i < empty_dummy_addr) // chroma (0~2:RGB)
                #endif
                {
                    /* Mark up for CCN */
                    /* if (byte_address + i == chroma_4tab_addr)
                    {
                        UBOOT_DEBUG("Start parsing chroma, reset counters\n");
                        v_cnt = 0;
                        h_cnt = 0;
                        c_cnt = 0;
                        t_cnt = 0;
                    } */

                    data[v_cnt][h_cnt][c_cnt][t_cnt] = line[i];

                    //! Add index of data matrix
                    t_cnt++;
                    /* Mark up for CCN */
                    /* if (t_cnt == 4)
                    {
                        t_cnt = 0;
                        h_cnt++;
                        if (h_cnt == 125)
                        {
                            h_cnt = 0;
                            v_cnt++;
                            if (v_cnt == 69)
                            {
                                v_cnt = 0;
                                c_cnt++;
                            }
                         }
                    } */
                }
                //! (total_sum % 256) to verify check_sum
                total_sum += line[i];
                total_sum %= 256;
            #elif defined (CONFIG_DEMURA_VENDOR_LGD_V18)
                if (byte_address + i >= gain_offset_addr) // 12 bytes Gain & Offset
                {
                    /* Mark up for CCN */
                    /* if (byte_address + i == gain_offset_addr)
                        fcic_gain[2] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 1)
                        fcic_gain[1] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 2)
                        fcic_gain[0] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 4)
                        fcic_ofst[0] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 7)
                        fcic_gain[3] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 9)
                        fcic_ofst[3] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 10)
                        fcic_ofst[2] = line[i] & 0xff;
                    else if (byte_address + i == gain_offset_addr + 11)
                        fcic_ofst[1] = line[i] & 0xff; */
                }
                else
                {
                    data[v_cnt][h_cnt / 4 * 4 + (3 - i % 4)][3][1] = line[i];
                    //! Add index of data matrix
                    h_cnt++;
                    /* Mark up for CCN */
                    /* if (h_cnt == 252)
                    {
                        h_cnt = 0;
                        v_cnt++;
                    } */
                }
                //! (total_sum % 256) to verify check_sum
                total_sum += (int)(line[i]);
                total_sum %= 256;
            #endif
        }
        byte_address += 32;
    }

    UBOOT_DEBUG("Checksum = %x\n", check_sum);
    UBOOT_DEBUG("2's complement(total_sum %% 256) = %x\n", ((-1 * total_sum) & 0xff));
    /* Mark up for CCN */
    /* if (((-1 * total_sum) & 0xff) == check_sum)
    {
        UBOOT_DEBUG("\nChecksum PASS !!!\n");
    }
    else
    {
        UBOOT_ERROR("\nChecksum FAIL !!!\n");
        FREE_ARRAY_DIM4(_data);
        dfree(fbuf);
        return FALSE;
    } */

    // Sign spi flash data
    MS_U32 sample_crc32;
    char  strbuf[1024];

    sample_crc32 = MDrv_CRC32_Cal_DeMura((MS_U8 *)(fbuf+SAMPLE_DAT_START), SAMPLE_DAT_LEN);
    if (sample_crc32 == 0xffffFFFF)
    {
        UBOOT_ERROR("MDrv_CRC32_Cal_DeMura error\n");
    }
    memset(strbuf, 0, sizeof(strbuf));
    snprintf(strbuf, sizeof(strbuf)-1, "0x%x:0x%x", check_sum, sample_crc32);
    env_set(ENV_DEMURA_SIG, strbuf);
#if CONFIG_DEMURA_ENV_SAVEENV
    env_save();
#endif
    // End sign spi flash data

    int i, j, t, c;
    /* Mark up for CCN */
   /*  for (t = 0; t < 4; t++)
    {
        UBOOT_DEBUG("Tab %d: gain = %d, ofst = %d\n", t, fcic_gain[t], fcic_ofst[t]);
    } */

    //! The data provided by LGD is 1's complement
    //! Convert 1's complement to 2's complement
    //! For Full&Simple FCIC, firstly convert G64 where the 1-layer data provided for "W" channel only
    /* Mark up for CCN */
    /* for (j = 0; j < n_Vnode; j++)
    {
        for (i = 0; i < n_Hnode; i++)
        {
            int sign = 0;
            int value = data[j][i][3][1] & 0x7f;
            //value = (value) / 2; // s6.1 to s7(s0_6.0)
            if ((data[j][i][3][1] & 0x80) == 0x80)
                sign = -1;
            else
                sign = 1;

            data[j][i][3][1] = sign * value;
        }
    } */

    /* Mark up for CCN */
    /* for (c = 0; c < 4; c++)
    {
        for (t = 0; t < 4; t++)
        {
            for (j = 0; j < n_Vnode; j++)
            {
                for (i = 0; i < n_Hnode; i++)
                {
                    int sign = 0;
                    int value = data[j][i][c][t] & 0x7f;
                    if ((data[j][i][c][t] & 0x80) == 0x80)
                        sign = -1;
                    else
                        sign = 1;

                    //! For Full&Simple FCIC, calculate G64 with gain&offset to generate the other tabs for "W" channel only
                    if (c == 3)
                    {
                        //test if (t != 1) // Skip 64 layer for gain & offset zero pattern
                        {
                            int final_out = ((int)(data[j][i][c][1]) * fcic_gain[t]) / 64 + fcic_ofst[t];
                            if (final_out < -128 || final_out > 127)
                            {
                                UBOOT_DEBUG("FCIC: Overflow according to data with the gains & offsets\n");
                                //FREE_ARRAY_DIM4(_data);
                                //dfree(fbuf);
                                //return FALSE;
                            }
                            data[j][i][c][t] = final_out;
                        }
                    }
                    else
                    {
                        data[j][i][c][t] = sign * value;
                    }
                }
            }
        }
    } */

    ///////////////////////////////////////////////////////////////////////////////////////
    // Step 3:    Convert block size of RGB from 32x32 to 8x8 or 16x16
    //            Add CIC to CUC:
    //            ->    comp_R = comp_R + comp_Luma
    //            ->    comp_G = comp_G + comp_Luma
    //            ->    comp_B = comp_B + comp_Luma
    //            ->    comp_W = comp_Luma
    ///////////////////////////////////////////////////////////////////////////////////////
    #if defined (CONFIG_DEMURA_VENDOR_LGD_CIC_CUC)
        printf("CONFIG_DEMURA_VENDOR_LGD_CIC_CUC\n");

        CUC_cvt_blk_size_add_CIC(data);
    #else
        printf("CONFIG_DEMURA_VENDOR_LGD_CIC_ONLY \n");
    #endif

    int nLayer = 0;
    for (t = 0; t < 4; t++)
    {
        //get level
        switch (t)
        {
            case 0:
                nLayer = info->reg_dmc_plane_level1 >> 2;
                break;
            case 1:
                nLayer = info->reg_dmc_plane_level2 >> 2;
                break;
            case 2:
                nLayer = info->reg_dmc_plane_level3 >> 2;
                break;
            case 3:
                nLayer = info->reg_dmc_plane_level4 >> 2;
                break;
        }

        /* Mark up for CCN */
        /* for (j = 0; j < n_Vnode; j++)
        {
            for (i = 0; i < n_Hnode; i++)
            {
                #if defined (CONFIG_DEMURA_VENDOR_LGD_CIC_CUC)
                    info->Lut_in[t][j * n_Hnode + i].dbr = (double)(data[j][i][0][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].r   = (data[j][i][0][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].dbg = (double)(data[j][i][1][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].g   = (data[j][i][1][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].dbb = (double)(data[j][i][2][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].b   = (data[j][i][2][t] >> 1) + nLayer;
                    #if (defined (CONFIG_DEMURA_M7622) || defined (CONFIG_DEMURA_M7632))
                    info->Lut_in[t][j * n_Hnode + i].dbw = (double)(data[j][i][3][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].w   = (data[j][i][3][t] >> 1) + nLayer;
                    #endif
                #else
                info->Lut_in[t][j * n_Hnode + i].dbr = (double)(data[j][i][3][t] >> 1) + nLayer;
                info->Lut_in[t][j * n_Hnode + i].r   = (data[j][i][3][t] >> 1) + nLayer;
                info->Lut_in[t][j * n_Hnode + i].dbg = (double)(data[j][i][3][t] >> 1) + nLayer;
                info->Lut_in[t][j * n_Hnode + i].g   = (data[j][i][3][t] >> 1) + nLayer;
                info->Lut_in[t][j * n_Hnode + i].dbb = (double)(data[j][i][3][t] >> 1) + nLayer;
                info->Lut_in[t][j * n_Hnode + i].b   = (data[j][i][3][t] >> 1) + nLayer;
                    #if (defined (CONFIG_DEMURA_M7622) || defined (CONFIG_DEMURA_M7632))
                    info->Lut_in[t][j * n_Hnode + i].dbw = (double)(data[j][i][3][t] >> 1) + nLayer;
                    info->Lut_in[t][j * n_Hnode + i].w   = (data[j][i][3][t] >> 1) + nLayer;
                    #endif
                #endif
            }
        } */
    }

    FREE_ARRAY_DIM4(_data);
    dfree(fbuf);
    UBOOT_TRACE("OK\n");
    return TRUE;
}

MS_BOOL transfer_LGD_Data_1tab(void *pdat_info, BinOutputInfo *pbin_info)
{
    #if defined (CONFIG_DEMURA_URSA13)
    interface_info *pDataInfo = (interface_info *)pdat_info;
    #elif (defined (CONFIG_DEMURA_M7622) || defined (CONFIG_DEMURA_M7632))
    interface_info *pDataInfo = (interface_info *)pdat_info;
    #else
    #error "Unkown DEMURA_URSA_TYPE !"
    #endif

    int n_Hnode, n_Vnode;
    MS_BOOL bFCIC = pbin_info->Vendor_ARG1;

    #if defined (CONFIG_DEMURA_VENDOR_LGD_V19)
        n_Hnode = 241;
        n_Vnode = 136;
        pDataInfo->reg_dmc_h_block            = 0x4;         //  3 bit
        pDataInfo->reg_dmc_v_block            = 0x4;         //  3 bit
    #elif defined (CONFIG_DEMURA_VENDOR_LGD_SQ18_19)
        n_Hnode = 481;
        n_Vnode = 271;
        pDataInfo->reg_dmc_h_block            = 0x3;         //  3 bit
        pDataInfo->reg_dmc_v_block            = 0x3;         //  3 bit
    #elif defined (CONFIG_DEMURA_VENDOR_LGD_V18)
        n_Hnode = 241;
        n_Vnode = 136;
        pDataInfo->reg_dmc_h_block            = 0x4;         //  3 bit
        pDataInfo->reg_dmc_v_block            = 0x4;         //  3 bit
    #endif

    pDataInfo->reg_dmc_plane_num              = 0x4;         //  4 bit

    #if defined (CONFIG_DEMURA_VENDOR_LGD_CIC_CUC)
        pDataInfo->reg_dmc_rgb_mode               = 0x1;         //  1 bit
    #else
    pDataInfo->reg_dmc_rgb_mode               = 0x0;         //  1 bit
    #endif

    pDataInfo->reg_dmc_panel_h_size           = 0xF00;       // 13 bit
    pDataInfo->reg_dmc_black_limit            = 0x000;       // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level1           = 0x200;       // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level2           = 0x400;       // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level3           = 0x800;       // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level4           = 0xC00;       // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level5           = 0;           // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level6           = 0;           // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level7           = 0;           // 12 bit , format 10.2
    pDataInfo->reg_dmc_plane_level8           = 0;           // 12 bit , format 10.2
    pDataInfo->reg_dmc_white_limit            = 0xFFF;       // 12 bit , format 10.2
    pDataInfo->reg_dmc_dither_en              = 0x1;         //  2 bit , 0 : rounding , 1 : dither , 2~3 : truncate
    pDataInfo->reg_dmc_dither_rst_e_4_frame   = 0x0;         //  1 bit
    pDataInfo->reg_dmc_dither_pse_rst_num     = 0x0;         //  2 bit

    // Init Output message
    pbin_info->HNode          = n_Hnode;
    pbin_info->VNode          = n_Vnode;
    pbin_info->LevelCount     = pDataInfo->reg_dmc_plane_num;
    pbin_info->Blk_h_size     = 1 << (pDataInfo->reg_dmc_h_block);
    pbin_info->Blk_v_size     = 1 << (pDataInfo->reg_dmc_v_block);

    MS_BOOL bRet = Alloc_LutIn_Space(pdat_info, pbin_info);
    if (bRet == FALSE)
    {
        UBOOT_ERROR("Alloc_LutIn_Space error\n");
        return FALSE;
    }

    bRet = parse_LG_bin_1tab(n_Hnode, n_Vnode, pDataInfo, bFCIC);
    if (bRet == FALSE)
    {
        UBOOT_ERROR("parse_LG_bin_1tab error\n");
        return FALSE;
    }

    return TRUE;
}

