
#ifndef __SYSTRACKER_H__
#define __SYSTRACKER_H__

#define BUS_DBG_NUM_TRACKER         (8)

#define BUS_DBG_BASE                 (0x10208000)
#define BUS_DBG_CON                 (BUS_DBG_BASE + 0x0000)
#define BUS_DBG_AR_TRACK_L(__n)     (BUS_DBG_BASE + 0x0100 + 8 * (__n))
#define BUS_DBG_AR_TRACK_H(__n)     (BUS_DBG_BASE + 0x0104 + 8 * (__n))
#define BUS_DBG_AW_TRACK_L(__n)     (BUS_DBG_BASE + 0x0200 + 8 * (__n))
#define BUS_DBG_AW_TRACK_H(__n)     (BUS_DBG_BASE + 0x0204 + 8 * (__n))


#define BUS_DBG_CON_IRQ_AR_STA0     (0x00000100)
#define BUS_DBG_CON_IRQ_AW_STA0     (0x00000200)
#define BUS_DBG_CON_IRQ_AR_STA1     (0x00100000)
#define BUS_DBG_CON_IRQ_AW_STA1     (0x00200000)
#define BUS_DBG_CON_TIMEOUT         (BUS_DBG_CON_IRQ_AR_STA0|BUS_DBG_CON_IRQ_AW_STA0| \
                                    BUS_DBG_CON_IRQ_AR_STA1|BUS_DBG_CON_IRQ_AW_STA1)

/* a 2KB buffer is sufficient for systracker */
#define SYSTRACKER_BUF_LENGTH   2000

static inline unsigned int extract_n2mbits(unsigned int input, unsigned int n, unsigned int m)
{
    /*
    * 1. ~0 = 1111 1111 1111 1111 1111 1111 1111 1111
    * 2. ~0 << (m - n + 1) = 1111 1111 1111 1111 1100 0000 0000 0000
    * // assuming we are extracting 14 bits, the +1 is added for inclusive selection
    * 3. ~(~0 << (m - n + 1)) = 0000 0000 0000 0000 0011 1111 1111 1111
    */
    int mask;

    if (n > m) {
        n = n + m;
        m = n - m;
        n = n - m;
    }
    mask = ~(~0 << (m - n + 1));
    return (input >> n) & mask;
}
int systracker_get(void **data, int *len, unsigned int entry_num);
void systracker_put(void **data);

#endif
