// SPDX-License-Identifier: (GPL-2.0-only OR BSD-3-Clause)
/*
 * Copyright (c) 2023 MediaTek Inc.
 */

#include <config.h>
#include <common.h>
#include <command.h>
#include <malloc.h>
#include <dm/uclass.h>
#include <mhal_panel.h>
#include <panel_impl.h>
#include <debug_impl.h>
#include <Gamma12BIT_256.inc>
#include <MsCommon.h>
#include <system_impl.h>
#include <dts_parser.h>
#include <mtk_panel.h>
#include <time.h>
#include <asm-generic/gpio.h>
#include <pwm.h>
#include <dm/uclass-id.h>
#include <display.h>
#include <iniutility.h>
//#include <mtk_demura.h>
#include <utility.h>
#ifdef CONFIG_ENABLE_DEMURA
#include <demura_impl.h>
#endif
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
#include <mtk_pnl_cust.h>
#endif

#define PANEL_PWM_DUTY_PARTITION_PATH		"persist"
#define PANEL_PWM_DUTY_INI_PATH		"pwm_duty.ini"
#define PANEL_DLG_PARTITION_PATH		"persist"
#define PANEL_DLG_INI_PATH		"panel_mode_cfg.ini"

//-------------------------------------------------------------------------------------------------
//  Global Variables
//-------------------------------------------------------------------------------------------------
unsigned long lPanleTimer =0;
unsigned long lPanelOnTiming=0;

unsigned long lVccOnTiming=0;
unsigned long lVccToModDelayTiming=0;
unsigned long lModOnTiming=0;
unsigned long lModToBLonTiming=0;

#ifdef CONFIG_ENABLE_DEMURA
#ifdef CONFIG_ENABLE_DEMURA_DLG
extern bool panel_dlg_already_read;
extern int panel_dlg_read_ret;
extern struct panel_dlg_info panel_dlg_info;
#endif
#endif

//--------------------------------------------------------------------------------------------------
// Local Variables
//--------------------------------------------------------------------------------------------------
static PanelType panelpara;
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
static st_multi_cust_ic_info multi_cust_ic;
static st_cust_tcon_info tcon_info;
#endif

#ifdef CONFIG_ENABLE_DEMURA
static DemuraImpl_Panel_Data PnlData = {0, 0, true, false, true, false, E_DEMURA_BIN_ORDER_NO_DLG};
#ifdef CONFIG_ENABLE_DEMURA_DLG
static DemuraImpl_Panel_Data PnlData2 = {0, 0, true, false, false, false, E_DEMURA_BIN_ORDER_NO_DLG};
#endif
#ifdef CONFIG_DEMURA_VENDOR_MULTI
static Demura_Panel_Vendor genDemuraVendorSelect;
#endif /* #ifdef CONFIG_DEMURA_VENDOR_MULTI */
#endif /* #ifdef CONFIG_ENABLE_DEMURA */

static MS_BOOL bPanleReady=FALSE;
static st_pwm_setting pwm_setting;
CurrentPanelType eCurPanelType = PANEL_TYPE_FULLHD_CMO216_H1L01;
static st_ldm_setting ldm_setting;

static void mtk_panel_ldm_display_enable(bool en)
{
    struct udevice *dev;
    struct display_timing timing;
    int ret = 0;

    UBOOT_DEBUG("[mtk_panel_ldm_display_enable] en = %d\n",en);
    if (en)
        timing.flags = DISPLAY_FLAGS_DE_HIGH;
    else
        timing.flags = DISPLAY_FLAGS_DE_LOW;

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "mediatek-ldm", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("LDM is not enabled\n");
    }
}



static void SetPWM(unsigned int Period,unsigned int Duty,unsigned short PwmDiv,PWM_ChNum PWMPort,unsigned short bPolPWM)
{
    PWM_Result result = E_PWM_FAIL;
    struct udevice *dev;

    uclass_get_device(UCLASS_PWM, 0, &dev);

    if (!dev)
        return;
    result = pwm_set_enable(dev, PWMPort, 1);
    if(result == E_PWM_FAIL)
    {
        UBOOT_DEBUG("Sorry , call pwm_set_enable() failed!~\n");
    }

    UBOOT_DEBUG("\nPeriod = %d Duty:%d\n",Period,Duty);
    result = pwm_set_config(dev, PWMPort, Period, Duty);
    if(result == E_PWM_FAIL)
    {
        UBOOT_DEBUG("Sorry , call pwm_set_config() failed!~\n");
    }

    UBOOT_DEBUG("\nbPolPWM = %d\n",bPolPWM);
    if(bPolPWM == 1)
        pwm_set_invert(dev, PWMPort, 1);
    else
        pwm_set_invert(dev, PWMPort, 0);
}

static int PWM_init(bool onoff)
{
    unsigned int u32PWMDuty = 0;
    unsigned int u32PWMPeriod = 0;
    unsigned short u32PWMDIV = 0;
    unsigned int u32maxPWM =0;
    unsigned int u32minPWM =0;
    PWM_ChNum PWMPort = 0;
    int ret = 0;
    struct panel_pwm_duty_info pnl_pwm_duty_info;

    ret = parse_dt("/video_out",pwm_dt_parser,(void*)&pwm_setting,NULL);
    if(ret < 0)
    {
        UBOOT_ERROR("Error: pwm information parse error in DTS\n");
        return -1;
    }
    u32PWMDIV = pwm_setting.m_u16DivPWM;
    u32PWMPeriod = pwm_setting.m_u32PWMPeriod;
    u32maxPWM = pwm_setting.m_u32maxPWM;
    u32minPWM = pwm_setting.m_u32minPWM;

    PWMPort = (PWM_ChNum)pwm_setting.m_u16PWMPort;
    if(PWMPort < 0 || PWMPort >9)
    {
        UBOOT_DEBUG("PWM CH out of range ,So use default CH2\n");
        PWMPort = (PWM_ChNum)E_PWM_CH2;
    }

    memset(&pnl_pwm_duty_info, 0, sizeof(struct panel_pwm_duty_info));
    ret = get_panel_pwm_duty_info(PANEL_PWM_DUTY_PARTITION_PATH, PANEL_PWM_DUTY_INI_PATH, &pnl_pwm_duty_info);
    if(ret < 0)
    {
        UBOOT_DEBUG("pwm_duty parse error in ini, use dts setting.\n");
    }
    if(pnl_pwm_duty_info.pwm_duty != PWM_DUTY_INVALID)
    {
        u32PWMDuty = pnl_pwm_duty_info.pwm_duty;
        UBOOT_DEBUG("u32PWMDuty_T in ini = 0x%x\n",pnl_pwm_duty_info.pwm_duty);
    }
    else
    {
        if(pwm_setting.m_u32PWMDuty > 0)
        {
            u32PWMDuty = pwm_setting.m_u32PWMDuty;
        }
        else
        {
            u32PWMDuty = ((u32maxPWM+u32minPWM) >> 1); // set to 50% duty.
        }
    }

    UBOOT_DEBUG("u32PWMPeriod_T = 0x%x\n",u32PWMPeriod);
    UBOOT_DEBUG("u32PWMDuty_T = 0x%x\n",u32PWMDuty);
    UBOOT_DEBUG("u32PWMDIV_T = 0x%x\n",u32PWMDIV);
    UBOOT_DEBUG("u32maxPWM_T = 0x%x\n",u32maxPWM);
    UBOOT_DEBUG("u32minPWM_T = 0x%x\n",u32minPWM);

    if (onoff)
    {
        SetPWM(u32PWMPeriod, u32PWMDuty, u32PWMDIV, PWMPort, pwm_setting.m_bPolPWM);
    }
    else if(pwm_setting.m_bPolPWM)
    {
        SetPWM(0, 0, 0, PWMPort, 0);
    }

    return 0;
}

unsigned int mtk_get_current_panel_width(void)
{
    return panelpara.m_wPanelWidth;
}

unsigned int mtk_get_current_panel_height(void)
{
	unsigned int ret = 0;

#ifdef CONFIG_ENABLE_DEMURA
#ifdef CONFIG_ENABLE_DEMURA_DLG
	if (!panel_dlg_already_read)
	{
		panel_dlg_read_ret = get_panel_dlg_info(PANEL_DLG_PARTITION_PATH, PANEL_DLG_INI_PATH, &panel_dlg_info);
		panel_dlg_already_read = true;
	}

	if ((panel_dlg_read_ret >= 0) && (panel_dlg_info.panel_dlg == 1))
		ret = PnlData2.u16PanelHeight; //DLG on
	else
		ret = PnlData.u16PanelHeight; //DLG off
#else
		ret = PnlData.u16PanelHeight; //No DLG
#endif

#else
	ret = panelpara.m_wPanelHeight; //No Demura
#endif
	return ret;
}

unsigned int mtk_get_current_panel_hstart(void)
{
    return panelpara.m_wPanelHStart;
}

int mtk_get_panel_bl_pwm_cusctrl(void)
{
    return panelpara.m_bBlPWMCusCtrl;
}

int mtk_is_panel_ready(void)
{
    if(bPanleReady==TRUE)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

void mtk_panel_backlight_off(void)
{
    PWM_init(0);
    mtk_panel_ldm_display_enable(0);
}

void mtk_panel_backlight_on(void)
{
    UBOOT_TRACE("IN\n");

    unsigned long lpanelDelayTime=0;
    unsigned long ltotalTime=get_timer(0);
    int ret;
    st_sys_misc_setting misc_data;
    memset(&misc_data, 0, sizeof(misc_data));
    ret = parse_dt("/video_out",misc_dt_parser,(void*)&misc_data,NULL);
    if(ret < 0)
    {
        UBOOT_ERROR("Error: misc information parse error in DTS\n");
    }

    if(misc_data.m_wPanelOnTiming3>0)
    {
        UBOOT_DEBUG("Delay PanelOnTiming3 0x%x ms between PWM_EN BL_EN \n",misc_data.m_wPanelOnTiming3);
        mdelay(misc_data.m_wPanelOnTiming3);
    }
    else
    {
        UBOOT_DEBUG("NO PanelOnTiming3 Delay .... \n");
    }

    if (lModToBLonTiming>(ltotalTime-lModOnTiming)) //T2
    {
        lpanelDelayTime=(lModToBLonTiming-(ltotalTime-lModOnTiming));
        UBOOT_DEBUG("\n---%s:%d Set MsDrv_PNL_BackLigth_On DelayTask %lu \n", __FUNCTION__, __LINE__,lpanelDelayTime);
        mdelay(lpanelDelayTime);
    }
    else
    {
        UBOOT_DEBUG("\n---%s:%d Set MsDrv_PNL_BackLigth_On No Delay \n", __FUNCTION__, __LINE__);
    }
    if (!mtk_get_panel_bl_pwm_cusctrl())
    {
        PWM_init(1);//PWM init
    }
    if (pwm_setting.m_PwmToBlDelay) {//T3
        UBOOT_DEBUG("Delay 0x%x ms between PWM_EN BL_EN \n",pwm_setting.m_PwmToBlDelay);
        mdelay(pwm_setting.m_PwmToBlDelay);
    } else {
        UBOOT_DEBUG("NO PWM to BL Delay .... \n");
    }
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    mtk_pnl_cust_settings_ontiming2_backlight(&multi_cust_ic);
#endif
	if (!mtk_get_panel_gpio_cusctrl())
	{
	    mtk_panel_enable_backlight(TRUE);
	}
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    mtk_pnl_cust_settings_after_backlight(&multi_cust_ic);
#endif
    mtk_panel_ldm_display_enable(1);
    UBOOT_TRACE("OK\n");
}

int mtk_panel_check_DLG_supported_version(int version)
{
	int ret = 0;
    st_cust_dmc_info st_cust_dmc_info;
    memset(&st_cust_dmc_info, 0x00, sizeof(st_cust_dmc_info));

    parse_dt("/video_out", cus_demura_dt_parser, (void*)&st_cust_dmc_info, NULL);
    UBOOT_TRACE("dmc dlg is %s.\n", st_cust_dmc_info.dmc_dlg_enable ? "enable" : "disable");
    if (st_cust_dmc_info.dmc_dlg_enable)
    {
        ret = (version == BOOT_PNL_VERSION0200) |
              (version == BOOT_PNL_VERSION0203) |
              (version == BOOT_PNL_VERSION0400) |
              (version == BOOT_PNL_VERSION0500) |
              (version == BOOT_PNL_VERSION0600);
    }
	return ret;
}

int mtk_set_panel_vcc_cusctrl(MS_BOOL vcc_bl_cusctrl)
{
	struct udevice *dev;
	struct gpio_desc gpio_vcc;
	int ret = 0;
	ret = uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);

	if (!ret) {
		gpio_request_by_name(dev, "vcc-gpios", 0, &gpio_vcc, GPIOD_IS_OUT);
	} else {
		UBOOT_ERROR("cannot get vcc-gpios from dtso");
		return -1;
	}

    if ((pm_check_back_ground_active() == 0) ||
        (panelpara.m_bqhb_bypass_en == 1)) {
        //enable VCC gpio
        if (vcc_bl_cusctrl != 1) {
            //get VCC/BL gpio
            ret = dm_gpio_set_value(&gpio_vcc, 1);
            if (ret != 0)
                UBOOT_ERROR("get gpio VCC value = %d\n",
                dm_gpio_get_value(&gpio_vcc));
        }
    } else {
        if (vcc_bl_cusctrl != 1) {
            //get VCC/BL gpio
            ret = dm_gpio_set_value(&gpio_vcc, 0);
            if (ret != 0)
                UBOOT_ERROR("get gpio VCC value = %d\n",
                dm_gpio_get_value(&gpio_vcc));
        }
        UBOOT_INFO("QHB case, vcc return\n");
    }
    return 0;
}

int mtk_get_panel_gpio_cusctrl(void)
{
	return panelpara.m_bVccBlCusCtrl;
}

#ifdef CONFIG_MTK_PANEL
int mtk_panel_init_device(void)
{
    UBOOT_TRACE("IN\n");
    struct udevice *dev;
    int ret;
	int ret_ldm = 0;
#ifdef CONFIG_ENABLE_DEMURA
    int nPropVal = 0;
#endif
	unsigned int pnl_lib_version = 0xFFFFFFFF;

    ret = parse_dt("/video_out/panel_info",panel_dt_parser,(void*)&panelpara,NULL);
    if(ret < 0)
    {
        UBOOT_ERROR("Error: panel information parse error in DTS\n");
        return -1;
    }
    ret = parse_dt("/video_out",cus_panel_dt_parser,(void*)&panelpara,NULL);
    if(ret < 0)
    {
        UBOOT_ERROR("Error: cus panel information parse error in DTS\n");
        return -1;
    }
    ret_ldm = parse_dt("/mediatek-ldm",ldm_dt_parser,(void*)&ldm_setting,NULL);
    if(ret_ldm < 0)
    {
        UBOOT_ERROR("Error: ldm information parse error in DTS\n");
    }
	if (parse_dt("/video_out/", integer_dt_parser,
        (void *)&pnl_lib_version, "pnl_lib_version") < 0)
    {
        pnl_lib_version = 0xFFFFFFFF;
    }

#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    parse_dt("/video_out", cust_pmic_dt_parser, (void*)&multi_cust_ic, NULL);
    parse_dt("/video_out", cust_pgamma_dt_parser, (void*)&multi_cust_ic, NULL);
    parse_dt("/video_out", cust_levelshift_dt_parser, (void*)&multi_cust_ic, NULL);
    parse_dt("/video_out", cust_vcomic_dt_parser, (void*)&multi_cust_ic, NULL);

    parse_dt("/video_out", cust_pmic_sub_dt_parser, (void*)&multi_cust_ic, NULL);
    parse_dt("/video_out", cust_pgamma_sub_dt_parser, (void*)&multi_cust_ic, NULL);
    parse_dt("/video_out", cust_levelshift_sub_dt_parser,(void*)&multi_cust_ic, NULL);
    parse_dt("/video_out", cust_vcomic_sub_dt_parser, (void*)&multi_cust_ic, NULL);

    parse_dt("/video_out", cust_tcon_dt_parser, (void*)&tcon_info,NULL);
#endif

    UBOOT_DEBUG("[mtk_panel_init_device] Get Diplay device from dts\n");
    lVccToModDelayTiming = panelpara.m_wPanelOnTiming1;
    lModToBLonTiming = panelpara.m_wPanelOnTiming2;

#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    mtk_pnl_cust_settings_befor_vcc(&multi_cust_ic);
#endif
	mtk_set_panel_vcc_cusctrl(panelpara.m_bVccBlCusCtrl);

    uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);
    uclass_get_device_by_name(UCLASS_DISPLAY, "ext_video_out", &dev);
    uclass_get_device_by_name(UCLASS_DISPLAY, "graphic_out", &dev);

    uclass_get_device_by_name(UCLASS_DISPLAY, "mediatek-ldm", &dev);

    lVccOnTiming=get_timer(0);

#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);
    if (!ret)
    {
        if (display_get_property(dev, E_PNL_PROP_VCOM_SEL, &nPropVal) == 0)
        {
            UBOOT_TRACE("Vcom sel=%d -> %d\n", multi_cust_ic.pmic_vcom_info.vcom_sel, nPropVal);
            multi_cust_ic.pmic_vcom_info.vcom_sel = nPropVal;
        }

        if (display_get_property(dev, E_PNL_PROP_TCON_EN, &nPropVal) == 0)
        {
            PnlData.btcon = nPropVal;
        }
    }
    else
        UBOOT_ERROR("Get video out device is fail\n");
    if (PnlData.btcon == TRUE)
        mtk_pnl_cust_settings_vcc_ontiming1(&multi_cust_ic);
    mtk_pnl_cust_set_panel_mode(&tcon_info);
#endif

#ifdef CONFIG_ENABLE_DEMURA
#ifdef CONFIG_ENABLE_DEMURA_DLG
    PnlData2.u16PanelWidth = panelpara.m_wPanelWidth;
    PnlData2.u16PanelHeight = panelpara.m_wPanelHeight>>1;
#endif
    PnlData.u16PanelWidth = panelpara.m_wPanelWidth;
    PnlData.u16PanelHeight = panelpara.m_wPanelHeight;
    UBOOT_TRACE("%d sel=%d\n",__LINE__, genDemuraVendorSelect);

    //get tcon bin demura
    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);
    if (!ret)
    {
        if (display_get_property(dev, E_PNL_PROP_TCON_EN, &nPropVal) == 0)
        {
            PnlData.btcon = nPropVal;
        }
        if (display_get_property(dev, E_PNL_PROP_DEMURA_SEL, &nPropVal) == 0)
        {
            UBOOT_TRACE("Demura select=%d -> %d\n", genDemuraVendorSelect, nPropVal);
            genDemuraVendorSelect = nPropVal;
        }
        if (display_get_property(dev, E_PNL_PROP_DEMURA_EN, &nPropVal) == 0)
        {
            UBOOT_TRACE("Demura en=%d -> %d\n", PnlData.bon, nPropVal);
            PnlData.bon = nPropVal;
        }
        if (display_get_property(dev, E_PNL_PROP_STI_ARCH, &nPropVal) == 0)
        {
            UBOOT_TRACE("STI arch=%d -> %d\n", PnlData.bsti, nPropVal);
            PnlData.bsti = nPropVal;
            PnlData2.bsti = nPropVal;
        }
    }
    else
        UBOOT_ERROR("Get video out device is fail\n");

#ifdef DEMURA_EFUSE_CHECK //SW efuse mode
	MS_BOOL efuse;

	efuse = MDrv_SYS_Query(E_SYS_QUERY_47);
	printf("[%s]efuse=%d\n", __func__, efuse);
	if (efuse) {
#ifdef CONFIG_DEMURA_VENDOR_MULTI
#ifdef CONFIG_ENABLE_DEMURA_DLG
		if (mtk_panel_check_DLG_supported_version(pnl_lib_version)) {
		#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
			if ((tcon_info.bVRR_HighFrameRateMode_Support)
				&& (mtk_pnl_is_dlg_mode() == TRUE)) {
				UBOOT_TRACE("[%d]Demura DLG mode ON \n", __LINE__);
				PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_FIRST;
				mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
				PnlData2.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_LAST;
				mtk_demura_init(PnlData2, genDemuraVendorSelect, E_DEMURA_BIN_DLG);
			} else {
				UBOOT_TRACE("[%d]Demura DLG mode OFF \n", __LINE__);
				PnlData2.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_FIRST;
				mtk_demura_init(PnlData2, genDemuraVendorSelect, E_DEMURA_BIN_DLG);
				PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_LAST;
				mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
			}
		#else
            UBOOT_TRACE("[%d]Demura DLG mode OFF \n", __LINE__);
            PnlData2.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_FIRST;
			mtk_demura_init(PnlData2, genDemuraVendorSelect, E_DEMURA_BIN_DLG);
			PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_LAST;
			mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
		#endif
		} else {
			PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_NO_DLG;
			mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
		}
#else
		PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_NO_DLG;
		mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
#endif

#else
		PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_NO_DLG;
		MApi_Demura_Init(PnlData);
#endif
    }
#else /* #else DEMURA_EFUSE_CHECK */
#ifdef CONFIG_DEMURA_VENDOR_MULTI
#ifdef CONFIG_ENABLE_DEMURA_DLG
	if (mtk_panel_check_DLG_supported_version(pnl_lib_version)) {
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
		if ((tcon_info.bVRR_HighFrameRateMode_Support) &&
			(mtk_pnl_is_dlg_mode() == TRUE)) {
			UBOOT_TRACE("[%d]Demura DLG mode ON \n", __LINE__);
			PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_FIRST;
			mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
			PnlData2.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_LAST;
			mtk_demura_init(PnlData2, genDemuraVendorSelect, E_DEMURA_BIN_DLG);
		} else {
			UBOOT_TRACE("[%d]Demura DLG mode OFF \n", __LINE__);
			PnlData2.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_FIRST;
			mtk_demura_init(PnlData2, genDemuraVendorSelect, E_DEMURA_BIN_DLG);
			PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_LAST;
			mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
		}
#else
        UBOOT_TRACE("[%d]Demura DLG mode OFF \n", __LINE__);
		PnlData2.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_FIRST;
		mtk_demura_init(PnlData2, genDemuraVendorSelect, E_DEMURA_BIN_DLG);
		PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_LAST;
		mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
#endif
	} else {
		PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_NO_DLG;
		mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
	}
#else
	PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_NO_DLG;
	mtk_demura_init(PnlData, genDemuraVendorSelect, E_DEMURA_BIN_MAIN);
#endif

#else
	PnlData.u8DemuraBinOrder = E_DEMURA_BIN_ORDER_NO_DLG;
	mtk_demura_init(PnlData);
#endif

#endif /* #ifdef DEMURA_EFUSE_CHECK */
#endif /* #ifdef CONFIG_ENABLE_DEMURA */

    bPanleReady=TRUE;
    UBOOT_TRACE("OK\n");
    return 0;
}

int mtk_panel_enable(bool en)
{
    struct udevice *dev;
    int ret = 0;
    unsigned long lCurrentTimer = 0;
    unsigned long lModOnDelayTime = 0;
    bool bqhq_en = 0;

    //struct dm_display_ops *ops = display_get_ops(dev);
    struct display_timing timing;
    UBOOT_DEBUG("[mtk_panel_enable] en = %d\n",en);
    lCurrentTimer = get_timer(0);
    if ((lCurrentTimer - lVccOnTiming) < lVccToModDelayTiming){
        lModOnDelayTime = (lVccToModDelayTiming - (lCurrentTimer - lVccOnTiming));
        mdelay(lModOnDelayTime);
        UBOOT_DEBUG("\n---%s:%d Set mtk_panel_enable DelayTask %lu \n", __FUNCTION__, __LINE__,lModOnDelayTime);
    }
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    mtk_pnl_cust_settings_ontiming1_data(&multi_cust_ic);
#endif
    if (en)
    {
        timing.flags = 1<<5; //set active high bit
    }
    else
    {
        timing.flags = 1<<4; //set active low bit
    }

    UBOOT_DEBUG("[%s][%d] panelpara.m_bqhb_bypass_en=%d\n", __FUNCTION__, __LINE__, panelpara.m_bqhb_bypass_en);

    if ((pm_check_back_ground_active() == 1) &&
        (panelpara.m_bqhb_bypass_en == 0))
    {
        UBOOT_INFO("QHB case, do_panel_output_enable return\n");
        bqhq_en = 1;
    }
    else
    {
        UBOOT_DEBUG("NOT QHB case, do_panel_output_enable continuous\n");
    }

    if (bqhq_en)
    {
        return 0;
    }

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("Video_out path is not enabled\n");
    }

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "ext_video_out", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("Ext_video_out path is not enabled\n");
    }

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "graphic_out", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("Graphic_out path is not enabled\n");
    }

    lModOnTiming = get_timer(0);
#ifdef CONFIG_ENABLE_CUST_IC_UPDATE
    mtk_pnl_cust_settings_data_ontiming2(&multi_cust_ic);
#endif
    return 0;
}

int mtk_panel_enable_backlight(bool en)
{
    struct udevice *dev;
    struct gpio_desc gpio_backlight;
    int ret = 0;
    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);

    if (!ret) {
        gpio_request_by_name(dev, "backlight-gpios", 0, &gpio_backlight, GPIOD_IS_OUT);

        UBOOT_DEBUG("en = %d , BL gpio flag = 0x%lx\n",en, gpio_backlight.flags);

        if (en == TRUE) {
            dm_gpio_set_value(&gpio_backlight, 1);
        } else {
            dm_gpio_set_value(&gpio_backlight, 0);
        }
    } else {
            UBOOT_DEBUG("Video_out path is not enabled\n");
    }

    return 0;
}

int mtk_panel_mute(bool en)
{
    struct udevice *dev;
    int ret = 0;
    #define MUTE_ENABLE_BIT 3
    #define MUTE_DISABLE_BIT 2
    bool bqhq_en = 0;

    //struct dm_display_ops *ops = display_get_ops(dev);
    struct display_timing timing;
    UBOOT_DEBUG("[mtk_panel_mute] en = %d\n",en);

    if (en)
    {
        timing.flags = 1<<MUTE_ENABLE_BIT; //set active high bit
    }
    else
    {
        timing.flags = 1<<MUTE_DISABLE_BIT; //set active low bit
    }

    if ((pm_check_back_ground_active() == 1) &&
        (panelpara.m_bqhb_bypass_en == 0))
    {
        UBOOT_INFO("QHB case, do_panel_mute return\n");
        bqhq_en = 1;
    }
    else
    {
        UBOOT_DEBUG("NOT QHB case, do_panel_mute continuous\n");
    }

    if (bqhq_en)
    {
        return 0;
    }

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "video_out", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("Video_out path is not enabled\n");
    }

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "ext_video_out", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("Ext_video_out path is not enabled\n");
    }

    ret = uclass_get_device_by_name(UCLASS_DISPLAY, "graphic_out", &dev);
    if (!ret) {
        display_enable(dev, 0, &timing);
    } else {
        UBOOT_DEBUG("Graphic_out path is not enabled\n");
    }

    return 0;
}

#endif
