// SPDX-License-Identifier: (GPL-2.0 OR BSD-3-Clause)
/*
 * Copyright (c) 2021 MediaTek Inc.
 */

#include <common.h>
#include <audio_codec.h>
#include <div64.h>
#include <dm.h>
#include <i2c.h>
#include <i2s.h>
#include <sound.h>
#include <asm/gpio.h>

#include "mtk_acm8625.h"

u8 ampinittblacm8625[] = {
//DataLen	Address	DataN	DataN+1...
	/* don't need to do sw reset here, hw reset is applied */
	1,	0x00,	0x00,
	1,	0x04,	0x00,
	1,	0xfc,	0x86,
	1,	0xfd,	0x25,
	1,	0xfe,	0x15,
	1,	0x00,	0x01,
	1,	0x02,	0x20,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	0x00,
};

u8 ampinittbldspcfg[] = {
//DataLen	Address	DataN	DataN+1...
	1,	0x00,	0x00,
	1,	0x04,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x0b,
	1,	0x5c,	0x80,
	1,	0x5d,	0x9e,
	1,	0x5e,	0x02,
	1,	0x5f,	0x9e,
	1,	0x60,	0x80,
	1,	0x61,	0x9e,
	1,	0x62,	0x03,
	1,	0x63,	0x9e,
	1,	0x00,	0x04,
	1,	0x88,	0x00,
	1,	0x89,	0xe2,
	1,	0x8a,	0xc4,
	1,	0x8b,	0x6b,
	1,	0x00,	0x01,
	1,	0x01,	0x00,
	1,	0x00,	0x00,
	1,	0x11,	0x03,
	1,	0x02,	0x00,
	1,	0x06,	0x30,
	1,	0x28,	0x00,
	1,	0x05,	0xc6,
	1,	0x03,	0x02,
	1,	0x01,	0x84,
	1,	0x00,	0x00,
	1,	0x04,	0x02,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x00,
	1,	0x00,	0x03,
	1,	0x13,	0x1d,
	1,	0x00,	0x00,
	1,	0x04,	0x03,
	0x00,
};

u8 ampvol[] = {
	1, 0x00, 0x04,
	4, 0x7c, 0x00, 0x80, 0x00, 0x00,
	4, 0x80, 0x00, 0x80, 0x00, 0x00,
	1, 0x00, 0x00,
	0x00,
};

u8 ampplay[] = {
	1, 0x04, 0x03,
	0x00,
};

static int mtk_amp_i2c_write(struct udevice *dev, u8 reg, u8 *buf, u8 size)
{
	struct amp_acm8625_priv *priv = dev_get_priv(dev);
	struct i2c_msg msgs;
	u8 msg_buf[10];
	int ret;
	int i;

	msg_buf[0] = reg;

	for (i = 1; i <= size; i++)
		msg_buf[i] = buf[i - 1];

	msgs.addr  = priv->i2c_addr;
	msgs.len   = size + 1;
	msgs.buf   = msg_buf;
	msgs.flags = I2C_M_STOP;

	ret = dm_i2c_xfer(dev, &msgs, 1);
	if (ret < 0)
		return ret;

	return 0;
}

static int amp_write(struct udevice *dev, unsigned char *inittable)
{
	u8 *pu8str = inittable;
	u8 *buf;
	u8 reg;
	u8 size;
	int ret;

	do {
		size = *pu8str;
		pu8str++;

		reg = *pu8str;
		pu8str++;

		buf = pu8str;
		ret = mtk_amp_i2c_write(dev, reg, buf, size);
		if (ret < 0) {
			dev_err(dev, "[AMP 8625]i2c write fail %d\n", ret);
			return -1;
		}

		pu8str = pu8str + size;
	} while (*pu8str != 0);

	return 0;
}

static int amp_acm8625_powerdown(struct udevice *dev, bool enable)
{
	struct amp_acm8625_priv *priv = dev_get_priv(dev);

	if (dm_gpio_is_valid(&priv->amp_powerdown))
		dm_gpio_set_value(&priv->amp_powerdown, enable);

	return 0;
}

static int amp_acm8625_set_params(struct udevice *dev, int interface, int rate, int mclk_freq,
				  int bits_per_sample, uint channels)
{
	int ret;

	//pull high amp PD pin
	amp_acm8625_powerdown(dev, 0);
	//wait t9 min time = 20ms then we can start I2C command
	mdelay(2);

	//normal operation
	ret = amp_write(dev, ampinittblacm8625);
	if (ret < 0) {
		dev_err(dev, "[AMP 8625]Unable to init AMP ACM8625\n");
		return -EINVAL;
	}
	mdelay(10);
	//set all channel mute during writing all of registers and RAM
	ret = amp_write(dev, ampinittbldspcfg);
	if (ret < 0) {
		dev_err(dev, "[AMP 8625]Unable to init AMP ACM8625\n");
		return -EINVAL;
	}

	//set all channel un-mute
	amp_write(dev, ampvol);
	amp_write(dev, ampplay);

	return 0;
}

static int amp_acm8625_probe(struct udevice *dev)
{
	struct amp_acm8625_priv *priv = dev_get_priv(dev);
	int ret;

	if (dev_read_enabled(dev) == 0)
		return 0;

	if (device_get_uclass_id(dev->parent) != UCLASS_I2C) {
		dev_err(dev, "[AMP 8625]get i2c parent fail\n");
		return -EPROTONOSUPPORT;
	}

	ret = dev_read_u32(dev, "reg", &priv->i2c_addr);
	if (ret) {
		dev_err(dev, "[AMP 8625]can't get slave addr\n");
		return -EINVAL;
	}

	ret = gpio_request_by_name(dev, "amppowerdown-gpios", 0, &priv->amp_powerdown, GPIOD_IS_OUT);
	if (ret) {
		dev_err(dev, "[AMP 8625]get amp power down gpio fail %d\n", ret);
		return -EINVAL;
	}

	return 0;
}

static const struct audio_codec_ops amp_acm8625_ops = {
	.set_params	= amp_acm8625_set_params,
};

static const struct udevice_id amp_acm8625_ids[] = {
	{ .compatible = "ACME, acm8625" },
	{ }
};

U_BOOT_DRIVER(acm8625) = {
	.name		= "acm8625",
	.id		= UCLASS_AUDIO_CODEC,
	.of_match	= amp_acm8625_ids,
	.probe		= amp_acm8625_probe,
	.ops		= &amp_acm8625_ops,
	.priv_auto_alloc_size = sizeof(struct amp_acm8625_priv),
};
