// SPDX-License-Identifier: (GPL-2.0 OR BSD-3-Clause)
/*
 * MediaTek Ldm driver
 *
 * Copyright (c) 2020 MediaTek Inc.
 * Author Sniper.Huang <sniper.huang@mediatek.com>
 */
//#define DEBUG //for open this file debug("xxx") log
#include <linux/delay.h>
#include "mtk_tv_ldm.h"
//#include "mtk_pnl_dts_st.h"
//#include "mtk_pnl_out_if.h"
//#include "mtk_pnl_clk_ctrl.h"
#include "mtk_pnl_utility.h"
#include <common.h>
#include <backlight.h>
#include <dm.h>
#include <panel.h>
#include <asm/gpio.h>
#include <power/regulator.h>
#include <display.h>
#include <dm/pinctrl.h>
#include <debug_impl.h>
#include "coda/LDC_BKA3EF.h"
#include "coda/LD1_BKA417.h"
#include <asm/io.h>
#include <spi.h>
#define SPI_XFER_BEGIN		BIT(0)	/* Assert CS before transfer */
#define SPI_XFER_END		BIT(1)	/* Deassert CS after transfer */
#define LDM_CHAR_NAME		(30)
#define BIT0			(0x0001)
#define LDM_3BITS		(3UL)
#define LDM_6BITS		(6UL)
#define LDM_7BITS		(7UL)
#define LDM_8BITS		(8UL)
#define LDM_9BITS		(9UL)
#define LDM_10BITS		(10UL)
#define LDM_CH_0		(0UL)
#define LDM_CH_1		(1UL)
#define LDM_CH_2		(2UL)
#define LDM_CH_3		(3UL)
#define LDM_CH_4		(4UL)
#define LDM_CH_5		(5UL)
#define LDM_CH_6		(6UL)
#define LDM_CH_7		(7UL)
#define LDM_CH_MAX		(8UL)
#define LDM_MASK		(0xFF)
#define LDM_BYTES_0		(0)
#define LDM_BYTES_1		(1)
#define LDM_BYTES_2		(2)
#define LDM_BYTES_3		(3)
#define LDM_BYTES_4		(4)
#define LDM_BYTES_5		(5)
#define LDM_BYTES_6		(6)
#define LDM_BYTES_7		(7)
#define LDM_BYTES_8		(8)
#define LDM_SPI_CLK		(1000000)
#define LDM_SPI_MODE	(0)
#define LDM_SPI_ADDR	(0x640800)//(0x1C640800)
#define LDM_SPI_OFFSET	(0x200)
#define LDM_SPI_REG_080	(0x080)
#define LDM_SPI_REG_084	(0x084)
#define LDM_SPI_REG_088	(0x088)
#define LDM_SPI_REG_08C	(0x08C)
#define LDM_SPI_REG_090	(0x090)
#define LDM_SPI_REG_094	(0x094)
#define LDM_SPI_REG_0A0	(0x0A0)
#define LDM_SPI_REG_0A4	(0x0A4)
#define LDM_SPI_REG_0A8	(0x0A8)
#define LDM_SPI_REG_0AC	(0x0AC)
#define LDM_SPI_REG_0B0	(0x0B0)
#define LDM_SPI_REG_0B4	(0x0B4)
#define LDM_SPI_REG_0E0	(0x0E0)
#define LDM_SPI_REG_130	(0x130)
#define LDM_SPI_REG_134	(0x134)
#define LDM_SPI_REG_138	(0x138)
#define LDM_SPI_REG_13C	(0x13C)

#define LDM_PWM_ADDR	(0x01483A00)
#define LDM_PWM_REG_004	(LDM_PWM_ADDR + 0x004)
#define LDM_PWM_REG_004_CLKEN_BITMAP 			(((uint32_t)7<<16)|(0<<8)|9)

#define LDM_PWM_REG_008	(LDM_PWM_ADDR + 0x008)
#define LDM_PWM_REG_0008_PWM0_VDBEN 	(((uint32_t)1<<16)|(9<<8)|10)
#define LDM_PWM_REG_0008_PWM0_RESET_EN 	(((uint32_t)1<<16)|(10<<8)|10)
#define LDM_PWM_REG_0008_PWM0_VDBEN_SW 	(((uint32_t)1<<16)|(14<<8)|10)

#define LDM_PWM_REG_00C	(LDM_PWM_ADDR + 0x00C)
#define LDM_PWM_REG_000C_PWM1_VDBEN 	(((uint32_t)1<<16)|(9<<8)|10)
#define LDM_PWM_REG_000C_PWM1_RESET_EN 	(((uint32_t)1<<16)|(10<<8)|10)
#define LDM_PWM_REG_000C_PWM1_VDBEN_SW 	(((uint32_t)1<<16)|(14<<8)|10)

#define LDM_PWM_REG_0A0 (LDM_PWM_ADDR + 0x0A0)
#define LDM_PWM_REG_0A0_VSYNC_LIKE_ST_0	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_0A4 (LDM_PWM_ADDR + 0x0A4)
#define LDM_PWM_REG_0A4_VSYNC_LIKE_ST_1	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_0A8 (LDM_PWM_ADDR + 0x0A8)
#define LDM_PWM_REG_0A8_VSYNC_LIKE_END_0	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_0AC (LDM_PWM_ADDR + 0x0AC)
#define LDM_PWM_REG_0AC_VSYNC_LIKE_END_1	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_0B0	(LDM_PWM_ADDR + 0x0B0)
#define LDM_PWM_REG_0B0_VSYNC_LIKE_SRC_SEL		(((uint32_t)1<<16)|(0<<8)|9)
#define LDM_PWM_REG_0B0_VSYNC_LIKE_SRC_SEL_V2	(((uint32_t)2<<16)|(0<<8)|9)

#define LDM_CLKEN_BITMAP		(0x07)

#define LDM_PWM_REG_1C8 (LDM_PWM_ADDR + 0x1C8)
#define LDM_PWM_REG_1C8_PWM0_PERIOD_0 	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_1CC (LDM_PWM_ADDR + 0x1CC)
#define LDM_PWM_REG_1CC_PWM0_PERIOD_1 	(((uint32_t)8<<16)|(0<<8)|1)
#define LDM_PWM_REG_1D0 (LDM_PWM_ADDR + 0x1D0)
#define LDM_PWM_REG_1D0_PWM0_SHIFT_0 	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_1D4 (LDM_PWM_ADDR + 0x1D4)
#define LDM_PWM_REG_1D4_PWM0_SHIFT_1 	(((uint32_t)8<<16)|(0<<8)|1)
#define LDM_PWM_REG_1D8 (LDM_PWM_ADDR + 0x1D8)
#define LDM_PWM_REG_1D8_PWM0_DUTY_0 	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_1DC (LDM_PWM_ADDR + 0x1DC)
#define LDM_PWM_REG_1DC_PWM0_DUTY_1 	(((uint32_t)8<<16)|(0<<8)|1)

#define LDM_PWM_REG_1E8 (LDM_PWM_ADDR + 0x1E8)
#define LDM_PWM_REG_1E8_PWM1_PERIOD_0 	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_1EC (LDM_PWM_ADDR + 0x1EC)
#define LDM_PWM_REG_1EC_PWM1_PERIOD_1 	(((uint32_t)8<<16)|(0<<8)|1)
#define LDM_PWM_REG_1F0 (LDM_PWM_ADDR + 0x1F0)
#define LDM_PWM_REG_1F0_PWM1_SHIFT_0 	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_1F4 (LDM_PWM_ADDR + 0x1F4)
#define LDM_PWM_REG_1F4_PWM1_SHIFT_1 	(((uint32_t)8<<16)|(0<<8)|1)
#define LDM_PWM_REG_1F8 (LDM_PWM_ADDR + 0x1F8)
#define LDM_PWM_REG_1F8_PWM1_DUTY_0 	(((uint32_t)16<<16)|(0<<8)|5)
#define LDM_PWM_REG_1FC (LDM_PWM_ADDR + 0x1FC)
#define LDM_PWM_REG_1FC_PWM1_DUTY_1 	(((uint32_t)8<<16)|(0<<8)|1)

#define LDM_SPI_MASK    (((uint32_t)16<<16)|(0<<8)|5)
#define LDM_SPI_MASK_0  (((uint32_t)1<<16)|(0<<8)|9)
#define msleep(a) udelay(a * 1000) //macro

#define LDM_VER_1	(1)
#define LDM_VER_2	(2)
#define LDM_VER_3	(3)
#define LDM_VER_4	(4)
#define LDM_VER_5	(5)

#define RETRY_COUNTER		(200UL)

#define ONEDATA_SIZE 5

//static uint32_t au32LDMLedMSP8Bit[LDM_BYTES_8] =  {LDM_BYTES_7, LDM_BYTES_7, LDM_BYTES_7, LDM_BYTES_7, LDM_BYTES_7, LDM_BYTES_7, LDM_BYTES_7, LDM_BYTES_7};
//static uint32_t au32LDMLedMSP10Bit[LDM_BYTES_8] = {LDM_BYTES_1, LDM_BYTES_7, LDM_BYTES_1, LDM_BYTES_7, LDM_BYTES_1, LDM_BYTES_7, LDM_BYTES_1, LDM_BYTES_7};

static uint8_t LED_Device_AS3824_lock[]=	//Lock
{
	LDM_AS382X_BCAST_SINGLE_BYTE_SAME_DATA, LDM_AS382X_UNLOCK_ADDR, LDM_AS382X_LOCK_CMD,

	0x00,0x00,0x00,0x00,0x00, //Dummy*25
	0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,
};

static uint8_t LED_Device_AS3824_unlock[]=
{
	LDM_AS382X_BCAST_SINGLE_BYTE_SAME_DATA, LDM_AS382X_UNLOCK_ADDR, LDM_AS382X_UNLOCK_CMD,

	0x00,0x00,0x00,0x00,0x00, //Dummy*25
	0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,
};

static struct mtk_ldm_spi_large stMSPI;		// big size

/*
 *static int _vsync_like_gpio_set_state(struct udevice *dev, enum VSYNC_LIKE_GPIO state)
 *{
 *	struct mtk_ldm_priv *priv = dev_get_priv(dev);
 *	int ret;
 *
 *	if (!dm_gpio_is_valid(&priv->vsync_like_gpio))
 *		return -EREMOTEIO;
 *
 *	switch (state) {
 *		case E_LOW:
 *		case E_HIGH:
 *			break;
 *		case E_TOGGLE:
 *			ret = dm_gpio_get_value(&priv->vsync_like_gpio);
 *			if (ret < 0)
 *				return ret;
 *			state = !ret;
 *			break;
 *		default:
 *			return -ENOSYS;
 *			}
 *	return dm_gpio_set_value(&priv->vsync_like_gpio, state);
 *}
 */

static int _mtk_ldm_spi(struct mtk_ldm_spi *st_spi)
{
	struct spi_slave *slave;
	int ret = 0;
	unsigned long flags = SPI_XFER_BEGIN;
#ifdef CONFIG_DM_SPI
	char name[LDM_CHAR_NAME], *str;
	struct udevice *dev;
	ret = snprintf(name, sizeof(name), "generic_%d:%d", st_spi->u32ch, 0);
	if (ret < 0) {
		return ret;
	}
	str = strdup(name);
	if (!str) {
		return -ENOMEM;
	}
	ret = spi_get_bus_and_cs(st_spi->u32ch, 0, st_spi->u32speed, st_spi->u32mode, "spi_generic_drv", str, &dev, &slave);
	if (ret) {
		return ret;
	}
#else
	slave = spi_setup_slave(st_spi->u32ch, 0, st_spi->u32speed, st_spi->u32mode);
	if (!slave) {
		UBOOT_DEBUG("Invalid device %d:%d\n", st_spi->u32ch, 0);
		return -EINVAL;
	}
#endif
	ret = spi_claim_bus(slave);
	if (ret) {
		goto done;
	}
	if (st_spi->u32in_bit_len == 0)
		flags |= SPI_XFER_END;
	ret = spi_xfer(slave, st_spi->u32out_bit_len, &st_spi->u8out, NULL, flags);
	if (ret) {
		UBOOT_DEBUG("spi_xfer tx fail\n");
	} else if (st_spi->u32in_bit_len != 0) {
		ret = spi_xfer(slave, st_spi->u32in_bit_len, NULL, st_spi->u8in, SPI_XFER_END);
		if (ret) {
			UBOOT_DEBUG("spi_xfer rx fail\n");
		}
	}
#ifndef CONFIG_DM_SPI
	/* We don't get an error code in this case */
	if (ret) {
		ret = -EIO;
	}
#endif
	if (ret) {
		UBOOT_DEBUG("Error %d during SPI transaction\n", ret);
	} else {
		//int j;
		//for (j = 0; j < ((stSPI->u32in_bit_len + LDM_7BITS) / LDM_8BITS); j++) {
			//printf("%02X", stSPI->u8in[j]);
		//}
		//printf("\n");
	}
done:
	spi_release_bus(slave);
#ifndef CONFIG_DM_SPI
	spi_free_slave(slave);
#endif
	return ret;
}

/*
static void _mtk_ldm_set_spi_wd(uint8_t u8CH, uint32_t *pu32Data)
{
	uint16_t u16Data_1,u16Data_2;
	uint32_t u32Addr0 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_130;
	uint32_t u32Addr1 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_134;
	uint32_t u32Addr2 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_080;
	uint32_t u32Addr3 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_084;
	uint32_t u32Addr4 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_088;
	uint32_t u32Addr5 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_08C;
	uint32_t u32Addr6 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_090;
	uint32_t u32Addr7 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_094;

	u16Data_1 = (pu32Data[LDM_BYTES_3] << LDM_9BITS) | (pu32Data[LDM_BYTES_2] << LDM_6BITS) | (pu32Data[LDM_BYTES_1] << LDM_3BITS) | (pu32Data[LDM_BYTES_0]);
	u16Data_2 = (pu32Data[LDM_BYTES_7] << LDM_9BITS) | (pu32Data[LDM_BYTES_6] << LDM_6BITS) | (pu32Data[LDM_BYTES_5] << LDM_3BITS) | (pu32Data[LDM_BYTES_4]);

	W2BYTEMSK(u32Addr0, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr1, u16Data_2, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr2, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr3, u16Data_2, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr4, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr5, u16Data_2, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr6, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr7, u16Data_2, LDM_SPI_MASK);
}

static void _mtk_ldm_set_spi_rd(uint8_t u8CH, uint32_t *pu32Data)
{
	uint16_t u16Data_1,u16Data_2;
	uint32_t u32Addr0 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_138;
	uint32_t u32Addr1 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_13C;
	uint32_t u32Addr2 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_0A0;
	uint32_t u32Addr3 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_0A4;
	uint32_t u32Addr4 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_0A8;
	uint32_t u32Addr5 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_0AC;
	uint32_t u32Addr6 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_0B0;
	uint32_t u32Addr7 = LDM_SPI_ADDR + (LDM_SPI_OFFSET * u8CH) + LDM_SPI_REG_0B4;

	u16Data_1 = (pu32Data[LDM_BYTES_3] << LDM_9BITS) | (pu32Data[LDM_BYTES_2] << LDM_6BITS) | (pu32Data[LDM_BYTES_1] << LDM_3BITS) | (pu32Data[LDM_BYTES_0]);
	u16Data_2 = (pu32Data[LDM_BYTES_7] << LDM_9BITS) | (pu32Data[LDM_BYTES_6] << LDM_6BITS) | (pu32Data[LDM_BYTES_5] << LDM_3BITS) | (pu32Data[LDM_BYTES_4]);

	W2BYTEMSK(u32Addr0, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr1, u16Data_2, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr2, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr3, u16Data_2, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr4, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr5, u16Data_2, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr6, u16Data_1, LDM_SPI_MASK);
	W2BYTEMSK(u32Addr7, u16Data_2, LDM_SPI_MASK);
}
*/
static void _mtk_ldm_set_spi_Init(struct mtk_ldm_priv *priv, uint8_t u8CH)
{
	struct mtk_ldm_spi stMSPI;
	stMSPI.u32ch = u8CH;
	stMSPI.u32mode = LDM_SPI_MODE;
	stMSPI.u32speed = LDM_SPI_CLK;
	stMSPI.u32out_bit_len = 0;
	stMSPI.u32in_bit_len = 0;
	_mtk_ldm_spi(&stMSPI);
	//_mtk_ldm_set_spi_wd(u8CH, au32LDMLedMSP8Bit);
	//_mtk_ldm_set_spi_rd(u8CH, au32LDMLedMSP8Bit);
}

static int _mtk_ldm_spi_data(struct mtk_ldm_priv *priv,
	u8 u8CH, struct mtk_ldm_spi_data *pwdata, struct mtk_ldm_spi_data *prdata)
{
	struct mtk_ldm_spi stMSPI;
	int32_t ret = 0;
	uint16_t i;
	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi));
	stMSPI.u32ch = u8CH;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	stMSPI.u32out_bit_len = pwdata->u16len;
	stMSPI.u32in_bit_len = prdata->u16len;

	for (i = 0; i < pwdata->u16len; i++)
		stMSPI.u8out[i] = pwdata->au8Data[i];

	stMSPI.u32out_bit_len = (pwdata->u16len * LDM_8BITS);
	stMSPI.u32in_bit_len = (prdata->u16len * LDM_8BITS);
	ret = _mtk_ldm_spi(&stMSPI);

	for (i = 0; i < prdata->u16len; i++)
		prdata->au8Data[i] = stMSPI.u8in[i];

	return ret;
}

static void _mtk_ldm_set_pwm_sel(struct mtk_ldm_priv *priv)
{
	if (priv->u8ldm_ver == LDM_VER_1)
		W2BYTEMSK(LDM_PWM_REG_0B0, 0x01, LDM_PWM_REG_0B0_VSYNC_LIKE_SRC_SEL);
	else
		W2BYTEMSK(LDM_PWM_REG_0B0, 0x01, LDM_PWM_REG_0B0_VSYNC_LIKE_SRC_SEL_V2);
}

static void _mtk_ldm_set_pwm_clken_bitmap(void)
{
	W2BYTEMSK(LDM_PWM_REG_004, LDM_CLKEN_BITMAP, LDM_PWM_REG_004_CLKEN_BITMAP);
}

static void _mtk_ldm_set_pwm_vsync_like_keep_low(void)
{
	W2BYTEMSK(LDM_PWM_REG_0A4, 0, LDM_PWM_REG_0A4_VSYNC_LIKE_ST_1);
	W2BYTEMSK(LDM_PWM_REG_0A0, 0, LDM_PWM_REG_0A0_VSYNC_LIKE_ST_0);
	W2BYTEMSK(LDM_PWM_REG_0AC, 0, LDM_PWM_REG_0AC_VSYNC_LIKE_END_1);
	W2BYTEMSK(LDM_PWM_REG_0A8, 0, LDM_PWM_REG_0A8_VSYNC_LIKE_END_0);
}

static void _mtk_ldm_set_pwm_vsync_like(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_0A4,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_vsync_width[LDM_BYTES_1],
		  LDM_PWM_REG_0A4_VSYNC_LIKE_ST_1);
	W2BYTEMSK(LDM_PWM_REG_0A0,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_vsync_width[LDM_BYTES_0],
		  LDM_PWM_REG_0A0_VSYNC_LIKE_ST_0);
	W2BYTEMSK(LDM_PWM_REG_0AC,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_vsync_width[LDM_BYTES_3],
		  LDM_PWM_REG_0AC_VSYNC_LIKE_END_1);
	W2BYTEMSK(LDM_PWM_REG_0A8,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_vsync_width[LDM_BYTES_2],
		  LDM_PWM_REG_0A8_VSYNC_LIKE_END_0);
}

static void _mtk_ldm_set_pwm0_period(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_1CC,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm0_period[LDM_BYTES_1],
		  LDM_PWM_REG_1CC_PWM0_PERIOD_1);
	W2BYTEMSK(LDM_PWM_REG_1C8,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm0_period[LDM_BYTES_0],
		  LDM_PWM_REG_1C8_PWM0_PERIOD_0);
}

static void _mtk_ldm_set_pwm0_shift(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_1D4,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm0_shift[LDM_BYTES_1],
		  LDM_PWM_REG_1D4_PWM0_SHIFT_1);
	W2BYTEMSK(LDM_PWM_REG_1D0,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm0_shift[LDM_BYTES_0],
		  LDM_PWM_REG_1D0_PWM0_SHIFT_0);
}

static void _mtk_ldm_set_pwm0_duty(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_1DC,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm0_duty[LDM_BYTES_1],
		  LDM_PWM_REG_1DC_PWM0_DUTY_1);
	W2BYTEMSK(LDM_PWM_REG_1D8,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm0_duty[LDM_BYTES_0],
		  LDM_PWM_REG_1D8_PWM0_DUTY_0);
}

static void _mtk_ldm_set_pwm1_period(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_1EC,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm1_period[LDM_BYTES_1],
		  LDM_PWM_REG_1EC_PWM1_PERIOD_1);
	W2BYTEMSK(LDM_PWM_REG_1E8,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm1_period[LDM_BYTES_0],
		  LDM_PWM_REG_1E8_PWM1_PERIOD_0);
}

static void _mtk_ldm_set_pwm1_shift(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_1F4,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm1_shift[LDM_BYTES_1],
		  LDM_PWM_REG_1F4_PWM1_SHIFT_1);
	W2BYTEMSK(LDM_PWM_REG_1F0,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm1_shift[LDM_BYTES_0],
		  LDM_PWM_REG_1F0_PWM1_SHIFT_0);
}

static void _mtk_ldm_set_pwm1_duty(struct mtk_ldm_priv *priv)
{
	W2BYTEMSK(LDM_PWM_REG_1FC,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm1_duty[LDM_BYTES_1],
		  LDM_PWM_REG_1FC_PWM1_DUTY_1);
	W2BYTEMSK(LDM_PWM_REG_1F8,
		  (uint16_t)priv->st_mtk_ldm_dma_info.au32ld_pwm1_duty[LDM_BYTES_0],
		  LDM_PWM_REG_1F8_PWM1_DUTY_0);
}

static void _mtk_ldm_set_pwm0_vdben(bool bEn)
{
	W2BYTEMSK(LDM_PWM_REG_008, (uint16_t)bEn, LDM_PWM_REG_0008_PWM0_VDBEN);
}

static void _mtk_ldm_set_pwm0_reset_en(bool bEn)
{
	W2BYTEMSK(LDM_PWM_REG_008, (uint16_t)bEn, LDM_PWM_REG_0008_PWM0_RESET_EN);
}

static void _mtk_ldm_set_pwm0_vdben_sw(bool bEn)
{
	W2BYTEMSK(LDM_PWM_REG_008, (uint16_t)bEn, LDM_PWM_REG_0008_PWM0_VDBEN_SW);
}

static void _mtk_ldm_set_pwm1_vdben(bool bEn)
{
	W2BYTEMSK(LDM_PWM_REG_00C, (uint16_t)bEn, LDM_PWM_REG_000C_PWM1_VDBEN);
}

static void _mtk_ldm_set_pwm1_reset_en(bool bEn)
{
	W2BYTEMSK(LDM_PWM_REG_00C, (uint16_t)bEn, LDM_PWM_REG_000C_PWM1_RESET_EN);
}

static void _mtk_ldm_set_pwm1_vdben_sw(bool bEn)
{
	W2BYTEMSK(LDM_PWM_REG_00C, (uint16_t)bEn, LDM_PWM_REG_000C_PWM1_VDBEN_SW);
}

static void _mtk_ldm_set_pwm_init(struct mtk_ldm_priv *priv)
{
	_mtk_ldm_set_pwm_clken_bitmap();

	_mtk_ldm_set_pwm0_vdben(1);
	_mtk_ldm_set_pwm0_reset_en(1);
	_mtk_ldm_set_pwm0_vdben_sw(1);
	_mtk_ldm_set_pwm0_period(priv);
	_mtk_ldm_set_pwm0_duty(priv);
	_mtk_ldm_set_pwm0_shift(priv);

	_mtk_ldm_set_pwm1_vdben(1);
	_mtk_ldm_set_pwm1_reset_en(1);
	_mtk_ldm_set_pwm1_vdben_sw(1);
	_mtk_ldm_set_pwm1_period(priv);
	_mtk_ldm_set_pwm1_duty(priv);
	_mtk_ldm_set_pwm1_shift(priv);

	_mtk_ldm_set_pwm_vsync_like(priv);
	_mtk_ldm_set_pwm_sel(priv);
}

int32_t _mtk_ldm_as3824_init(struct mtk_ldm_priv *priv, uint8_t u8CH)
{
	int32_t ret = 0;
	int i = 0;
	uint8_t u8temp = 0;
	uint32_t u32SPI_Devicenum;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;
	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
	uint8_t LED_Device_AS3824_initial_Reg0x01to0x15[]=
	{
		LDM_AS382X_BCAST_MULTI_BYTE_SAME_DATA, Reg_0x01_to_0x15_NUM, LDM_AS382X_CUR_ON_1_ADDR,
		0x00,0x00,0x00,0x00,0x00, //addr 0x1~0x5 CUR_ON_1, CUR_ON_2 , FAULT_1, GPIO_CTIL, FB_SEL1
		0x00,0x00,0x00,0x00, //addr 0x6~0x9 , FB_SEL2, CURRctrl, SHORTLED1, SHORTLED2
		0x00,0x00,0x00,0x00, //addr 0xA~0xD, OPENLED1, OPENLED2, VDAC_H, VDAC_L
		0x00,0x00,0x00,0x00, //addr 0xE~0x11, FB_ON_1, FB_ON_2, IDAC_FB1_COUNTER, IDAC_FB2_COUNTER
		0x00,0x00,0x00,0x00, //addr 0x12~0x15, FBLOOP_CTRL, PWMCTRL, PWMperiodLSB, PWMperiodMSB

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_AS3824_initial_PLLmulti[]=
	{
		LDM_AS382X_BCAST_MULTI_BYTE_SAME_DATA, PLL_MULTI_BYTE_NUM, LDM_AS382X_PLLmulti1_ADDR,
		0x00,0x00,

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_AS3824_initial_HDR[]=
	{
		LDM_AS382X_BCAST_SINGLE_BYTE_SAME_DATA, LDM_AS382X_HDR_ADDR, LDM_NUM_16,

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_AS3824_initial_PWM_Delay[]= //Delay
	{
		LDM_AS382X_BCAST_MULTI_BYTE_SAME_DATA, LDM_AS382X_DELAY_NUM, LDM_AS382X_DELAY1_ADDR, //addr 0x16~0x35
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_AS3824_initial_PWM_Duty[]= //Duty
	{
		LDM_AS382X_BCAST_MULTI_BYTE_SAME_DATA, LDM_AS382X_PWM_HTIME_NUM, LDM_AS382X_PWM_HTIME1_ADDR, //addr 0x37~0x56
		0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_AS3824_initial_BDAC[]= //BDAC
	{
		LDM_AS382X_BCAST_MULTI_BYTE_SAME_DATA, LDM_AS382X_BDAC_NUM, LDM_AS382X_BDAC1_ADDR, //addr 0x37~0x56
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_AS3824_read_single_data[]=
	{
		LDM_AS382X_SINGLE_BYTE_SAME_DATA, 0x00, 0x00,

		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	LED_Device_AS3824_read_single_data[LDM_BYTES_0] = (LDM_AS382X_SINGLE_BYTE_SAME_DATA | LDM_BYTES_1);
	LED_Device_AS3824_read_single_data[LDM_BYTES_1] = LDM_AS382X_STATUS_ADDR;
	LED_Device_AS3824_read_single_data[LDM_BYTES_2] = 0x00;

	// Wait 3824 power on
	// check 0xff for Nova case
	UBOOT_DEBUG("Wait AS3824 Power");
	if ((priv->st_mtk_ldm_device_info.u32device_num[u8CH] + LDM_AS382X_WRITE_SAME_DATA_PWM_CMD_NUM)
		>= MAX_LDM_SPI_BYTES)
	{
		debug("[%s,%5d] Over Size!!!\n", __func__, __LINE__);
		return 0;
	}
	u32SPI_Devicenum = priv->st_mtk_ldm_device_info.u32device_num[u8CH];
	do
	{
		memset((void *)&WData,0,sizeof(struct mtk_ldm_spi_data));
		memset((void *)&RData,0,sizeof(struct mtk_ldm_spi_data));
		WData.u16len = LDM_AS382X_SINGLE_READ_CMD_NUM + u32SPI_Devicenum;
		memcpy(&WData.au8Data, LED_Device_AS3824_read_single_data, sizeof(LED_Device_AS3824_read_single_data));

		RData.u16len = 1;
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
		u8temp = RData.au8Data[0];

		UBOOT_DEBUG("check as3824 power %x:", u8temp);
	}while((u8temp == NT50516_POWER_ON)||(u8temp&BIT0)== 0);
	UBOOT_DEBUG("AS3824 power on");

	/*unlock device*/
	WData.u16len = LDM_AS382X_BCAST_SINGLE_WRITE_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;

	memcpy(&WData.au8Data, LED_Device_AS3824_unlock, sizeof(LED_Device_AS3824_unlock));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*inital reg0x01~0x15*/
	for(i = 0; i < Reg_0x01_to_0x0B_NUM ; i++)
	{
		LED_Device_AS3824_initial_Reg0x01to0x15[LDM_AS382X_HEADER_NUM + i] = (uint8_t)priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x01_to_0x0B[i];
	}

	for(i = 0; i < VDAC_BYTE_NUM ; i++)
	{
		LED_Device_AS3824_initial_Reg0x01to0x15[LDM_AS382X_HEADER_NUM +  LDM_AS382X_VDAC_LEDH_ADDR - 1 + i] =  (uint8_t)priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x0C_0x0D_VDAC_MBR_OFF[i];
	}

	for(i = 0; i < Reg_0x0E_to_0x15_NUM ; i++)
	{
		LED_Device_AS3824_initial_Reg0x01to0x15[LDM_AS382X_HEADER_NUM + LDM_AS382X_FB_ON1_ADDR -1 + i] = (uint8_t)priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x0E_to_0x15[i];
	}

	for(i = 0; i < LDM_AS382X_PWM_HTIME_NUM ; i++)
	{
		LED_Device_AS3824_initial_PWM_Duty[LDM_AS382X_HEADER_NUM + i] = (uint8_t)(priv->st_ldm_Device_AS3824.u16AS3824_PWM_Duty_Init & MASK_2BYTES);
		LED_Device_AS3824_initial_PWM_Duty[LDM_AS382X_HEADER_NUM + i + 1] = (uint8_t)(priv->st_ldm_Device_AS3824.u16AS3824_PWM_Duty_Init >> SHIFT_2BYTES);
	}

	WData.u16len = LDM_AS382X_WRITE_SAME_DATA_REG_0x01_0x15_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_initial_Reg0x01to0x15, sizeof(LED_Device_AS3824_initial_Reg0x01to0x15));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*inital PLLmulti*/
	WData.u16len = LDM_AS382X_WRITE_SAME_DATA_PLL_MULTI_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_initial_PLLmulti, sizeof(LED_Device_AS3824_initial_PLLmulti));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*inital HDR*/
	WData.u16len = LDM_AS382X_BCAST_SINGLE_WRITE_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_initial_HDR, sizeof(LED_Device_AS3824_initial_HDR));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*inital PWM_delay*/
	WData.u16len = LDM_AS382X_WRITE_SAME_DATA_PWM_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_initial_PWM_Delay, sizeof(LED_Device_AS3824_initial_PWM_Delay));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*inital PWM_Duty*/
	WData.u16len = LDM_AS382X_WRITE_SAME_DATA_PWM_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_initial_PWM_Duty, sizeof(LED_Device_AS3824_initial_PWM_Duty));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*inital BDAC*/
	WData.u16len = LDM_AS382X_WRITE_SAME_DATA_BDAC_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_initial_BDAC, sizeof(LED_Device_AS3824_initial_BDAC));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*lock device*/
	WData.u16len = LDM_AS382X_BCAST_SINGLE_WRITE_CMD_NUM + u32SPI_Devicenum;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_AS3824_lock, sizeof(LED_Device_AS3824_lock));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		debug("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	return ret;

}

u16 _mtk_ldm_read_iw7039_one(struct mtk_ldm_priv *priv, u8 u8CH, u8 u8devnum, u8 u8devid, u16 u16regaddr)
{
	u8 u8dummy_no = ((u8devnum + IW7039_DUMMY_CAL_NUM - 1) / IW7039_DUMMY_CAL_NUM);
	u16 u16temp = 0;
	uint8_t au8Data[LDM_NUM_256];
	u16 u16read_val = 0;
	uint8_t i = 0;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;
	u16 u16checksum = 0;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
	if (u8devnum > IW7039_DAISY_CHAIN_CHIP_NUM_MAX) {
		UBOOT_DEBUG("[MSPI_INIT]MSPI_Read_Iw7039_SingleData number of device wrong!\n");
		return 0;
	}

	if (u8devid > IW7039_SINGLE_DATA_READ_DEVICE_NUM_MAX) {
		UBOOT_DEBUG("[MSPI_INIT]MSPI_Read_Iw7039_SingleData read device no. wrong!\n");
		return 0;
	}
	// command
	au8Data[LDM_BYTES_0] = (uint8_t)((COMMAND_IW7039_READ_SINGLE_DEVICE >> SHIFT_2BYTES) + u8devid);
	au8Data[LDM_BYTES_1] = (uint8_t)(COMMAND_IW7039_READ_SINGLE_DEVICE & MASK_2BYTES);
	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16temp = au8Data[LDM_BYTES_0] * LDM_NUM_256 + au8Data[LDM_BYTES_1];
		u16checksum += u16temp;
	}

	// address
	u16temp = (ADDRESS_IW7039_READ_SINGLE_DEVICE + u16regaddr);
	au8Data[LDM_BYTES_2] = (uint8_t)(u16temp >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_3] = (uint8_t)((u16temp & MASK_2BYTES));
	if (priv->stiw7039.u8iw7039_check_sum_flag)
		u16checksum += u16temp;

	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16checksum = ~u16checksum;
		au8Data[LDM_NUM_4] = (u8)(u16checksum >> SHIFT_8);
		au8Data[LDM_NUM_5] = (u8)((u16checksum & MASK_0XFF));

		// device dummy
		for (i = 0; i < u8dummy_no; i++) {
			au8Data[LDM_NUM_6 + BYTE2 * i] = 0;
			au8Data[LDM_NUM_6 + BYTE2 * i + 1] = 0;
			if (priv->stiw7039.u8iw7039_check_sum_flag) {
				u16temp = au8Data[LDM_NUM_6 + BYTE2 * i] * LDM_NUM_256 + au8Data[LDM_NUM_6 + BYTE2 * i + 1];
				u16checksum += u16temp;
			}
		}

		WData.u16len = LDM_NUM_4 + IW7039_BYTE_NUM_OF_VDATA + (u8dummy_no * BYTE2);
		memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
		memcpy(&WData.au8Data, au8Data, WData.u16len);
		RData.u16len = LDM_NUM_2;
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
		u16read_val = RData.au8Data[1] | ((RData.au8Data[0] & MASK_2BYTES) << SHIFT_2BYTES);
	} else {
		// device dummy
		for (i = 0; i < u8dummy_no; i++) {
			au8Data[LDM_NUM_4 + BYTE2 * i] = 0;
			au8Data[LDM_NUM_4 + BYTE2 * i + 1] = 0;
			if (priv->stiw7039.u8iw7039_check_sum_flag) {
				u16temp = au8Data[LDM_NUM_6 + BYTE2 * i] * LDM_NUM_256 + au8Data[LDM_NUM_6 + BYTE2 * i + 1];
				u16checksum += u16temp;
			}
		}

		WData.u16len = LDM_NUM_4 + (u8dummy_no * BYTE2);
		memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
		memcpy(&WData.au8Data, au8Data, WData.u16len);
		RData.u16len = LDM_NUM_2;
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
		u16read_val = RData.au8Data[1] | ((RData.au8Data[0] & MASK_2BYTES) << SHIFT_2BYTES);
	}

	return u16read_val;

}

void _mtk_ldm_w_all_iw7039_sameone(struct mtk_ldm_priv *priv, u8 u8CH, u8 u8devnum, u16 u16regaddr, u16 u16in)
{
	u8 u8dummy_no = ((u8devnum + IW7039_DUMMY_CAL_NUM - 1) / IW7039_DUMMY_CAL_NUM);
	u16 u16temp = 0;
	uint8_t au8Data[LDM_NUM_256];
	uint8_t i = 0;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;
	u16 u16checksum = 0;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
	if (u8devnum > IW7039_DAISY_CHAIN_CHIP_NUM_MAX) {
		UBOOT_DEBUG("[MSPI_INIT]MSPI_Write_Iw7039_SetDaisyChainChipNum number of device wrong!\n");
		return;
	}

	// SPI SW mode, pull high VSync GPIO7
	//GPIO_SetOut(7, 1);

	// command
	au8Data[LDM_BYTES_0] = (uint8_t)(COMMAND_IW7039_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_1] = (uint8_t)(COMMAND_IW7039_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE & MASK_2BYTES);
	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16temp = au8Data[LDM_BYTES_0] * LDM_NUM_256 + au8Data[LDM_BYTES_1];
		u16checksum += u16temp;
	}

	// address
	u16temp = (ADDRESS_IW7039_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE + u16regaddr);
	au8Data[LDM_BYTES_2] = (uint8_t)(u16temp >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_3] = (uint8_t)((u16temp & MASK_2BYTES));
	if (priv->stiw7039.u8iw7039_check_sum_flag)
		u16checksum += u16temp;

	// data
	au8Data[LDM_BYTES_4] = (uint8_t)(u16in >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_5] = (uint8_t)((u16in & MASK_2BYTES));
	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16temp = au8Data[LDM_BYTES_4] * LDM_NUM_256 + au8Data[LDM_BYTES_5];
		u16checksum += u16temp;
	}

	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16checksum = ~u16checksum;
		au8Data[LDM_NUM_6] = (u8)(u16checksum >> SHIFT_8);
		au8Data[LDM_NUM_7] = (u8)((u16checksum & MASK_0XFF));

		// dummy
		for (i = 0; i < u8dummy_no; i++) {
			au8Data[LDM_NUM_8 + BYTE2 * i] = 0;
			au8Data[LDM_NUM_8 + BYTE2 * i + 1] = 0;
		}

		WData.u16len = (LDM_NUM_6 + IW7039_BYTE_NUM_OF_VDATA + (BYTE2 * u8dummy_no));
		RData.u16len = 0;
		memcpy(&WData.au8Data, au8Data, WData.u16len);
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);

		// SPI SW mode, pull low VSync GPIO7
		//GPIO_SetOut(7, 0);
	} else {
		// dummy
		for (i = 0; i < u8dummy_no; i++) {
			au8Data[LDM_NUM_6 + BYTE2 * i] = 0;
			au8Data[LDM_NUM_6 + BYTE2 * i + 1] = 0;
		}

		WData.u16len = (LDM_NUM_6 + (BYTE2 * u8dummy_no));
		RData.u16len = 0;
		memcpy(&WData.au8Data, au8Data, WData.u16len);
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);

		// SPI SW mode, pull low VSync GPIO7
		//GPIO_SetOut(7, 0);
	}
}

void _mtk_ldm_w_all_iw7039_same32(struct mtk_ldm_priv *priv, u8 u8CH, u8 u8devnum, u16 u16regaddr, u16 *p16in)
{
	u8 u8dummy_no = ((u8devnum + IW7039_DUMMY_CAL_NUM - 1) / IW7039_DUMMY_CAL_NUM);
	u16 u16temp = 0;
	uint16_t i = 0;
	uint8_t au8Data[LDM_NUM_256];
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;
	u16 u16checksum = 0;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
	if (u8devnum > IW7039_DAISY_CHAIN_CHIP_NUM_MAX) {
		UBOOT_DEBUG("[MSPI_INIT]MSPI_Write_Iw7039_SetDaisyChainChipNum number of device wrong!\n");
		return;
	}

	// SPI SW mode, pull high VSync GPIO7
	//GPIO_SetOut(7, 1);

	// command
	au8Data[LDM_BYTES_0] = (uint8_t)(COMMAND_IW7039_WRITE_SAME_32_WORD_OF_ALL_DEVICE >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_1] = (uint8_t)(COMMAND_IW7039_WRITE_SAME_32_WORD_OF_ALL_DEVICE & MASK_2BYTES);
	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16temp = au8Data[LDM_BYTES_0] * LDM_NUM_256 + au8Data[LDM_BYTES_1];
		u16checksum += u16temp;
	}

	// address
	u16temp = (ADDRESS_IW7039_WRITE_SAME_32_WORD_OF_ALL_DEVICE + u16regaddr);
	au8Data[LDM_BYTES_2] = (uint8_t)(u16temp >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_3] = (uint8_t)((u16temp & MASK_2BYTES));
	if (priv->stiw7039.u8iw7039_check_sum_flag)
		u16checksum += u16temp;

	// data
	for (i = 0; i < IW7039_32WORD; i++) {
		au8Data[LDM_NUM_4 + (BYTE2 * i)] = (uint8_t)(p16in[i] >> SHIFT_2BYTES);
		au8Data[LDM_NUM_4 + (BYTE2 * i) + 1] = (uint8_t)((p16in[i] & MASK_2BYTES));
		if (priv->stiw7039.u8iw7039_check_sum_flag) {
			u16temp = au8Data[LDM_NUM_4 + (BYTE2 * i)] * LDM_NUM_256 + au8Data[LDM_NUM_4 + (BYTE2 * i) + 1];
			u16checksum += u16temp;
		}
	}

	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16checksum = ~u16checksum;
		au8Data[LDM_NUM_4 + (IW7039_32WORD * BYTE2)] = (u8)(u16checksum >> SHIFT_8);
		au8Data[LDM_NUM_4 + (IW7039_32WORD * BYTE2) + 1] = (u8)((u16checksum & MASK_0XFF));

		// dummy
		for (i = 0; i < u8dummy_no; i++) {
			au8Data[LDM_NUM_4 + (IW7039_32WORD * BYTE2) + IW7039_BYTE_NUM_OF_VDATA + (BYTE2 * i)] = 0;
			au8Data[LDM_NUM_4 + (IW7039_32WORD * BYTE2) + IW7039_BYTE_NUM_OF_VDATA + (BYTE2 * i) + 1] = 0;
		}

		WData.u16len = (LDM_NUM_4 + (IW7039_32WORD * BYTE2) + IW7039_BYTE_NUM_OF_VDATA + (BYTE2 * u8dummy_no));
		RData.u16len = 0;
		memcpy(&WData.au8Data, au8Data, WData.u16len);
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);

		// SPI SW mode, pull low VSync GPIO7
		//GPIO_SetOut(7, 0);
	} else {
		// dummy
		for (i = 0; i < u8dummy_no; i++) {
			au8Data[LDM_NUM_4 + (IW7039_32WORD * BYTE2) + (BYTE2 * i)] = 0;
			au8Data[LDM_NUM_4 + (IW7039_32WORD * BYTE2) + (BYTE2 * i) + 1] = 0;
		}

		WData.u16len = (LDM_NUM_4 + (IW7039_32WORD * BYTE2) + (BYTE2 * u8dummy_no));
		RData.u16len = 0;
		memcpy(&WData.au8Data, au8Data, WData.u16len);
		_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);

		// SPI SW mode, pull low VSync GPIO7
		//GPIO_SetOut(7, 0);
	}
}

void _mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(struct mtk_ldm_priv *priv, uint8_t u8CH, uint8_t u8chip_num)
{
	u8 u8dummy_no = ((u8chip_num + IW7039_DUMMY_CAL_NUM - 1) / IW7039_DUMMY_CAL_NUM);
	u8 i = 0;
	u8 au8Data[LDM_NUM_256];
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));

	if (u8chip_num > IW7039_DAISY_CHAIN_CHIP_NUM_MAX) {
		UBOOT_ERROR("[MSPI_INIT]MSPI_Write_Iw7039_SetDaisyChainChipNum number of device wrong!\n");
		return;
	}

	// command
	au8Data[LDM_BYTES_0] = (u8)(COMMAND_IW7039_SET_NUM_OF_DEVICE >> SHIFT_2BYTES);
	au8Data[LDM_BYTES_1] = (u8)(u8chip_num);

	// device dummy
	for (i = 0; i < u8dummy_no; i++) {
		au8Data[BYTE2 + (BYTE2 * i)] = 0;
		au8Data[BYTE2 + (BYTE2 * i) + 1] = 0;
	}

	// Send SPI command
	WData.u16len = (LDM_NUM_2 + (BYTE2 * u8dummy_no));
	RData.u16len = 0;
	memcpy(&WData.au8Data, au8Data, WData.u16len);
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
}

void _mtk_ldm_iw7039_init_turn_on_backlight(struct mtk_ldm_priv *priv, uint8_t u8CH)
{
	u8 u8SPI_Devicenum;

	if (priv->st_mtk_ldm_device_info.u32device_num[u8CH] > MAX_LDM_SPI_BYTES)
		priv->st_mtk_ldm_device_info.u32device_num[u8CH] = MAX_LDM_SPI_BYTES;
	u8SPI_Devicenum = priv->st_mtk_ldm_device_info.u32device_num[u8CH];

	//12.Soft start IW7039 calibration process
	if (priv->stiw7039.u8iw7039_check_sum_flag)
		_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, 0x000, IW7039_VDATA_SOFT_START);
	else
		_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, 0x000, IW7039_SOFT_START);

	msleep(SLEEP_TIME_1);

	UBOOT_DEBUG("pqu_ldm_iw7039_init end\n");
}

int32_t _mtk_ldm_iw7039_init(struct mtk_ldm_priv *priv, uint8_t u8CH)
{
	int32_t ret = 0;
	u16 u16read_val = 0;
	uint8_t u8SPI_Devicenum;
	uint8_t i = 0;
	u8 u8retry = 0;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));

	u16 intReg_Conf[REG_0X00_TO_0X1F_NUM] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											// 0X00 ~ 0X07 , reg 0x001 config to use global ISET with 30mA,
											// reg 0x05 config to 4x Vsync
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											// 0X08 ~ 0X0F ( 0x20 set ILED range as 60mA)
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											// 0X10 ~ 0X17 , Reg 0x10 , 0x11 enable 32 CHs,
											// Reg 0x14 turn-off all fault , ADJ-short doesn¡¯t turn off
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											//0X18 ~0X1F , reg 0x18 and 0x19 enable ADP for 32 CHs
											}; //Initializing

	// Setting individual ILED
	u16 intReg_ISET[IW7039_PWM_ISET_NUM] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x20 ~ 0x27
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x28 ~ 0x2F
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x30 ~ 0x37
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x38 ~ 0x3F
											};//0x20 to 0x3F

	u16 intReg_HT[IW7039_PWM_HTIME_NUM] = {0, 0, 0, 0, 0, 0, 0, 0,	// 0x60 ~ 0x67
											0, 0, 0, 0, 0, 0, 0, 0,	// 0x68 ~ 0x6F
											0, 0, 0, 0, 0, 0, 0, 0,	// 0x70 ~ 0x77
											0, 0, 0, 0, 0, 0, 0, 0,	// 0x78 ~ 0x7F
											} ;//0x60 to 0x7F
	if (priv->st_mtk_ldm_device_info.u32device_num[u8CH] > MAX_LDM_SPI_BYTES)
		priv->st_mtk_ldm_device_info.u32device_num[u8CH] = MAX_LDM_SPI_BYTES;
	u8SPI_Devicenum = priv->st_mtk_ldm_device_info.u32device_num[u8CH];
	msleep(SLEEP_TIME_100);

	UBOOT_DEBUG("pqu_ldm_iw7039_init start");
	msleep(SLEEP_TIME_10);

	_mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(priv, u8CH, u8SPI_Devicenum);

	msleep(SLEEP_TIME_10);

	UBOOT_DEBUG("check IW7039 1 power\n");

	do
	{
		u16read_val = _mtk_ldm_read_iw7039_one(priv, u8CH, u8SPI_Devicenum, 1, IW7039_0x344_ADDR);
		u16read_val = (u16read_val >> SHIFT_2BYTES) & MASK_2BYTES;
		u8retry++;
	} while ((u16read_val != IW7039_POWER_ON) && (u8retry <= RETRY_COUNTER));

	if (u16read_val == IW7039_POWER_ON)
		UBOOT_DEBUG("IW7039 CH:%d power 1 on\n", u8CH);
	else
		UBOOT_ERROR("IW7039 CH:%d power Off!!!!!!!!!!!!!!!!!!\n", u8CH);

	_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, 0x000, IW7039_INIT_REG0_VALUE);
	msleep(SLEEP_TIME_10);

	UBOOT_DEBUG("IW7039 init table\n");
	for (i = 0; i < REG_0X00_TO_0X1F_NUM ; i++)
		intReg_Conf[i] = priv->stiw7039.u32iw7039_teg_0x000_to_0x01F[i];

	_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, 0x000, intReg_Conf);
	msleep(SLEEP_TIME_10);

	UBOOT_DEBUG("IW7039 init ISET\n");
	for(i = 0; i < IW7039_PWM_ISET_NUM ; i++)
	{
		intReg_ISET[i] = priv->stiw7039.u16IW7039_ISET_MBR_OFF;
	}

	//Init ISET 0x20 ~ 0x3F
	_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_ISET_ADDR, intReg_ISET);
	msleep(SLEEP_TIME_10);

	//Init HT 0x60 ~ 0x7F to 0 before read calibration state
	UBOOT_DEBUG("IW7039 init DUTY\n");
	if(1) //quiet hot boot
	{
		for(i = 0; i < IW7039_PWM_HTIME_NUM ; i++)
		{
			intReg_HT[i] = priv->stiw7039.u16iw7039_pwm_duty_init;
		}

		_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_PWM_HTIME1_ADDR, intReg_HT);
		msleep(SLEEP_TIME_10);
	}
	else
	{
		_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_PWM_HTIME1_ADDR, intReg_HT);
	}

	msleep(SLEEP_TIME_200);

	return ret;

}

int32_t _mtk_ldm_mbi6353_init(struct mtk_ldm_priv *priv, uint8_t u8CH)
{
	int32_t ret = 0;
	int i = 0;
	uint32_t u32SPI_Devicenum;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;
	uint8_t LED_Device_MBI6353_initial_Mask[]=
	{
		MBI6353_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE_H, MBI6353_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE_L, 0x00, MBI6353_MASK_NUM, MBI6353_MASK_ADDR_H, MBI6353_MASK_ADDR_L, //header
		0x00,0x00,0x00,0x00,0x00,0x00, //data
		0x00,0x00, //check sum
		0x00,0x00,0x00,0x00, //dummy
		0x00,0x00,0x00,0x00, 
		0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00, 
		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_MBI6353_initial_Config[]=
	{
		MBI6353_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE_H, MBI6353_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE_L, 0x00, MBI6353_CFG_NUM, MBI6353_CFG_ADDR_H, MBI6353_CFG_ADDR_L, //header
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00,0x00, //check sum
		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};

	uint8_t LED_Device_MBI6353_initial_Duty[]=
	{
		MBI6353_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE_H, MBI6353_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE_L, 0x00, MBI6353_DUTY_NUM, MBI6353_SCAN1_DUTY_H, MBI6353_SCAN1_DUTY_L, //header
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, // 192 = 48 X 4 SCAN
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,

		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 

		0x00,0x00, //check sum
		0x00,0x00,0x00,0x00,0x00, //Dummy*25
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
		0x00,0x00,0x00,0x00,0x00,
	};
	if (priv->st_mtk_ldm_device_info.u32device_num[u8CH] >= MAX_LDM_SPI_BYTES)
	{
		debug("[%s,%5d] Over Size!!!\n", __func__, __LINE__);
		return 0;
	}
	u32SPI_Devicenum = priv->st_mtk_ldm_device_info.u32device_num[u8CH];
/*inital Mask*/
	for(i = 0; i < MBI6353_MASK_NUM * BYTE2; i++)
	{
		LED_Device_MBI6353_initial_Mask[(MBI6353_HEADER_NUM * BYTE2) + i] = (uint8_t)priv->st_ldm_Device_MBI6353.u32MBI6353_Mask[i];
	}

	for(i = 0; i < MBI6353_CFG_NUM * BYTE2 ; i++)
	{
		LED_Device_MBI6353_initial_Config[(MBI6353_HEADER_NUM * BYTE2) + i] = (uint8_t)priv->st_ldm_Device_MBI6353.u32MBI6353_Config[i];
	}

	for(i = 0; i < MBI6353_DUTY_NUM; i++)
	{
		LED_Device_MBI6353_initial_Duty[(MBI6353_HEADER_NUM * BYTE2) + (BYTE2 * i)] = (uint8_t)(priv->st_ldm_Device_MBI6353.u16MBI6353_PWM_Duty_Init>> SHIFT_2BYTES);
		LED_Device_MBI6353_initial_Duty[(MBI6353_HEADER_NUM * BYTE2) + (BYTE2 * i) + 1] = (uint8_t)(priv->st_ldm_Device_MBI6353.u16MBI6353_PWM_Duty_Init & MASK_2BYTES);
	}

	/*SEND MASK*/
	WData.u16len = (MBI6353_HEADER_NUM + MBI6353_MASK_NUM + MBI6353_CHECKSUM_NUM + u32SPI_Devicenum - 1) * BYTE2;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_MBI6353_initial_Mask, sizeof(LED_Device_MBI6353_initial_Mask));
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		UBOOT_DEBUG("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*SEND CONFIG*/
	WData.u16len = (MBI6353_HEADER_NUM + MBI6353_CFG_NUM + MBI6353_CHECKSUM_NUM + u32SPI_Devicenum - 1) * BYTE2;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_MBI6353_initial_Config, sizeof(LED_Device_MBI6353_initial_Config));
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		UBOOT_DEBUG("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}

	/*SEND DUTY INIT*/
	WData.u16len = (MBI6353_HEADER_NUM + MBI6353_DUTY_NUM + MBI6353_CHECKSUM_NUM + u32SPI_Devicenum - 1) * BYTE2;
	RData.u16len = 0;
	memcpy(&WData.au8Data, LED_Device_MBI6353_initial_Duty, sizeof(LED_Device_MBI6353_initial_Duty));
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if(ret)
	{
		UBOOT_DEBUG("[%s,%5d] SPI transfer error = %d!!!\n", __func__, __LINE__,  ret);
	}
	

	return ret;

}

void _mtk_ldm_iw7027_same_one(struct mtk_ldm_priv *priv, u8 u8CH, u8 u8regaddr, u8 u8in)
{
	u8 au8Data[LDM_NUM_256];
	u8 u8dummy_no = priv->stiw7027.u8iw7027_dev_num - 1;
	u8 i = 0;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;

	if (priv->stiw7027.u8iw7027_dev_num > IW7027_DAISY_CHAIN_CHIP_NUM_MAX)
		return;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));

	// command
	au8Data[LDM_BYTES_0] = (u8)IW7027_SEDN;

	//addr
	au8Data[LDM_BYTES_1] = (u8)u8regaddr;

	//data
	au8Data[LDM_BYTES_2] = (u8)u8in;

	// device dummy
	for (i = 0; i < u8dummy_no; i++)
		au8Data[LDM_NUM_3 + i] = 0;

	// Send SPI command
	WData.u16len = LDM_NUM_3 + u8dummy_no;
	RData.u16len = 0;
	memcpy(&WData.au8Data, au8Data, WData.u16len);
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
}

void _mtk_ldm_iw7027_write_same_block(struct mtk_ldm_priv *priv, u8 u8CH, u8 u8_reg_addr, u8 *pu8_data, u8 u8_data_len)
{
	u8 au8Data[LDM_NUM_256];
	u8 u8dummy_no = priv->stiw7027.u8iw7027_dev_num - 1;
	u8 i = 0;
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;

	if (priv->stiw7027.u8iw7027_dev_num > IW7027_DAISY_CHAIN_CHIP_NUM_MAX)
		return;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));

	// command
	au8Data[LDM_BYTES_0] = (u8)COMMAND_IW7027_WRITE_SAME_32_WORD_OF_ALL_DEVICE;

	// NrOfBytes
	au8Data[LDM_BYTES_1] = u8_data_len;

    // address
	au8Data[LDM_BYTES_2] = (u8)(ADDRESS_IW7027_WRITE_SAME_32_WORD_OF_ALL_DEVICE + (u8_reg_addr & MASK_0X7F));

	// data
	for (i = 0; i < u8_data_len; i++)
		au8Data[LDM_BYTES_3 + i] = pu8_data[i];

	// Dummy
	for (i = 0; i < u8dummy_no; i++)
		au8Data[LDM_BYTES_3 + u8_data_len + i] = 0;

	WData.u16len = LDM_NUM_3 + u8_data_len + u8dummy_no;
	RData.u16len = 0;
	memcpy(&WData.au8Data, au8Data, WData.u16len);
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
}

u8 _mtk_ldm_read_iw7027(struct mtk_ldm_priv *priv, u8 u8CH, u8 u8dev_id, u16 u16regaddr)
{
	u8 au8Data[LDM_NUM_256];
	u8 u8read_val = 0;
	u8 u8dummy_no = priv->stiw7027.u8iw7027_dev_num - 1;
	u8 i = 0;

	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;

	if (priv->stiw7027.u8iw7027_dev_num > IW7027_DAISY_CHAIN_CHIP_NUM_MAX)
		return 0;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));

	// command
	au8Data[LDM_BYTES_0] = (u8)(IW7027_WRITE_HEADER + u8dev_id);

   // address
	au8Data[LDM_BYTES_1] = (u8)(IW7027_READ | u16regaddr);

	// read data dummy
	for (i = 0; i < u8dummy_no; i++)
		au8Data[LDM_BYTES_2 + i] = 0;

	WData.u16len = LDM_NUM_3 + u8dummy_no;
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));
	memcpy(&WData.au8Data, au8Data, WData.u16len);
	RData.u16len = LDM_NUM_1;
	_mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	u8read_val = RData.au8Data[0];

	return u8read_val;
}

void _mtk_ldm_send_iw7027_array(struct mtk_ldm_priv *priv, struct spi_slave *slave, u8 *pu8in, u32 u32size)
{
	u8 u8dummy_no = priv->stiw7027.u8iw7027_dev_num - 1;
	u8 i;

	if (priv->stiw7027.u8iw7027_dev_num > IW7027_DAISY_CHAIN_CHIP_NUM_MAX)
		return;

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	stMSPI.u32out_bit_len = (u32size + u8dummy_no) * LDM_NUM_8;

	for (i = 0; i < u32size; i++)
		stMSPI.u8out[i] = pu8in[i];

	// dummy
	for (i = 0; i < u8dummy_no; i++)
		stMSPI.u8out[u32size + i] = 0;

	spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
}

s32 _mtk_ldm_iw7027_init(struct mtk_ldm_priv *priv, u8 u8CH)
{
	struct spi_slave *slave;
	char name[LDM_CHAR_NAME], *str;
	struct udevice *dev;
	struct udevice *mainlddev;
	static u8 au8Data[LDM_NUM_1024];

	s32 ret = 0;
	int i, j, k;
	u32 u32size = 0;
	u8 u8readvalue;
	u8 u8SPI_Devicenum;

	//configure 0x00 to 0x1F Register Value
	u8 au8Iwatt7027_init_reg00_to_1F[r0x00_to_0x1F_num] = {0};
	//configure 0x20 to 0x39 Register Value
	u8 au8Iwatt7027_init_reg20_to_39[r0x20_to_0x39_num] = {0};
	//configure 0x40 to 0x5F HT Register Value
	//0x40 to 0x5F for initial duty, if you need to modify, please consult the Dialog Engineer
	u8 u8Iwatt7027_init_HT40_to_5F[r0x40_to_0x5F_num] = {0};

	//check led devIC num
	if (priv->st_mtk_ldm_device_info.u32device_num[u8CH] >= MAX_LDM_SPI_BYTES) {
		debug("[%s,%5d] Over Size!!!\n", __func__, __LINE__);
		return 1;
	}

	// make sure vsync pull low
	ret = gpio_request(LDM_NUM_9, "gpio_set_on");

	// vsync low(pull down)
	gpio_direction_output(LDM_NUM_9, 0);
	gpio_set_value(LDM_NUM_9, 0);

	// need control IW7027 Power PAD? Normaly iw7027 driver board power control by panel BL_ON,
	// no need control iw7027 power PAD
	if (priv->stiw7027.b_iw7027_need_power_pad_flag) {
		// for 7027_Power_PAD
	    ret = gpio_request(priv->stiw7027.u8_iw7027_power_pad_num, "gpio_set_on");		// use GPIO 60 as iw7039 EN

	    // Set the iw7039 Power PAD to low(pull down)
	    // set EN GPIO as output direction and pull low(0)
	    gpio_direction_output(priv->stiw7027.u8_iw7027_power_pad_num, 0);
	    gpio_set_value(priv->stiw7027.u8_iw7027_power_pad_num, 0);	// set EN GPIO pull low(0)
	}

	msleep(SLEEP_TIME_100);

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));
	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;

	//MSPI0 PAD init as GPIO, so here switch to MSPI0 function, for DCON vsync like PAD use as GPIO in mboot stage
	//W2BYTEMSK(0x2E4000, 0x5, Fld(4, 4, AC_MSKB0));	// [7:4] set 0 for MSPI0's SCZ
	//W2BYTEMSK(0x2E4000, 0x55, Fld(8, 8, AC_FULLB1));	// [3:0] set 5 for MSPI0's SCK,
														// [7:4] set 0 for MSPI0's MISO
	//W2BYTEMSK(0x2E4008, 0x05, Fld(8, 0, AC_FULLB0));	// [3:0] set 5 for MSPI0's MOSI,
														// [7:4] set 0 for vsync like PAD as GPIO

	ret = snprintf(name, sizeof(name), "generic_%d:%d", stMSPI.u32CH, 0);
	if (ret < 0)
		return ret;

	str = strdup(name);
	if (!str)
		return -ENOMEM;

	ret = spi_get_bus_and_cs(stMSPI.u32CH, 0, stMSPI.u32speed, stMSPI.u32mode, "spi_generic_drv", str, &dev, &slave);
	if (ret)
		return ret;

	ret = spi_claim_bus(slave);

	if (ret)
		goto done;

	// 1. Startup DC/DC, provide 12V to IC VIN pin.
	// need control IW7027 Power PAD? Normaly iw7027 driver board power control by panel BL_ON,
	// no need control iw7027 power PAD
	if (priv->stiw7027.b_iw7027_need_power_pad_flag) {
		// for 7039_EN_PIN
		gpio_direction_output(priv->stiw7027.u8_iw7027_power_pad_num, 1);
		gpio_set_value(priv->stiw7027.u8_iw7027_power_pad_num, 1);
	}

	// 2. Keep Vsync signal at low level
	//action: set start/end of vsync pull high time to 0 (like as vsync like keep low)
	_mtk_ldm_set_pwm_vsync_like_keep_low();

    //3.Dealy >= 8 ms, waiting for iw7039 logic working
	msleep(SLEEP_TIME_10);

	u8SPI_Devicenum = (u8)(priv->st_mtk_ldm_device_info.u32device_num[u8CH]);

	//copy init table from dtso
	for (i = 0; i < r0x00_to_0x1F_num; i++)
		au8Iwatt7027_init_reg00_to_1F[i] = (u8)priv->stiw7027.au32_iw7027_0x00_to_0x1F[i];

	for (i = 0; i < r0x20_to_0x39_num; i++)
		au8Iwatt7027_init_reg20_to_39[i] = (u8)priv->stiw7027.au32_iw7027_0x20_to_0x39[i];

	for (i = 0; i < r0x40_to_0x5F_num; i++)
		u8Iwatt7027_init_HT40_to_5F[i] = (u8)priv->stiw7027.u16_iw7027_HT;

	_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_READ_EXTD, 0);
	// 4. Write 0x00 = 0x06
	_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_MODE, LDM_NUM_6);

	// 2.read back 0x00reg
	for (j = 1; j <= u8SPI_Devicenum; j++) {
		for (i = 0; i < LDM_NUM_256; i++)  {
			u8readvalue = _mtk_ldm_read_iw7027(priv, u8CH, j, IWATT7027_REG_MODE);
			if (u8readvalue == LDM_NUM_6) {
				UBOOT_INFO("iw7027 dev[%d], loop[%d] Read 0x00 reg value match 0x06 !!!\n", j, i);
				printf("iw7027 dev[%d], loop[%d] Read 0x00 reg value match 0x06 !!!\n", j, i);
				break;
			}
			//udelay(LDM_NUM_30);
			//_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_READ_EXTD, 0);
			udelay(LDM_NUM_30);
			// Write 0x00 = 0x06
			_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_MODE, LDM_NUM_6);
		}
	}

	// 6. SPI start to write registers
	_mtk_ldm_iw7027_write_same_block(priv, u8CH, IWATT7027_REG_MODE, au8Iwatt7027_init_reg00_to_1F, r0x00_to_0x1F_num);
	_mtk_ldm_iw7027_write_same_block(priv, u8CH, IWATT7027_REG_0X20, au8Iwatt7027_init_reg20_to_39, r0x20_to_0x39_num);
	// HT
	_mtk_ldm_iw7027_write_same_block(priv, u8CH, IWATT7027_REG_0X40, u8Iwatt7027_init_HT40_to_5F, r0x40_to_0x5F_num);

	// Init every iw7027 device different register value
	j = 0;
	for (i = 0; i < priv->stiw7027.u8iw27_packnum; i++) {
		for (k = 0; k < priv->stiw7027.au32iw27_evepack[i]; k++) {
			au8Data[k] = priv->stiw7027.au8iw7027_allpackdata[k + j];
			UBOOT_INFO("0x%x, ", au8Data[k]);
			printf("0x%x, ", au8Data[k]);
		}
		UBOOT_INFO("\n");
		printf("\n");

		u32size = priv->stiw7027.au32iw27_evepack[i];
		_mtk_ldm_send_iw7027_array(priv, slave, au8Data, u32size);

		j += priv->stiw7027.au32iw27_evepack[i];
		msleep(SLEEP_TIME_10);
	}

	//if (pm_check_back_ground_active() == 0) {	// non QHb
		// 7. Generate extenal VSYNC to VSYNC/PWM pin
		ret = uclass_get_device_by_name(UCLASS_DISPLAY, "mediatek-ldm", &mainlddev);
		pinctrl_select_state(mainlddev, "vsync_like1_pad_mux");
		// [3:0] set 5 for MSPI0's MOSI[7:4] set 1 for vsync like PAD as vsync like function
		//W2BYTEMSK(0x2E4008, 0x15, Fld(8, 0, AC_FULLB0));
		udelay(SLEEP_TIME_100);

		_mtk_ldm_set_pwm0_reset_en(0);
		_mtk_ldm_set_pwm_vsync_like(priv);
		_mtk_ldm_set_pwm_sel(priv);
		msleep(SLEEP_TIME_10);
	//}

	// 8. Waiting 500ms
	msleep(LDM_NUM_256);
	msleep(LDM_NUM_256);

	// 9. Set 0x00 = 0x07
	_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_MODE, IW7027_REG_MODE_EN_VAL);

	// 10. Set 0x78[7] = 1
	_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_READ_EXTD, IW7027_REG_READ_EXTD_VAL);

	// 11. Wait until reg0xB3 LSB 4bit is 0101, it means iw7027 OTF state.
	for (j = 1; j <= u8SPI_Devicenum; j++) {
		for (i = 0; i < LDM_NUM_256; i++)  {
			u8readvalue = _mtk_ldm_read_iw7027(priv, u8CH, j, IWATT7027_REG_STATUS);
			if ((u8readvalue & MASK_0X0F) == LDM_NUM_5) {
				UBOOT_INFO("iw7027 dev[%d], loop[%d] Read 0xB3 reg LSB value match 0x5 !!!\n", j, i);
				printf("iw7027 dev[%d], loop[%d] Read 0xB3 reg LSB value match 0x5 !!!\n", j, i);
				break;
			}
		}
	}

	_mtk_ldm_iw7027_same_one(priv, u8CH, IWATT7027_REG_READ_EXTD, 0);

done:
	spi_release_bus(slave);

	return ret;
}

u16 _mtk_ldm_read_iw7038(struct mtk_ldm_priv *priv, struct spi_slave *slave, u8 u8dev_id, u16 u16regaddr)
{
	u16 u16temp = 0;
	//u_int8_t i = 0;
	//struct mtk_ldm_spi_large stMSPI;
	//int ret = 0;
	u8 temp1;

	// 7038 daisy chain max devide number is 62
	if (priv->st_iw7038.u8iw7038_dev_num > IW7038_MAX_DEV_NUM)
		return 0;

	if (u8dev_id > priv->st_iw7038.u8iw7038_dev_num)
		return 0;

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;

	// read dev
	stMSPI.u8out[0] = (u8)(COMMAND_IW7038_READ_SINGLE_DEVICE + (u8dev_id & MASK_0X3F));

	u16temp = (ADDRESS_IW7038_READ_SINGLE_DEVICE + (u16regaddr & MASK_0X3FF));
	stMSPI.u8out[1] = (u8)(u16temp >> SHIFT_8);
	stMSPI.u8out[LDM_NUM_2] = (u8)u16temp;

	// V Data
	stMSPI.u8out[LDM_NUM_3] = 0;

	stMSPI.u32out_bit_len = LDM_NUM_5 * LDM_NUM_8;
	stMSPI.u32in_tit_len = IW7038_READ_BYTE * LDM_NUM_8;
	spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN);
	spi_xfer(slave, stMSPI.u32in_tit_len, NULL, stMSPI.u8in, SPI_XFER_END);

	temp1 = stMSPI.u8in[0];

	return temp1;
}

void mtk_ldm_send_iw7038_One(struct mtk_ldm_priv *priv, struct spi_slave *slave, u8 u8dev_id, u16 u16regaddr, u8 u8in)
{
	u8 u8dummy_no = priv->st_iw7038.u8iw7038_dev_num - 1;
	u16 u16temp = 0;
	u8 i = 0;
	//struct mtk_ldm_spi_large stMSPI;
	//int ret = 0;

	// 7038 daisy chain max devide number is 62
	if (priv->st_iw7038.u8iw7038_dev_num > IW7038_MAX_DEV_NUM) {
		UBOOT_DEBUG("[mboot_iw7038]mtk_ldm_spi_iw7038_OneData number of device wrong!\n");
		return;
	}

	if (u8dev_id > priv->st_iw7038.u8iw7038_dev_num) {
		UBOOT_DEBUG("[mboot_iw7038]mtk_ldm_spi_iw7038_OneData u8dev_id > u8ChipNum wrong!\n");
		return;
	}

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	stMSPI.u32out_bit_len = (ONEDATA_SIZE + u8dummy_no) * LDM_NUM_8;

	// DevAddr
	stMSPI.u8out[0] = (u8)COMMAND_IW7038_WRITE_ONE_WORD_OF_ONE_DEVICE + u8dev_id;

	// RegAddr
	u16temp = (ADDRESS_IW7038_WRITE_ONE_WORD_OF_ONE_DEVICE + u16regaddr);
	stMSPI.u8out[1] = (u8)(u16temp >> SHIFT_8);
	stMSPI.u8out[LDM_NUM_2] = (u8)u16temp;

	// Data
	stMSPI.u8out[LDM_NUM_3] = u8in;

	// check Sum
	stMSPI.u8out[LDM_NUM_4] = 0;

	if (priv->st_iw7038.u8iw7038_check_sum_flag) {
		for (i = 0; i < ONEDATA_SIZE - 1; i++)
			stMSPI.u8out[LDM_NUM_4] ^= stMSPI.u8out[i];
	}

	// dummy
	for (i = 0; i < u8dummy_no; i++)
		stMSPI.u8out[ONEDATA_SIZE + i] = 0;

	spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
}

void mtk_ldm_send_iw7038_Same(struct mtk_ldm_priv *priv, struct spi_slave *slave, u16 u16regaddr, u8 u8in)
{
	u8 u8dummy_no = priv->st_iw7038.u8iw7038_dev_num - 1;
	u16 u16temp = 0;
	u8 i = 0;
	//struct mtk_ldm_spi_large stMSPI;
	//int ret = 0;

	// 7038 daisy chain max devide number is 62
	if (priv->st_iw7038.u8iw7038_dev_num > IW7038_MAX_DEV_NUM) {
		UBOOT_DEBUG("[mboot_iw7038]MSPI_Write_Iw7038_SetDaisyChainChipNum number of device wrong!\n");
		return;
	}

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	stMSPI.u32out_bit_len = (LDM_NUM_5 + u8dummy_no) * LDM_NUM_8;

	// DevAddr
	stMSPI.u8out[0] = (u8)COMMAND_IW7038_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE;

	// RegAddr
	u16temp = (ADDRESS_IW7038_WRITE_SAME_ONE_WORD_OF_ALL_DEVICE + (u16regaddr & MASK_0X3FF));
	stMSPI.u8out[1] = (u8)(u16temp >> SHIFT_8);
	stMSPI.u8out[LDM_NUM_2] = (u8)u16temp;

	// data
	stMSPI.u8out[LDM_NUM_3] = (u8)u8in;

	// vdata
	stMSPI.u8out[LDM_NUM_4] = 0;

	if (priv->st_iw7038.u8iw7038_check_sum_flag) {
		for (i = 0; i < ONEDATA_SIZE - 1; i++)
		   stMSPI.u8out[LDM_NUM_4] ^= stMSPI.u8out[i];
	}
	// dummy
	for (i = 0; i < u8dummy_no; i++)
		stMSPI.u8out[ONEDATA_SIZE + i] = 0;

	spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
}

void _mtk_ldm_send_iw7038_array(struct mtk_ldm_priv *priv, struct spi_slave *slave, u8 *pu8in, u32 u32size)
{
	u8 u8dummy_no = priv->st_iw7038.u8iw7038_dev_num - 1;
	u8 i;
	u8 u8check_sum = 0;
	u16 u16checksum = 0;

	//int ret = 0;
	//struct mtk_ldm_spi_large stMSPI;

	// 7038 daisy chain max devide number is 62
	if (priv->st_iw7038.u8iw7038_dev_num > IW7038_MAX_DEV_NUM) {
		UBOOT_DEBUG("[mboot_iw7038]MSPI_Write_Iw7038_SetDaisyChainChipNum number of device wrong!\n");
		return;
	}

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	stMSPI.u32out_bit_len = (u32size + IW7038_BYTE_NUM_OF_VDATA + u8dummy_no) * LDM_NUM_8;

	for (i = 0; i < u32size; i++) {
		stMSPI.u8out[i] = pu8in[i];
		if (priv->st_iw7038.u8iw7038_check_sum_flag)
			u16checksum += pu8in[i];
	}

	// vdata
	if (priv->st_iw7038.u8iw7038_check_sum_flag) {
		u16checksum = ((u16checksum / LDM_NUM_256) & MASK_0XFF) + (u16checksum & MASK_0XFF);
		u16checksum = ((u16checksum / LDM_NUM_256) & MASK_0XFF) + (u16checksum & MASK_0XFF);
		u8check_sum = u16checksum;
		u8check_sum = ~u8check_sum;
		stMSPI.u8out[u32size] = u8check_sum;
	} else {
		stMSPI.u8out[u32size] = 0;
	}

	// dummy
	for (i = 0; i < u8dummy_no; i++)
		stMSPI.u8out[u32size + IW7038_BYTE_NUM_OF_VDATA + i] = 0;

	spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
}

static int _mtk_ldm_spi_iw7038_init(struct mtk_ldm_priv *priv)
{
	struct spi_slave *slave;
	int ret = 0;
	int i, j, k;
	u8 temp1;
	u32 u32size = 0;

	char name[LDM_CHAR_NAME], *str;
	struct udevice *dev;

	bool b_check_pass = true;
	//bool bCalibrationDev1 = false;
	//bool bCalibrationDev2 = false;
	static u8 au8Data[LDM_NUM_1024];

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	// make sure vsync pull low
	ret = gpio_request(LDM_NUM_9, "gpio_set_on");

	// vsync low(pull down)
	gpio_direction_output(LDM_NUM_9, 0);
	gpio_set_value(LDM_NUM_9, 0);

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;

	//MSPI0 PAD init as GPIO, so here switch to MSPI0 function, for DCON vsync like PAD use as GPIO in mboot stage
	//W2BYTEMSK(0x2E4000, 0x5, Fld(4, 4, AC_MSKB0));	// [7:4] set 0 for MSPI0's SCZ
	//W2BYTEMSK(0x2E4000, 0x55, Fld(8, 8, AC_FULLB1));	// [3:0] set 5 for MSPI0's SCK,
														// [7:4] set 0 for MSPI0's MISO
	//W2BYTEMSK(0x2E4008, 0x05, Fld(8, 0, AC_FULLB0));	// [3:0] set 5 for MSPI0's MOSI,
														// [7:4] set 0 for vsync like PAD as GPIO

	//vsync pull high time set to 0 (like as vsync like keep low)
	_mtk_ldm_set_pwm_vsync_like_keep_low();

	ret = snprintf(name, sizeof(name), "generic_%d:%d", stMSPI.u32CH, 0);
	if (ret < 0)
		return ret;

	str = strdup(name);
	if (!str)
		return -ENOMEM;

	ret = spi_get_bus_and_cs(stMSPI.u32CH, 0, stMSPI.u32speed, stMSPI.u32mode, "spi_generic_drv", str, &dev, &slave);
	if (ret)
		return ret;

	ret = spi_claim_bus(slave);

	if (ret)
		goto done;

	//4.Dealy >= 8 ms
	mdelay(LDM_NUM_10);

	//5.Write reg. 0x00=0x12
	//6.Read back 0x00,if 0x00=0x12,go on to step 7,or back to step 5,loop util 0x00=0x12
	mtk_ldm_send_iw7038_Same(priv, slave, 0x000, LDM_NUM_0X12);
	mdelay(1);

	for (i = 0; i < LDM_NUM_256; i++) {
		b_check_pass = true;
		for (j = 1; j < (priv->st_iw7038.u8iw7038_dev_num + 1); j++) {
			temp1 = _mtk_ldm_read_iw7038(priv, slave, j, 0x000);
			udelay(1);
			if (temp1 != LDM_NUM_0X12)
				b_check_pass = false;
		}
		if (b_check_pass)
			break;
	}

	//7.Supply stable clock to IW7038
	//8.Generate external VSYNC to VSYNC/PWM pin.
	mdelay(LDM_NUM_16);

	//9.Configure SPI verification mode, set SPI mode to normal mode 0x80[7:5]
	temp1 = _mtk_ldm_read_iw7038(priv, slave, 1, IW7038_REG_0X080);
	mdelay(1);
	// set 0x80[7:5] to 0 for normal mode
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X080, (temp1 & MASK_0X1F));
	mdelay(1);

	//10.release protected register 0xA1=0xA3
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X0A1, IW7038_REG_0X0A1_RELEASE);
	mdelay(1);

	//11.SPI start to write register/configure init register
	//11.1 - 1.7038 in reset state;2.head shift;3.SPI mode
	//11.2 - Phase delay Sub_NUM(0~15)
	//11.3 - Channel delay
	//11.4 - Channel enable
	j = 0;
	for (i = 0; i < priv->st_iw7038.u8packnum; i++) {
		for (k = 0; k < priv->st_iw7038.u32len_eve_pack[i]; k++) {
			au8Data[k] = priv->st_iw7038.u8allpackdata[k + j];
			UBOOT_INFO("0x%x, ", au8Data[k]);
		}
		UBOOT_INFO("\n");

		u32size = priv->st_iw7038.u32len_eve_pack[i];
		_mtk_ldm_send_iw7038_array(priv, slave, au8Data, u32size);

		j += priv->st_iw7038.u32len_eve_pack[i];
		mdelay(LDM_NUM_3);
	}

	//12.Lock register: write protection
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X0A1, 0);
	mdelay(1);
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X09A, 0);
	mdelay(LDM_NUM_16);

	//W2BYTEMSK(0x2E4008, 0x15, Fld(8, 0, AC_FULLB0));
	mdelay(LDM_NUM_2);

	_mtk_ldm_set_pwm0_reset_en(0);
	_mtk_ldm_set_pwm_vsync_like(priv);
	_mtk_ldm_set_pwm_sel(priv);

	//mdelay(360);
	mdelay(LDM_NUM_256);

	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X09A, IW7038_REG_0X09A_RELEASE);   // control reg.
	mdelay(1);

	//14.Soft start IW7038 calibration process, set [SOFT_RST_N]=1 to exit reset state
	mtk_ldm_send_iw7038_Same(priv, slave, 0x000, IW7038_REG_0X000_START);
	mdelay(1);
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X09A, 0);   // control reg.
	mdelay(LDM_NUM_3);

	//15.Calibration done or not
	for (j = 1; j < (priv->st_iw7038.u8iw7038_dev_num + 1); j++) {
		for (i = 0; i < LDM_NUM_256; i++) {
			temp1 = _mtk_ldm_read_iw7038(priv, slave, j, IW7038_REG_0X142);
			mdelay(1);
			if (LDM_NUM_4 == (temp1 & MASK_0X7))
				break;
		}
	}
	mdelay(LDM_NUM_3);

	// set open, short fault don't disable channel automatically
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X09A, IW7038_REG_0X09A_RELEASE);   // control reg.
	mdelay(1);
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X023, IW7038_REG_0X023_INIT);   // control reg.
	mdelay(1);
	// set 0x80[7:5] to 0 for normal mode
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X080, IW7038_REG_0X080_INIT);
	mdelay(LDM_NUM_3);
	mtk_ldm_send_iw7038_Same(priv, slave, IW7038_REG_0X09A, 0);   // control reg.
	mdelay(1);

done:
	spi_release_bus(slave);

	return ret;
}

static int _mtk_ldm_spi_bcon_onepath_common_init(struct mtk_ldm_priv *priv, u8 u8CH)
{
	//static struct mtk_ldm_spi_large stMSPI;
	struct spi_slave *slave;
	int ret = 0;
	int i;
	u16 u16check_sum = 0;
	u16 u16temp = 0;
	u32 u32temp = 0;
	u16 j;
	char name[LDM_CHAR_NAME], *str;
	struct udevice *dev;

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	// for DCON, MSPI0 PAD init as GPIO, so here switch to MSPI0 function,
	// for DCON vsync like PAD use as GPIO in mboot stage
	//W2BYTEMSK(0x2E4000, 0x5, Fld(4, 4, AC_MSKB0));	// [7:4] set 0 for MSPI0's SCZ
	//W2BYTEMSK(0x2E4000, 0x55, Fld(8, 8, AC_FULLB1));	// [3:0] set 5 for MSPI0's SCK,
														// [7:4] set 0 for MSPI0's MISO
	//W2BYTEMSK(0x2E4008, 0x05, Fld(8, 0, AC_FULLB0));	// [3:0] set 5 for MSPI0's MOSI,
														// [7:4] set 0 for vsync like PAD as GPIO

	//vsync pull high time set to 0 (like as vsync like keep low)
	_mtk_ldm_set_pwm_vsync_like_keep_low();

	ret = gpio_request(LDM_NUM_9, "gpio_set_on");

	// vsync low(pull down)
	gpio_direction_output(LDM_NUM_9, 0);
	gpio_set_value(LDM_NUM_9, 0);

	stMSPI.u32CH = u8CH;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	u32temp = priv->st_mtk_ldm_panel_info.u16led_width * priv->st_mtk_ldm_panel_info.u16led_height;
	stMSPI.u32out_bit_len = (LDM_NUM_2 + LDM_NUM_2 + u32temp * LDM_NUM_2 + LDM_NUM_2) * LDM_NUM_8;

	ret = snprintf(name, sizeof(name), "generic_%d:%d", stMSPI.u32CH, 0);
	if (ret < 0)
		return ret;

	str = strdup(name);
	if (!str)
		return -ENOMEM;

	ret = spi_get_bus_and_cs(stMSPI.u32CH, 0, stMSPI.u32speed, stMSPI.u32mode, "spi_generic_drv", str, &dev, &slave);
	if (ret)
		return ret;

	ret = spi_claim_bus(slave);
	if (ret)
		goto done;

	mdelay(LDM_NUM_10);

	for (j = 0; j < LDM_NUM_3; j++) {
		//1.2.1 Configure the iw7038 EN GPIO4 to high(pull up)
		gpio_direction_output(LDM_NUM_9, 1);
		gpio_set_value(LDM_NUM_9, 1);

		udelay(LDM_NUM_30);

		//1.2.1 Configure the iw7038 EN GPIO4 to low(pull down)
		gpio_direction_output(LDM_NUM_9, 0);
		gpio_set_value(LDM_NUM_9, 0);

		// indecate
		stMSPI.u8out[0] = BCON_DCON_HEADER_0XAA;
		stMSPI.u8out[1] = BCON_DCON_HEADER_0X55;
		u16check_sum = BCON_DCON_HEADER_0XAA55;
		// cmd
		stMSPI.u8out[LDM_NUM_2] = 0x00;
		stMSPI.u8out[LDM_NUM_3] = 0x00;
		u16check_sum ^= 0x0000;
		// data
		for (i = 0; i < priv->st_mtk_ldm_panel_info.u16led_width * priv->st_mtk_ldm_panel_info.u16led_height; i++) {
			if (i > (MAX_LDM_SPI_BYTES_LARGE / LDM_NUM_8 - LDM_NUM_8))
				break;

			u16temp = priv->st_mtk_ldm_dma_info.u16_a_dimming_val;
			u16temp <<= priv->st_mtk_ldm_dma_info.u8_p_dimming_bits;
			// default is 1/2 p dimming max value
			u16temp += (priv->st_mtk_ldm_dma_info.u16_p_dimming_max / LDM_NUM_2);

			stMSPI.u8out[LDM_NUM_4 + i * LDM_NUM_2] = ((u16temp >> SHIFT_8) & MASK_0XFF);
			stMSPI.u8out[LDM_NUM_4 + i * LDM_NUM_2 + 1] = (u16temp & MASK_0XFF);
			u16check_sum ^= u16temp;
		}
		//check sum
		stMSPI.u8out[LDM_NUM_4 + i * LDM_NUM_2] = ((u16check_sum >> SHIFT_8) & MASK_0XFF);
		stMSPI.u8out[LDM_NUM_4 + i * LDM_NUM_2 + 1] = (u16check_sum & MASK_0XFF);
		u32temp = priv->st_mtk_ldm_panel_info.u16led_width * priv->st_mtk_ldm_panel_info.u16led_height;
		// idicator(2)+cmd(2)+H*v*2+checksum(2)
		stMSPI.u32out_bit_len = (LDM_NUM_2 + LDM_NUM_2 + u32temp * LDM_NUM_2 + LDM_NUM_2) * LDM_NUM_8;
		stMSPI.u32in_tit_len = 0;
		spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
	}
	mdelay(LDM_NUM_16);
done:
	spi_release_bus(slave);
	return ret;
}

static int _mtk_ldm_spi_dcon_onepath_common_init(struct mtk_ldm_priv *priv, u8 u8CH)
{
	//static struct mtk_ldm_spi_large stMSPI;
	struct spi_slave *slave;
	int ret = 0;
	int i;
	u16 u16check_sum = 0;
	u16 u16temp = 0;
	u16 j;
	u32 u32temp = 0;
	char name[LDM_CHAR_NAME], *str;
	struct udevice *dev;

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	// for DCON, MSPI0 PAD init as GPIO, so here switch to MSPI0 function,
	// for DCON vsync like PAD use as GPIO in mboot stage
	//W2BYTEMSK(0x2E4000, 0x5, Fld(4, 4, AC_MSKB0));	// [7:4] set 0 for MSPI0's SCZ
	//W2BYTEMSK(0x2E4000, 0x55, Fld(8, 8, AC_FULLB1));	// [3:0] set 5 for MSPI0's SCK,
														// [7:4] set 0 for MSPI0's MISO
	//W2BYTEMSK(0x2E4008, 0x05, Fld(8, 0, AC_FULLB0));	// [3:0] set 5 for MSPI0's MOSI,
														// [7:4] set 0 for vsync like PAD as GPIO

	//vsync pull high time set to 0 (like as vsync like keep low)
	_mtk_ldm_set_pwm_vsync_like_keep_low();

	ret = gpio_request(LDM_NUM_9, "gpio_set_on");

	// vsync low(pull down)
	gpio_direction_output(LDM_NUM_9, 0);
	gpio_set_value(LDM_NUM_9, 0);

	stMSPI.u32CH = u8CH;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	u32temp = priv->st_mtk_ldm_panel_info.u16led_width * priv->st_mtk_ldm_panel_info.u16led_height;
	stMSPI.u32out_bit_len = (LDM_NUM_2 + LDM_NUM_2 + LDM_NUM_2 + u32temp * LDM_NUM_2 + LDM_NUM_2) * LDM_NUM_8;

	ret = snprintf(name, sizeof(name), "generic_%d:%d", stMSPI.u32CH, 0);
	if (ret < 0)
		return ret;

	str = strdup(name);
	if (!str)
		return -ENOMEM;

	ret = spi_get_bus_and_cs(stMSPI.u32CH, 0, stMSPI.u32speed, stMSPI.u32mode, "spi_generic_drv", str, &dev, &slave);
	if (ret)
		return ret;

	ret = spi_claim_bus(slave);
	if (ret)
		goto done;

	mdelay(LDM_NUM_10);

	for (j = 0; j < LDM_NUM_6; j++) {
		//1.2.1 Configure the iw7038 EN GPIO4 to high(pull up)
		gpio_direction_output(LDM_NUM_9, 1);
		gpio_set_value(LDM_NUM_9, 1);

		udelay(LDM_NUM_30);

		// vsync low(pull down)
		gpio_direction_output(LDM_NUM_9, 0);
		gpio_set_value(LDM_NUM_9, 0);

		// indecate
		stMSPI.u8out[0] = BCON_DCON_HEADER_0XAA;
		stMSPI.u8out[1] = BCON_DCON_HEADER_0X55;
		u16check_sum = BCON_DCON_HEADER_0XAA55;
		// cmd1
		stMSPI.u8out[LDM_NUM_2] = 0x00;
		stMSPI.u8out[LDM_NUM_3] = 0x00;
		u16check_sum ^= 0x0000;
		// cmd2
		stMSPI.u8out[LDM_NUM_4] = 0x00;
		stMSPI.u8out[LDM_NUM_5] = 0x00;
		u16check_sum ^= 0x0000;
		// data
		for (i = 0; i < (priv->st_mtk_ldm_panel_info.u16led_width * priv->st_mtk_ldm_panel_info.u16led_height); i++) {
			if (i > (MAX_LDM_SPI_BYTES_LARGE / LDM_NUM_8 - LDM_NUM_8))
				break;

			u16temp = priv->st_mtk_ldm_dma_info.u16_a_dimming_val;
			u16temp <<= priv->st_mtk_ldm_dma_info.u8_p_dimming_bits;
			// default is 1/2 p dimming max value
			u16temp += (priv->st_mtk_ldm_dma_info.u16_p_dimming_max / LDM_NUM_2);

			stMSPI.u8out[LDM_NUM_6 + i * LDM_NUM_2] = ((u16temp >> SHIFT_8) & MASK_0XFF);
			stMSPI.u8out[LDM_NUM_6 + i * LDM_NUM_2 + 1] = (u16temp & MASK_0XFF);
			u16check_sum ^= u16temp;
		}
		//check sum
		stMSPI.u8out[LDM_NUM_6 + i * LDM_NUM_2] = ((u16check_sum >> SHIFT_8) & MASK_0XFF);
		stMSPI.u8out[LDM_NUM_6 + i * LDM_NUM_2 + 1] = (u16check_sum & MASK_0XFF);
		u32temp = priv->st_mtk_ldm_panel_info.u16led_width * priv->st_mtk_ldm_panel_info.u16led_height;
		// idicator(2)+cmd1(2)+cmd2(2)+H*v*2+checksum(2)
		stMSPI.u32out_bit_len = (LDM_NUM_2 + LDM_NUM_2 + LDM_NUM_2 + u32temp * LDM_NUM_2 + LDM_NUM_2) * LDM_NUM_8;
		stMSPI.u32in_tit_len = 0;
		spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
	}
	mdelay(LDM_NUM_16);
done:
	spi_release_bus(slave);
	return ret;
}

void _mtk_ldm_send_iw7039_array(struct mtk_ldm_priv *priv, struct spi_slave *slave, u8 *pu8in, u32 u32size)
{
	u8 u8dummy_no = ((priv->stiw7039.u8iw7039_dev_num + IW7039_DUMMY_CAL_NUM - 1) / IW7039_DUMMY_CAL_NUM);
	u8 i;
	u16 u16temp = 0;
	u16 u16checksum = 0;

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));

	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;
	stMSPI.u32out_bit_len = (u32size + IW7039_BYTE_NUM_OF_VDATA + u8dummy_no * LDM_NUM_2) * LDM_NUM_8;

	for (i = 0; i < u32size; i++) {
		stMSPI.u8out[i] = pu8in[i];
		if (priv->stiw7039.u8iw7039_check_sum_flag)
			if (0 == (i % LDM_NUM_2)) {
				u16temp = pu8in[i] * LDM_NUM_256 + pu8in[i + 1];
				u16checksum += u16temp;
			}
	}

	// vdata
	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		u16checksum = ~u16checksum;
		stMSPI.u8out[u32size] = (u8)(u16checksum >> SHIFT_8);
		stMSPI.u8out[u32size + 1] = (u8)((u16checksum & MASK_0XFF));
	} else {
		stMSPI.u8out[u32size] = 0;
		stMSPI.u8out[u32size + 1] = 0;
	}

	// dummy
	for (i = 0; i < u8dummy_no; i++) {
		stMSPI.u8out[u32size + IW7039_BYTE_NUM_OF_VDATA + i * LDM_NUM_2] = 0;
		stMSPI.u8out[u32size + IW7039_BYTE_NUM_OF_VDATA + i * LDM_NUM_2 + 1] = 0;
	}

	spi_xfer(slave, stMSPI.u32out_bit_len, stMSPI.u8out, NULL, SPI_XFER_BEGIN | SPI_XFER_END);
}

// _mtk_ldm_iw7039_init_v2: use IW7039_Packet_Num & IW7039_Length_Of_Every_Packet &
// IW7039_All_Packet_Data solution init iw7039
int32_t _mtk_ldm_iw7039_init_v2(struct mtk_ldm_priv *priv, uint8_t u8CH)
{
	struct spi_slave *slave;
	int ret = 0;
	int i, j, k;
	u32 u32size = 0;
	u8 u8chip_idx = 1;

	char name[LDM_CHAR_NAME], *str;
	struct udevice *dev;

	u8 u8SPI_Devicenum;
	u16 u16read_val = 0;
	u8 u8retry = 0;
	struct udevice *mainlddev;
	static u8 au8Data[LDM_NUM_1024];

	u16 intReg_Conf[REG_0X00_TO_0X1F_NUM] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											//0X00 ~ 0X07 , reg 0x001 config to use global ISET with 30mA,
											// reg 0x05 config to 4x Vsync
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											// 0X08 ~ 0X0F ( 0x20 set ILED range as 60mA)
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											// 0X10 ~ 0X17 , Reg 0x10 , 0x11 enable 32 CHs, Reg 0x14 turn-off all fault,
											// ADJ-short doesn¡¯t turn off
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
											//0X18 ~0X1F , reg 0x18 and 0x19 enable ADP for 32 CHs
											}; //Initializing

	u16 intReg_ISET[IW7039_PWM_ISET_NUM] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x20 ~ 0x27
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x28 ~ 0x2F
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x30 ~ 0x37
											0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x38 ~ 0x3F
											};//0x20 to 0x3F

	u16 intReg_DT[IW7039_DT_NUM] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x40 ~ 0x47
									0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x48 ~ 0x4F
									0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x50 ~ 0x57
									0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,	// 0x58 ~ 0x5F
									};//0x40 to 0x5F

	u16 intReg_HT[IW7039_PWM_HTIME_NUM] = {0, 0, 0, 0, 0, 0, 0, 0, // 0x60 ~ 0x67
											0, 0, 0, 0, 0, 0, 0, 0, // 0x68 ~ 0x6F
											0, 0, 0, 0, 0, 0, 0, 0, // 0x70 ~ 0x77
											0, 0, 0, 0, 0, 0, 0, 0, // 0x78 ~ 0x7F
											} ;//0x60 to 0x7F

	// make sure vsync pull low
	ret = gpio_request(LDM_NUM_9, "gpio_set_on");

	// vsync low(pull down)
	gpio_direction_output(LDM_NUM_9, 0);
	gpio_set_value(LDM_NUM_9, 0);

	// need control IW7039 EN?
	if (priv->stiw7039.b_iw7039_need_en_pad_flag) {
		// for 7039_EN_PIN
		// use GPIO 60 as iw7039 EN
	    ret = gpio_request(priv->stiw7039.u8iw7039_en_pad_num, "gpio_set_on");

	    //1.2.1 Configure the iw7039 EN to low(pull down)
	    // set EN GPIO as output direction and pull low(0)
	    gpio_direction_output(priv->stiw7039.u8iw7039_en_pad_num, 0);
	    gpio_set_value(priv->stiw7039.u8iw7039_en_pad_num, 0);	// set EN GPIO pull low(0)
	}

	msleep(SLEEP_TIME_100);

	memset((void *)&stMSPI, 0, sizeof(struct mtk_ldm_spi_large));
	stMSPI.u32CH = priv->st_mtk_ldm_mspi_info.u8mspi_channel;
	stMSPI.u32mode = priv->st_mtk_ldm_mspi_info.u8mspi_mode;
	stMSPI.u32speed = priv->st_mtk_ldm_mspi_info.u32mspi_clk;

	//MSPI0 PAD init as GPIO, so here switch to MSPI0 function, for DCON vsync like PAD use as GPIO in mboot stage
	//W2BYTEMSK(0x2E4000, 0x5, Fld(4, 4, AC_MSKB0));	// [7:4] set 0 for MSPI0's SCZ
	//W2BYTEMSK(0x2E4000, 0x55, Fld(8, 8, AC_FULLB1));	// [3:0] set 5 for MSPI0's SCK,
														// [7:4] set 0 for MSPI0's MISO
	//W2BYTEMSK(0x2E4008, 0x05, Fld(8, 0, AC_FULLB0));	// [3:0] set 5 for MSPI0's MOSI,
														// [7:4] set 0 for vsync like PAD as GPIO

	//1.2.1 Configure the iw7039 vsync to low(pull down)
	//action: set start/end of vsync pull high time to 0 (like as vsync like keep low)
	_mtk_ldm_set_pwm_vsync_like_keep_low();

	ret = snprintf(name, sizeof(name), "generic_%d:%d", stMSPI.u32CH, 0);
	if (ret < 0)
		return ret;

	str = strdup(name);
	if (!str)
		return -ENOMEM;

	ret = spi_get_bus_and_cs(stMSPI.u32CH, 0, stMSPI.u32speed, stMSPI.u32mode, "spi_generic_drv", str, &dev, &slave);
	if (ret)
		return ret;

	ret = spi_claim_bus(slave);

	if (ret)
		goto done;

	//2. set the iw7039 EN GPIO 60 to high(pull up)
	// need control IW7039 EN?
	if (priv->stiw7039.b_iw7039_need_en_pad_flag) {
		// for 7039_EN_PIN
	    gpio_direction_output(priv->stiw7039.u8iw7039_en_pad_num, 1);
	    gpio_set_value(priv->stiw7039.u8iw7039_en_pad_num, 1);
	}

    //3.Dealy >= 8.9 ms, waiting for iw7039 logic table after EN pull high
	msleep(SLEEP_TIME_10);

	//4.SPI communication check

    // 4.1 Send data 0xFE08 to the daisy chain, Set the align command by daisy chain,
    // depended on how many devices on a daisy chain.

	//if (priv->st_mtk_ldm_device_info.u32device_num[u8CH] > MAX_LDM_SPI_BYTES)
	//	priv->st_mtk_ldm_device_info.u32device_num[u8CH] = MAX_LDM_SPI_BYTES;

	u8SPI_Devicenum = priv->stiw7039.u8iw7039_dev_num;

	_mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(priv, u8CH, u8SPI_Devicenum);
	udelay(SLEEP_TIME_30);
	_mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(priv, u8CH, u8SPI_Devicenum);
	udelay(SLEEP_TIME_30);

	//4.2 read back the chip1 CN number in 0x344[15:8], the first chip in the daisy chain

    // 4.3 check whether the 0x344[15:8] == 0x04 or not?
    // if not, go back to the step 4.1;
    // otherwise go to the next step

    //read back the chip 1 SN number in 0x344[15:8] is 0x8 or not? default SPI mode is normal mode
	u8retry = 0;
	do {
		u16read_val = _mtk_ldm_read_iw7039_one(priv, u8CH, u8SPI_Devicenum, 1, IW7039_0x344_ADDR);
		u16read_val = (u16read_val >> SHIFT_2BYTES) & MASK_2BYTES;
		u8retry++;
		_mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(priv, u8CH, u8SPI_Devicenum);
		udelay(SLEEP_TIME_100);
	} while ((u16read_val != IW7039_POWER_ON) && (u8retry <= RETRY_COUNTER));

	//if (u16read_val == IW7039_POWER_ON)
	//	UBOOT_DEBUG("IW7039 CH:%d power ON OK\n", u8CH);
	//else
	//	UBOOT_ERROR("IW7039 CH:%d power Off!!!!!!!!!!!!!!!!!!\n", u8CH);

	//if (pm_check_back_ground_active() == 0) {	// non QHb
		//6.Supply stable and correct Vsync to IW7039
		ret = uclass_get_device_by_name(UCLASS_DISPLAY, "mediatek-ldm", &mainlddev);
		pinctrl_select_state(mainlddev, "vsync_like1_pad_mux");
		// [3:0] set 5 for MSPI0's MOSI[7:4] set 1 for vsync like PAD as vsync like function
		//W2BYTEMSK(0x2E4008, 0x15, Fld(8, 0, AC_FULLB0));
		udelay(SLEEP_TIME_100);

		_mtk_ldm_set_pwm0_reset_en(0);
		_mtk_ldm_set_pwm_vsync_like(priv);
		_mtk_ldm_set_pwm_sel(priv);
		msleep(SLEEP_TIME_10);
	//}

	// 7.Write 0x000=0x0802, Configure SPI verification mode,
	// checksum mode ignore carry bit, set SPI mode to normal mode
	_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, 0x0000, IW7039_INIT_REG0_VALUE);
	udelay(SLEEP_TIME_30);

	//8.Write 0x0A0=0xA5FF (Default value), release protected register
	_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, IW7039_LOCK_ADDR, IW7039_UNLOCK);
	udelay(SLEEP_TIME_30);

	//9.init CONFIG, ISET, DT, HT

	//Init ISET 0x020 ~ 0x03F
	UBOOT_DEBUG("IW7039 init ISET\n");
	for (i = 0; i < IW7039_PWM_ISET_NUM ; i++)
		intReg_ISET[i] = priv->stiw7039.u16IW7039_ISET_MBR_OFF;

	_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_ISET_ADDR, intReg_ISET);
	msleep(SLEEP_TIME_10);

    //Init DT 0x040 ~ 0x05F
	UBOOT_DEBUG("IW7039 init DT\n");
	for (i = 0; i < IW7039_DT_NUM ; i++)
		intReg_DT[i] = priv->stiw7039.u32IW7039_DT[i];

	_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_DT_ADDR, intReg_DT);
	msleep(SLEEP_TIME_10);

	//Init HT 0x060 ~ 0x07F
	for (i = 0; i < IW7039_PWM_HTIME_NUM ; i++)
		intReg_HT[i] = priv->stiw7039.u16iw7039_pwm_duty_init;

	_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_PWM_HTIME1_ADDR, intReg_HT);
	msleep(SLEEP_TIME_10);

	//Init Config 0x000 ~ 0x01F
	UBOOT_DEBUG("IW7039 init table\n");
	printf("IW7039 init table\n");
	for (i = 0; i < REG_0X00_TO_0X1F_NUM ; i++)
		intReg_Conf[i] = priv->stiw7039.u32iw7039_teg_0x000_to_0x01F[i];

	_mtk_ldm_w_all_iw7039_same32(priv, u8CH, u8SPI_Devicenum, IW7039_CONFIG_ADDR, intReg_Conf);
	msleep(SLEEP_TIME_10);

	// Init every iw7039 device different register value
	j = 0;
	for (i = 0; i < priv->stiw7039.u8iw39_packnum; i++) {
		for (k = 0; k < priv->stiw7039.u32iw39_evepack[i]; k++) {
			au8Data[k] = priv->stiw7039.u8iw7039_allpackdata[k + j];
			UBOOT_INFO("0x%x, ", au8Data[k]);
			printf("0x%x, ", au8Data[k]);
		}
		UBOOT_INFO("\n");
		printf("\n");

		u32size = priv->stiw7039.u32iw39_evepack[i];
		_mtk_ldm_send_iw7039_array(priv, slave, au8Data, u32size);

		j += priv->stiw7039.u32iw39_evepack[i];
		msleep(SLEEP_TIME_10);
	}

	//mdelay(1);
	//10.lock register: Eanbel the DT registers write protection
	_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, IW7039_LOCK_ADDR, IW7039_LOCK);
	udelay(SLEEP_TIME_30);

	//11.Delay >= 100 ms
	msleep(SLEEP_TIME_100); //110
	msleep(SLEEP_TIME_10);

	//if (pm_check_back_ground_active() != 0)	// QHb
	//	return 0;

	if (priv->stiw7039.u8iw7039_check_sum_flag) {
		//12.Soft start IW7039 calibration process with checksum
		_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, 0x0000, IW7039_VDATA_SOFT_START);
		udelay(SLEEP_TIME_30);
	} else {
		//12.Soft start IW7039 calibration process
		_mtk_ldm_w_all_iw7039_sameone(priv, u8CH, u8SPI_Devicenum, 0x0000, IW7039_SOFT_START);
		udelay(SLEEP_TIME_30);
	}

	//13.Calibration done or not?
	//13.1 Send data 0xFE0A,0x0000 to the daisy chain; // Set the align command by daisy chain,
	_mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(priv, u8CH, u8SPI_Devicenum);
	udelay(SLEEP_TIME_30);

	//13.2 Read back 0x34A[14:9] from the first to the last chip

	// 13.3 Check whether all chip's 0x34A[14:9] == 0x10 or not? if not,
	// go back to the step 13.1; otherwise, go to the next step
	for (u8chip_idx = 1; u8chip_idx <= u8SPI_Devicenum; u8chip_idx++) {
		for (u8retry = 0; u8retry < RETRY_COUNTER; u8retry++) {
			u16read_val = _mtk_ldm_read_iw7039_one(priv, u8CH, u8SPI_Devicenum, u8chip_idx, IW7039_0x34A_ADDR);
			udelay(SLEEP_TIME_30);
			u16read_val = (u16read_val >> LDM_NUM_9) & MASK_0X3F;

			if (u16read_val == LDM_NUM_0X10)
				break;

			_mtk_ldm_mspi_write_iw7039_set_daisy_chain_chip_num(priv, u8CH, u8SPI_Devicenum);
			udelay(SLEEP_TIME_30);
		}
	}

	//14. Enter OTF Status: Can start to update the local dimming data that get from the system.

done:
	spi_release_bus(slave);

	return ret;
}

int32_t _mtk_ldm_cus_led_device_init(struct mtk_ldm_priv *priv, u8 u8CH)
{
	s32 ret = 0;
	u8 au8WData[LDM_NUM_256] = {0};
	struct mtk_ldm_spi_data WData;
	struct mtk_ldm_spi_data RData;

	memset((void *)&WData, 0, sizeof(struct mtk_ldm_spi_data));
	memset((void *)&RData, 0, sizeof(struct mtk_ldm_spi_data));

	if ((priv->st_mtk_ldm_device_info.u32device_num[u8CH] + LDM_NUM_10) >= MAX_LDM_SPI_BYTES) {
		debug("[%s,%5d] Over Size!!!\n", __func__, __LINE__);
		return 0;
	}
	WData.u16len = LDM_NUM_10 + priv->st_mtk_ldm_device_info.u32device_num[u8CH];
	RData.u16len = 0;

	memcpy(&WData.au8Data, au8WData, sizeof(au8WData));
	ret = _mtk_ldm_spi_data(priv, u8CH, &WData, &RData);
	if (ret)
		UBOOT_DEBUG("SPI transfer error = %d!!!\n", ret);

	return ret;
}

s32 _mtk_ldm_leddevice_init(struct mtk_ldm_priv *priv)
{
	s32 ret = 0;
	u8 i = 0;
	u8 u8ch = 0;

	if (!priv) {
		debug("[%s] get device private fail\n", __func__);
		return ret;
	}

	if (priv->u8ldm_ver == LDM_VER_1 || priv->u8ldm_ver == LDM_VER_2 || priv->u8ldm_ver == LDM_VER_3)
		u8ch = MSPI_MAX_NUM;
	else
		u8ch = BYTE2;

	_mtk_ldm_set_pwm_init(priv);
	switch (priv->st_mtk_ldm_device_info.u8led_device_type) {
	case E_LD_DEVICE_UNSUPPORT:
		UBOOT_DEBUG("LD Device is not support\n");
		break;
	case E_LD_DEVICE_AS3824:
		UBOOT_DEBUG("LD Device AS3824 init start\n");
		for (i = 0; i < u8ch; i++) {
			if (priv->st_mtk_ldm_device_info.u32device_num[i] > 0) {
				_mtk_ldm_set_spi_Init(priv, i);
				_mtk_ldm_as3824_init(priv, i);
			}
		}
		break;
	case E_LD_DEVICE_NT50585:
		UBOOT_DEBUG("LD Device NT50585 init start\n");
		break;
	case E_LD_DEVICE_IW7039_CN:
		UBOOT_DEBUG("LD Device IW7039_CN init start\n");
		_mtk_ldm_set_spi_Init(priv, 0);
		_mtk_ldm_iw7039_init_v2(priv, 0);
		break;
	case E_LD_DEVICE_IW7039:
		UBOOT_DEBUG("LD Device IW7039 init start\n");
		for (i = 0; i < u8ch; i++) {
			if (priv->st_mtk_ldm_device_info.u32device_num[i] > 0) {
				_mtk_ldm_set_spi_Init(priv, i);
				_mtk_ldm_iw7039_init(priv, i);
			}
		}
		for (i = 0; i < u8ch; i++) {
			if (priv->st_mtk_ldm_device_info.u32device_num[i] > 0)
				_mtk_ldm_iw7039_init_turn_on_backlight(priv, i);
		}
		break;
	case E_LD_DEVICE_MCU:
		UBOOT_DEBUG("LD Device MCU init start\n");
		break;
	case E_LD_DEVICE_CUS:
		UBOOT_DEBUG("LD Device CUS init start\n");

		for (i = 0; i < u8ch; i++) {
			if (priv->st_mtk_ldm_device_info.u32device_num[i] > 0)
				_mtk_ldm_cus_led_device_init(priv, i);
		}
		break;
	case E_LD_DEVICE_MBI6353:
		UBOOT_DEBUG("[LDM]LD Device MBI6353 init start\n");
		for (i = 0; i < u8ch; i++) {
			if (priv->st_mtk_ldm_device_info.u32device_num[i] > 0)
				_mtk_ldm_mbi6353_init(priv, i);
			}
		break;
	case E_LD_DEVICE_IW7027:
		UBOOT_DEBUG("[LDM]LD Device IW7027 init start\n");

		_mtk_ldm_set_spi_Init(priv, 0);
		_mtk_ldm_iw7027_init(priv, 0);
		break;
	case E_LD_DEVICE_IW7038:
		//ulStart = get_timer(0);
		//UBOOT_DEBUG("[%s][%d]iw7038_test!\n", __FUNCTION__,__LINE__);
		UBOOT_DEBUG("LD Device IW7038 common init start\n");

		_mtk_ldm_set_spi_Init(priv, 0);
		_mtk_ldm_spi_iw7038_init(priv);
		break;
	case E_LD_DEVICE_BCON_ONEPATH:
		UBOOT_DEBUG("LD Device BCON One Path init start\n");

		_mtk_ldm_set_spi_Init(priv, 0);
		_mtk_ldm_spi_bcon_onepath_common_init(priv, 0);
		break;
	case E_LD_DEVICE_DCON_ONEPATH:
		UBOOT_DEBUG("LD Device DCON One Path init start\n");

		_mtk_ldm_set_spi_Init(priv, 0);
		_mtk_ldm_spi_dcon_onepath_common_init(priv, 0);
		break;
	default:
		UBOOT_DEBUG("LD Device is not support\n");
		break;
	}

	return ret;
}

static int mtk_ldm_ofdata_to_platdata(struct udevice *dev)
{
	//dump device tree.
	struct mtk_ldm_priv *priv = dev_get_priv(dev);
	ofnode info_node;
	u32 u32Data;
	u32 u32alldatanum = 0;
	int ret = 0;
	int i = 0;

	if (!priv) {
		dev_err(dev, "Get device priv FAIL\n");
		return -EINVAL;
	}

	dev_read_u32(dev, LDM_TAG, &u32Data);
	priv->u32ldm_support = u32Data;
	dev_read_u32(dev, LDM_VERSION, &u32Data);
	priv->u8ldm_ver = (u8)u32Data;
	dev_read_u32(dev, LDM_MAKER, &u32Data);
	priv->bldm_cus_pathu_boot = (bool)u32Data;
	UBOOT_DEBUG("[%s]info=%d %d %d\n", __func__, priv->u32ldm_support, priv->bldm_cus_pathu_boot, priv->u8ldm_ver);

	dev_read_u32(dev, LED_MSPI_EN, &u32Data);
	priv->bled_mspi_en = (bool)u32Data;

	dev_read_u32(dev, MTK_DRM_DTS_LD_CHANGE_VSYNCLIKE_PADMUXMODE, &u32Data);
	priv->bChange_Vsynclike_PadmuxMode = (bool)u32Data;

	info_node = dev_read_subnode(dev, LDM_PANEL_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_PANEL_INFO);
	} else {
		ofnode_read_u32(info_node, LDM_LED_WIDTH, &u32Data);
		priv->st_mtk_ldm_panel_info.u16led_width = (u16)u32Data;
		ofnode_read_u32(info_node, LDM_LED_HEIGHT, &u32Data);
		priv->st_mtk_ldm_panel_info.u16led_height = (u16)u32Data;
	}

	info_node = dev_read_subnode(dev, LDM_DMA_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_DMA_INFO);
	} else {
		ofnode_read_u32_array(info_node, LDM_VSYNC_WIDTH,
				      priv->st_mtk_ldm_dma_info.au32ld_vsync_width, LDM_VSYNC_WIDTH_SIZE);
		ofnode_read_u32_array(info_node, LDM_PWM0_PERIOD,
				      priv->st_mtk_ldm_dma_info.au32ld_pwm0_period, LDM_PWM0_PERIOD_SIZE);
		ofnode_read_u32_array(info_node, LDM_PWM0_DUTY,
				      priv->st_mtk_ldm_dma_info.au32ld_pwm0_duty, LDM_PWM0_DUTY_SIZE);
		ofnode_read_u32_array(info_node, LDM_PWM0_SHIFT,
				      priv->st_mtk_ldm_dma_info.au32ld_pwm0_shift, LDM_PWM0_SHIFT_SIZE);
		ofnode_read_u32_array(info_node, LDM_PWM1_PERIOD,
				      priv->st_mtk_ldm_dma_info.au32ld_pwm1_period, LDM_PWM1_PERIOD_SIZE);
		ofnode_read_u32_array(info_node, LDM_PWM1_DUTY,
				      priv->st_mtk_ldm_dma_info.au32ld_pwm1_duty, LDM_PWM1_DUTY_SIZE);
		ofnode_read_u32_array(info_node, LDM_PWM1_SHIFT,
				      priv->st_mtk_ldm_dma_info.au32ld_pwm1_shift, LDM_PWM1_SHIFT_SIZE);

		ofnode_read_u32(info_node, A_DIMMING_VAL, &u32Data);
		priv->st_mtk_ldm_dma_info.u16_a_dimming_val = (u16)u32Data;
		ofnode_read_u32(info_node, P_DIMMING_BIT_NUM, &u32Data);
		priv->st_mtk_ldm_dma_info.u8_p_dimming_bits = (u8)u32Data;
		ofnode_read_u32(info_node, P_DIMMING_MAX, &u32Data);
		priv->st_mtk_ldm_dma_info.u16_p_dimming_max = (u16)u32Data;
	}

	info_node = dev_read_subnode(dev, LDM_LED_MSPI_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_LED_MSPI_INFO);
	} else {
		ofnode_read_u32(info_node, LDM_MSPICLK, &u32Data);
		priv->st_mtk_ldm_mspi_info.u32mspi_clk = u32Data;
		ofnode_read_u32(info_node, LDM_MSPIMODE, &u32Data);
		priv->st_mtk_ldm_mspi_info.u8mspi_mode = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_MSPICHANNEL, &u32Data);
		priv->st_mtk_ldm_mspi_info.u8mspi_channel = (u8)u32Data;
	}

	info_node = dev_read_subnode(dev, LDM_LED_DEVICE_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_LED_DEVICE_INFO);
	} else {
		ofnode_read_u32(info_node, LDM_LEDDEVICE_TYPE, &u32Data);
		priv->st_mtk_ldm_device_info.u8led_device_type = (uint8_t)u32Data;
		ofnode_read_u32_array(info_node, LDM_DEVICE_NUM, priv->st_mtk_ldm_device_info.u32device_num, MSPI_MAX_NUM);
	}

	info_node = dev_read_subnode(dev, LDM_DEVICE_AS3824_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_DEVICE_AS3824_INFO);
	} else {
		ofnode_read_u32(info_node, LDM_DEVICE_AS3824_RSENSE, &u32Data);
		priv->st_ldm_Device_AS3824.u16Rsense = (uint16_t)u32Data;
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_REG_01_TO_0B,
			priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x01_to_0x0B, Reg_0x01_to_0x0B_NUM);
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_REG_0E_TO_15,
			priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x0E_to_0x15, Reg_0x0E_to_0x15_NUM);
		ofnode_read_u32(info_node, LDM_DEVICE_AS3824_REG_67, &u32Data);
		priv->st_ldm_Device_AS3824.u8AS3824_Reg_0x67 = (uint8_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_AS3824_PWM_DUTY_INIT, &u32Data);
		priv->st_ldm_Device_AS3824.u16AS3824_PWM_Duty_Init = (uint16_t)u32Data;
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_PWM_PHASE_MBR_OFF,
			priv->st_ldm_Device_AS3824.u32AS3824_PWM_Phase_MBR_OFF, LD_MAX_HEIGHT);
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_PWM_PHASE_MBR_ON,
			priv->st_ldm_Device_AS3824.u32AS3824_PWM_Phase_MBR_ON, LD_MAX_HEIGHT);
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_REG_0C_0D_VDAC_MBR_OFF,
			priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x0C_0x0D_VDAC_MBR_OFF, VDAC_BYTE_NUM);
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_REG_0C_0D_VDAC_MBR_ON,
			priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x0C_0x0D_VDAC_MBR_ON, VDAC_BYTE_NUM);
		ofnode_read_u32_array(info_node, LDM_DEVICE_AS3824_REG_61_62_PLL_MULTI,
			priv->st_ldm_Device_AS3824.u32AS3824_Reg_0x61_0x62_PLL_multi, PLL_MULTI_BYTE_NUM);
		ofnode_read_u32(info_node, LDM_DEVICE_AS3824_BDAC_HIGH_LIMIT, &u32Data);
		priv->st_ldm_Device_AS3824.u8AS3824_BDAC_High_Limit = (uint8_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_AS3824_BDAC_LOW_LIMIT, &u32Data);
		priv->st_ldm_Device_AS3824.u8AS3824_BDAC_Low_Limit = (uint8_t)u32Data;
	}

	info_node = dev_read_subnode(dev, LDM_DEVICE_NT50585_INFO);
	if(!ofnode_valid(info_node))
	{
		debug("Read subnode (%s) FAIL\n", LDM_DEVICE_NT50585_INFO);
	}
	else
	{
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_REG_01_TO_0B,
			priv->st_ldm_Device_NT50585.u32NT50585_Reg_0x01_to_0x0B, Reg_0x01_to_0x0B_NUM);
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_REG_1D_TO_1F,
			priv->st_ldm_Device_NT50585.u32NT50585_Reg_0x1D_to_0x1F, Reg_0x1D_to_0x1F_NUM);
		ofnode_read_u32(info_node, LDM_DEVICE_NT50585_REG_60, &u32Data);
		priv->st_ldm_Device_NT50585.u8NT50585_Reg_0x60 = (uint8_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_NT50585_REG_68, &u32Data);
		priv->st_ldm_Device_NT50585.u8NT50585_Reg_0x68 = (uint8_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_NT50585_PWM_DUTY_INIT, &u32Data);
		priv->st_ldm_Device_NT50585.u16NT50585_PWM_Duty_Init = (uint16_t)u32Data;
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_PWM_PHASE_MBR_OFF,
			priv->st_ldm_Device_NT50585.u32NT50585_PWM_Phase_MBR_OFF, LD_MAX_HEIGHT);
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_PWM_PHASE_MBR_ON,
			priv->st_ldm_Device_NT50585.u32NT50585_PWM_Phase_MBR_ON, LD_MAX_HEIGHT);
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_REG_14_15_IDAC_MBR_OFF,
			priv->st_ldm_Device_NT50585.u32NT50585_Reg_0x14_0x15_IDAC_MBR_OFF, VDAC_BYTE_NUM);
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_REG_14_15_IDAC_MBR_ON,
			priv->st_ldm_Device_NT50585.u32NT50585_Reg_0x14_0x15_IDAC_MBR_ON, VDAC_BYTE_NUM);
		ofnode_read_u32_array(info_node, LDM_DEVICE_NT50585_REG_66_67_PLL_MULTI,
			priv->st_ldm_Device_NT50585.u32NT50585_Reg_0x66_0x67_PLL_multi, PLL_MULTI_BYTE_NUM);
		ofnode_read_u32(info_node, LDM_DEVICE_NT50585_BDAC_HIGH_LIMIT, &u32Data);
		priv->st_ldm_Device_NT50585.u16NT50585_BDAC_High_Limit = (uint16_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_NT50585_BDAC_LOW_LIMIT, &u32Data);
		priv->st_ldm_Device_NT50585.u16NT50585_BDAC_Low_Limit = (uint16_t)u32Data;
	}

	info_node = dev_read_subnode(dev, LDM_DEVICE_IW7039_INFO);
	if(!ofnode_valid(info_node))
	{
		debug("Read subnode (%s) FAIL\n", LDM_DEVICE_IW7039_INFO);
	}
	else
	{
		ofnode_read_u32_array(info_node, LDM_DEVICE_IW7039_REG_00_TO_1F,
			priv->stiw7039.u32iw7039_teg_0x000_to_0x01F, REG_0X00_TO_0X1F_NUM);
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_PWM_DUTY_INIT, &u32Data);
		priv->stiw7039.u16iw7039_pwm_duty_init = (uint16_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_ISET_MBR_OFF, &u32Data);
		priv->stiw7039.u16IW7039_ISET_MBR_OFF = (uint16_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_ISET_MBR_ON, &u32Data);
		priv->stiw7039.u16IW7039_ISET_MBR_ON = (uint16_t)u32Data;
		ofnode_read_u32_array(info_node, LDM_DEVICE_IW7039_DT, priv->stiw7039.u32IW7039_DT, reg_0x40_to_0x5F_NUM);
		ofnode_read_u32_array(info_node, LDM_IW7039_P_P_MBR_OFF, priv->stiw7039.u32iw7039_p_p_mbr_off, LD_MAX_HEIGHT);
		ofnode_read_u32_array(info_node, LDM_IW7039_P_P_MBR_ON, priv->stiw7039.u32iw7039_p_p_mbr_on, LD_MAX_HEIGHT);
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_ISET_HIGH_LIMIT, &u32Data);
		priv->stiw7039.u16iw7039_iset_high_limit = (uint16_t)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_ISET_LOW_LIMIT, &u32Data);
		priv->stiw7039.u16iw7039_iset_low_limit = (uint16_t)u32Data;

		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_NEED_CTRL_EN_FLAG, &u32Data);
		priv->stiw7039.b_iw7039_need_en_pad_flag = (bool)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_EN_PAD, &u32Data);
		priv->stiw7039.u8iw7039_en_pad_num = (u8)u32Data;

		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_CHECK_SUM_FLAG, &u32Data);
		priv->stiw7039.u8iw7039_check_sum_flag = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_PACKET_NUM, &u32Data);
		priv->stiw7039.u8iw39_packnum = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7039_DEV_NUM, &u32Data);
		priv->stiw7039.u8iw7039_dev_num = (u8)u32Data;
		ofnode_read_u32_array(info_node, IW39_EVEPACK, priv->stiw7039.u32iw39_evepack, priv->stiw7039.u8iw39_packnum);
		for (i = 0; i < priv->stiw7039.u8iw39_packnum; i++)
			u32alldatanum += priv->stiw7039.u32iw39_evepack[i];
		ofnode_read_u32_array(info_node, IW7039_ALL_PACKET_DATA, priv->stiw7039.u8iw7039_allpackdata, u32alldatanum);
	}

	info_node = dev_read_subnode(dev, LDM_DEVICE_MBI6353_INFO);
	if(!ofnode_valid(info_node))
	{
		debug("Read subnode (%s) FAIL\n", LDM_DEVICE_MBI6353_INFO);
	}
	else
	{
		ofnode_read_u32_array(info_node, LDM_DEVICE_MBI6353_MASK,
			priv->st_ldm_Device_MBI6353.u32MBI6353_Mask, MBI6353_MASK_NUM * LDM_NUM_2);
		ofnode_read_u32_array(info_node, LDM_DEVICE_MBI6353_CFG,
			priv->st_ldm_Device_MBI6353.u32MBI6353_Config, MBI6353_CFG_NUM * LDM_NUM_2);
		ofnode_read_u32_array(info_node, LDM_DEVICE_MBI6353_CFG_1,
			priv->st_ldm_Device_MBI6353.u32MBI6353_Config_1, MBI6353_CFG_NUM * LDM_NUM_2);
		ofnode_read_u32(info_node, LDM_DEVICE_MBI6353_PWM_Duty_Init, &u32Data);
		priv->st_ldm_Device_MBI6353.u16MBI6353_PWM_Duty_Init = (uint16_t)u32Data;
	}

	info_node = dev_read_subnode(dev, LDM_DEVICE_IW7027_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_DEVICE_IW7027_INFO);
	} else {
		ofnode_read_u32_array(info_node, IW7027_00_TO_1F, priv->stiw7027.au32_iw7027_0x00_to_0x1F, r0x00_to_0x1F_num);
		ofnode_read_u32_array(info_node, IW7027_20_TO_39, priv->stiw7027.au32_iw7027_0x20_to_0x39, r0x20_to_0x39_num);
		ofnode_read_u32(info_node, IW7027_INIT_HT, &u32Data);
		priv->stiw7027.u16_iw7027_HT = (u16)u32Data;
		ofnode_read_u32(info_node, IW7027_NEED_POWER_PIN_CTRL, &u32Data);
		priv->stiw7027.b_iw7027_need_power_pad_flag = (bool)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7027_EN_PAD, &u32Data);
		priv->stiw7027.u8_iw7027_power_pad_num = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7027_PACKET_NUM, &u32Data);
		priv->stiw7027.u8iw27_packnum = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7027_DEV_NUM, &u32Data);
		priv->stiw7027.u8iw7027_dev_num = (u8)u32Data;
		ofnode_read_u32_array(info_node, IW27_EVEPACK, priv->stiw7027.au32iw27_evepack, priv->stiw7027.u8iw27_packnum);
		for (i = 0; i < priv->stiw7027.u8iw27_packnum; i++)
			u32alldatanum += priv->stiw7027.au32iw27_evepack[i];
		ofnode_read_u32_array(info_node, IW7027_ALL_PACKET_DATA, priv->stiw7027.au8iw7027_allpackdata, u32alldatanum);
	}

	info_node = dev_read_subnode(dev, LDM_DEVICE_IW7038_INFO);
	if (!ofnode_valid(info_node)) {
		debug("Read subnode (%s) FAIL\n", LDM_DEVICE_IW7038_INFO);
	} else {
		ofnode_read_u32(info_node, LDM_DEVICE_IW7038_CHECK_SUM_FLAG, &u32Data);
		priv->st_iw7038.u8iw7038_check_sum_flag = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7038_PACKET_NUM, &u32Data);
		priv->st_iw7038.u8packnum = (u8)u32Data;
		ofnode_read_u32(info_node, LDM_DEVICE_IW7038_DEV_NUM, &u32Data);
		priv->st_iw7038.u8iw7038_dev_num = (u8)u32Data;
		ofnode_read_u32_array(info_node, LEN_EVE_PACK, priv->st_iw7038.u32len_eve_pack, priv->st_iw7038.u8packnum);
		for (i = 0; i < priv->st_iw7038.u8packnum; i++)
			u32alldatanum += priv->st_iw7038.u32len_eve_pack[i];
		ofnode_read_u32_array(info_node, IW7038_ALL_PACKET_DATA, priv->st_iw7038.u8allpackdata, u32alldatanum);
	}

	return ret;
}

static void _mtk_ldm_LDC_Ctrl(struct udevice *dev)
{
	//int i;
	struct mtk_ldm_priv *priv = dev_get_priv(dev);
	if (priv == NULL) {
		debug("[%s] get device private fail\n", __func__);
		return;
	}
	switch (priv->u8ldm_ver) {
		case E_M_LDM_VERSION_1:
		case E_M_LDM_VERSION_2:
		case E_M_LDM_VERSION_3:
		{
			if (priv->u32ldm_support == E_LDM_SUPPORT_TRUNK_PQU || priv->u32ldm_support == E_LDM_SUPPORT_TRUNK_FRC) {
				W2BYTEMSK(REG_0004_LDC_BKA3EF, 1, REG_0004_LDC_BKA3EF_REG_LDC_BYPASS);
				W2BYTEMSK(REG_0004_LDC_BKA3EF, 0, REG_0004_LDC_BKA3EF_REG_LDC_PATH_SEL);
				W2BYTEMSK(REG_00DC_LD1_BKA417, 0, REG_00DC_LD1_BKA417_REG_LD_EMPTY_EN);
			} else if (priv->u32ldm_support == E_LDM_SUPPORT_CUS) {
				if (priv->bldm_cus_pathu_boot) {
					W2BYTEMSK(REG_0004_LDC_BKA3EF, 0, REG_0004_LDC_BKA3EF_REG_LDC_BYPASS);
					W2BYTEMSK(REG_0004_LDC_BKA3EF, 1, REG_0004_LDC_BKA3EF_REG_LDC_PATH_SEL);
					W2BYTEMSK(REG_0020_LDC_BKA3EF, 0, REG_0020_LDC_BKA3EF_REG_LDC_XIU2AHB_SEL0);
					W2BYTEMSK(REG_0020_LDC_BKA3EF, 0, REG_0020_LDC_BKA3EF_REG_LDC_XIU2AHB_SEL1);
					W2BYTEMSK(REG_00DC_LD1_BKA417, 1, REG_00DC_LD1_BKA417_REG_LD_EMPTY_EN);
				} else {
					W2BYTEMSK(REG_0004_LDC_BKA3EF, 1, REG_0004_LDC_BKA3EF_REG_LDC_BYPASS);
					W2BYTEMSK(REG_0004_LDC_BKA3EF, 0, REG_0004_LDC_BKA3EF_REG_LDC_PATH_SEL);
					W2BYTEMSK(REG_00DC_LD1_BKA417, 0, REG_00DC_LD1_BKA417_REG_LD_EMPTY_EN);
				}
			} else {
				W2BYTEMSK(REG_0004_LDC_BKA3EF, 1, REG_0004_LDC_BKA3EF_REG_LDC_BYPASS);
				W2BYTEMSK(REG_0004_LDC_BKA3EF, 0, REG_0004_LDC_BKA3EF_REG_LDC_PATH_SEL);
				W2BYTEMSK(REG_00DC_LD1_BKA417, 1, REG_00DC_LD1_BKA417_REG_LD_EMPTY_EN);
			}
			break;
		}
		case E_M_LDM_VERSION_4:
		case E_M_LDM_VERSION_5:
		default:
		{
			if (priv->u32ldm_support == E_LDM_SUPPORT_TRUNK_PQU || priv->u32ldm_support == E_LDM_SUPPORT_TRUNK_R2) {
				W2BYTEMSK(REG_00DC_LD1_BKA417, 0, REG_00DC_LD1_BKA417_REG_LD_EMPTY_EN);
			} else {
				W2BYTEMSK(REG_00DC_LD1_BKA417, 1, REG_00DC_LD1_BKA417_REG_LD_EMPTY_EN);
			}
			break;
		}
	}
}

static int mtk_ldm_enable_output(struct udevice *dev, int panel_bpp,
				 const struct display_timing *timing)
{
	struct mtk_ldm_priv *priv = dev_get_priv(dev);

	if (!timing || !dev) {
		UBOOT_ERROR("[%s] param FAIL\n", __func__);
		return -EPERM;
	}
	if (!priv) {
			UBOOT_ERROR("[%s] get device private fail\n", __func__);
			return -EPERM;
	}
	UBOOT_DEBUG("[%s] timing->flags = 0x%x\n", __func__, timing->flags);
	if ((timing->flags & DISPLAY_FLAGS_DE_HIGH) ||
	    (timing->flags & DISPLAY_FLAGS_VSYNC_HIGH)) {
		if (priv->bled_mspi_en)
			_mtk_ldm_leddevice_init(priv);
	}

	if ((timing->flags & DISPLAY_FLAGS_DE_LOW) ||
	    (timing->flags & DISPLAY_FLAGS_VSYNC_LOW)) {
		// todo
	}

	return 0;
}

static void _mtk_ldm_select_pins_to_func(struct udevice *dev)
{
	struct mtk_ldm_priv *priv = dev_get_priv(dev);
	if (priv == NULL) {
		debug("[%s] get device private fail\n", __func__);
		return;
	}
#if CONFIG_PINCTRL
	if (priv->u32ldm_support == E_LDM_SUPPORT_CUS)
		pinctrl_select_state(dev, "fun_ldc_in_out_pmux");
#endif
	UBOOT_DEBUG("[%s] done\n", __func__);
}

static void _vsync_like_gpio_probe(struct udevice *dev)
{
	struct mtk_ldm_priv *priv = dev_get_priv(dev);
	int ret;

	ret = gpio_request_by_name(dev, "gpios", 0, &priv->vsync_like_gpio, GPIOD_IS_OUT);

	if (ret)
		UBOOT_DEBUG("[%s] no gpios\n", __func__);
}

static int mtk_ldm_probe(struct udevice *dev)
{
	//read dts table and init clk
	struct mtk_ldm_priv *priv = dev_get_priv(dev);
  	if (priv == NULL) {
		debug("[%s] get device private fail\n", __func__);
		return -ENXIO;
	}
	_mtk_ldm_LDC_Ctrl(dev);
	_mtk_ldm_select_pins_to_func(dev);
	if (priv->bChange_Vsynclike_PadmuxMode)
		_vsync_like_gpio_probe(dev);
	UBOOT_DEBUG("[%s] done\n", __func__);
	return 0;
}

/*
 *static const struct panel_ops mtk_panel_ops = {
 *	.enable_backlight	= mtk_panel_enable_backlight,
 *	.set_backlight		= mtk_panel_set_backlight,
 *};
 */
static const struct udevice_id ldm_ids[] = {
	{ .compatible = "mediatek,mediatek-ldm" },
	{ },
};

static const struct dm_display_ops mtk_ldm_ops = {
	.enable			= mtk_ldm_enable_output,
};

U_BOOT_DRIVER(mediatek_ldm) = {
	.name = "mtk_ldm",
	.id = UCLASS_DISPLAY,
	.of_match = ldm_ids,
	.ops = &mtk_ldm_ops,
	.ofdata_to_platdata = mtk_ldm_ofdata_to_platdata,
	.probe = mtk_ldm_probe,
	.priv_auto_alloc_size = sizeof(struct mtk_ldm_priv),
};



